(function($) {
    'use strict';

    /**
     * Initialize admin property google map.
     */
    function esInitMetaboxMap() {
        var $lon = $('#es-longitude-input').val();
        var $lat = $('#es-latitude-input').val();
        var map = document.getElementById('es-property-map');

        if ($lon && $lat && map && EsGoogleMap != 'undefined') {
            var instance = new EsGoogleMap(map, $lon, $lat).init();
            instance.setMarker();

            map.classList.add('es-map-border');

            instance.getGeocoderInfo($lat, $lon, function(results, status) {
                if (status == 'OK') {
                    $('#es-address-input').val(results[0].formatted_address);
                    $('#es-address_components-input').val(JSON.stringify(results[0].address_components));
                }
            });
        } else {
            if (map) {
                map.classList.remove('es-map-border');
            }
        }
    }

    $.moveColumn = function (table, from, to) {
        var rows = jQuery('tr', table);
        var cols;
        rows.each(function() {
            cols = jQuery(this).children('th, td');
            cols.eq(from).detach().insertBefore(cols.eq(to));
        });
    };

    /**
     * Change display css param in $searchPagesContainer using $el value.
     *
     * @param $el
     * @param $searchPagesContainer
     */
    function esSwitchSearchPagesContainerState($el, $searchPagesContainer) {
        var val = $el.val();

        if (val == 'all') {
            $searchPagesContainer.removeClass('show');
        } else {
            $searchPagesContainer.addClass('show');
        }
    }

    $(function() {
        var $unitsTooltipFields = $( '.es-field-area-unit input' );
        var map = document.getElementById('es-property-map');
        var addressInput = document.getElementById('es-address-input');
        var $propMetaBoxDataTabs = $('.property-data-tabs');
        var $dataManagerItems = $('.es-data-manager-item');
        var $themesDashboardSlider = $('.es-themes-slider');
        var $styledCheckboxes = $('.es-switch-input');
        var $layoutRadio = $('.js-es-layout-checkbox');
        var $scrollList = $('.es-scroll-list');
        var $confirmPopup = $('.es-confirm-popup');
        var $messagePopup = $('.es-message-popup');

        $( '.es-field-date input' ).datetimepicker( {
            format: Estatik.settings.dateFormat,
            timepicker: false
        } );

        $( '.nav-tab-wrapper .nav-tab-menu li a' ).click( function() {
            var $wrapper = $( this ).closest( '.nav-tab-wrapper' );
            var tabContainer = $( this ).attr( 'href' );

            if ( tabContainer.length > 1 ) {

                $wrapper.find( 'li' ).removeClass( 'active' );
                $( this ).closest( 'li' ).addClass( 'active' );

                $wrapper.find( '.es-tab' ).hide();
                $( tabContainer ).show();

                var yScroll = document.body.scrollTop;

                if ( history.pushState ) {
                    history.pushState( null, null, tabContainer );
                }
                else {
                    location.hash = tabContainer;
                }

                document.body.scrollTop = yScroll;

                esInitMetaboxMap();
            }

            return false;
        } );


        if ( window.location.hash ) {
            if ( $( window.location.hash ).hasClass( 'es-tab' ) ) {
                $( 'a[href=' + window.location.hash + ']' ).trigger( 'click' );
            }
        }

        $( '.nav-tab-wrapper' ).each( function() {
            if ( ! $( this ).find( '.nav-tab-menu li.active' ).length ) {
                $( this ).find( '.nav-tab-menu li:first-child a' ).trigger( 'click' );
            }

            window.scrollTo(0, 0);         // execute it straight away
            setTimeout(function() {
                window.scrollTo(0, 0);     // run it a bit later also for browser compatibility
            }, 1);
        } );

        $( '.es-field-datetime-local input' ).datetimepicker( {
            format: Estatik.settings.dateTimeFormat
        } );

        $( '.post-type-es_subscription .page-title-action, .post-type-properties .page-title-action' ).removeClass( 'page-title-action' ).addClass( 'es-button' ).addClass( 'es-button-green' ).addClass( 'es-button-add' );
        $( '.post-type-es_order .page-title-action' ).remove();

        // Append estatik logo to the page.
        if ( typeof Estatik.html != 'undefined' ) {
            if ( typeof Estatik.html.logo != 'undefined' )
                $('.post-type-properties .wrap h1, .post-type-es_subscription .wrap h1, .post-type-es_order .wrap h1').before(Estatik.html.logo);
        }
        var $ratingField = $('.js-rating-input');
        var $ratingEl = $(".es-rating");

        $ratingEl.starRating({
            useFullStars: true,
            disableAfterRate: false,
            starSize: 20,
            callback: function(currentRating, $el){
                $('.js-rating-input').val(currentRating);
            }
        });

        $('.es-subscription__input__enabled').on('change', function() {

            var $this = $( this );
            var $container = $( '.es-subscription__settings' );
             if ( $this.is(':checked') ) {
                 $container.show();
             } else {
                 $container.hide();
             }
        } ).trigger('change');

        $( '.js-es-remove-attachment' ).on( 'click', function() {
            var $el = $( this );
            $el.closest( '.es-field__content' ).find( 'input' ).val( '' );
            $el.closest( '.es-manage-attachments' ).remove();

            return false;

        } );

        // Flag for 782 window width.
        var flag782 = true;

        $( window ).resize(function() {
            if ( $( window ).width() <= 782 ) {
                if (flag782) {
                    $.moveColumn($('.post-type-es_subscription .wp-list-table, .post-type-es_order .wp-list-table'), 1, 0);
                    flag782 = false;
                }
            }
        });

        var $orderTitleLink = $('.post-type-es_order .column-title a');
        $orderTitleLink.attr('href', '');
        $orderTitleLink.click(function () {
            return false;
        });

        $( window ).trigger( 'resize' );

        if ( $ratingField.length && $ratingField.val() ) {
            $ratingEl.starRating('setRating', parseFloat($ratingField.val()))
        }

        if ($confirmPopup.length) {
            $confirmPopup.esPopup({confirmPopup: true});
        }

        if ($messagePopup.length) {
            $messagePopup.esPopup();
        }

        // Dashboard custom scroll for list blocks.
        if ($scrollList.length) {
            $scrollList.mCustomScrollbar();
        }

        if ($layoutRadio.length) {
            $layoutRadio.change(function() {
                var $el = $(this);
                var $wrap = $el.closest('.es-layout-wrap');

                $wrap.find('.es-sprite').removeClass('es-sprite-active');
                $el.closest('.es-layout-box').find('.es-sprite').addClass('es-sprite-active');
            });
        }

        if ($styledCheckboxes.length) {
            $styledCheckboxes.esCheckbox({
                labelTrue: Estatik.tr.yes,
                labelFalse: Estatik.tr.no
            });
        }

        if ($themesDashboardSlider.length) {
            $themesDashboardSlider.slick({
                arrows: true,
                slidesToShow: 5,
                centerMode: true,
                centerPadding: '10px',
                responsive: [
                    {
                        breakpoint: 1550,
                        settings: {
                            slidesToShow: 4,
                            infinite: true,
                            dots: true
                        }
                    },
                    {
                        breakpoint: 1290,
                        settings: {
                            slidesToShow: 3,
                            infinite: true,
                            dots: true
                        }
                    },
                    {
                        breakpoint: 1024,
                        settings: {
                            slidesToShow: 2,
                            infinite: true,
                            dots: true
                        }
                    },
                    {
                        breakpoint: 600,
                        settings: {
                            slidesToShow: 1
                        }
                    }
                ]
            });
        }

        if ($dataManagerItems.length) {
            $dataManagerItems.each(function() {
                $(this).dataManagerItem();
            })
        }

        var $searchWidgetFieldsList = $('.es-search-widget-fields');

        var $showPagesSearchField = $('.js-show-search-pages');
        var $searchPagesContainer = $('.js-search-pages');

        $(document).ajaxSuccess(function() {
            var $searchWidgetFieldsList = $('.es-search-widget-fields');
            var $showPagesSearchField = $('.js-show-search-pages');
            // var $searchPagesContainer = $('.js-search-pages');

            if ($searchWidgetFieldsList.length) {
                $searchWidgetFieldsList.sortable();
            }

            if ($showPagesSearchField.length) {
                $showPagesSearchField.each(function() {
                    var $el = $(this);
                    esSwitchSearchPagesContainerState($el, $el.closest('.es-widget__wrap').find('.js-search-pages'));
                });
            }
        });

        $(document).on('change', '.js-show-search-pages', function() {
            var $el = $(this);
            esSwitchSearchPagesContainerState($(this), $el.closest('.es-widget__wrap').find('.js-search-pages'));
        });

        if ($showPagesSearchField.length) {
            $showPagesSearchField.each(function() {
                var $el = $(this);
                esSwitchSearchPagesContainerState($el, $el.closest('.es-widget__wrap').find('.js-search-pages'));
            });
        }

        $(document).on( 'change', '.js-es-field-select', function() {
            var $el = $( this );
            var $option = $el.find( 'option:selected' );
            var value = $el.val();
            var $searchWidgetFieldsList = $( this ).closest( '.es-search-widget__wrap' ).find( '.es-search-widget-fields' );
            var name = $(this).closest( '.es-search-widget__wrap' ).find( '.es-fields-name' ).attr( 'name' );

            if ( value ) {
                if ( ! $searchWidgetFieldsList.find('li[data-field-name=' + value + ']').length ) {
                    $searchWidgetFieldsList.append( '' +
                        '<li data-field-name="' + value + '">' + $option.html() + '<a href="#" class="es-remove-field">×</a>' +
                        '<input type="hidden" name="' + name + '" value="' + value + '"></li>' );
                }
            }
            $el.val( '' );
        });

        if ( $searchWidgetFieldsList.length ) {
            $searchWidgetFieldsList.sortable();
        }

        $( document ).on( 'click', '.es-remove-field', function() {
            $( this ).closest( 'li' ).remove();


            return false;
        } );

        // Initialize property data meta box tabs.
        if ( $propMetaBoxDataTabs.length ) {

            // Initialize metabox map.
            esInitMetaboxMap();
        }

        $('.js-es-add-custom').click( function() {
            var value = $('[name=es-custom-field]').val();

            if (value) {
                var content = '<div class="es-field es-field-custom">' +
                    '<div class="es-field__label">' + value + '</div>' +
                    '<div class="es-field__content">' +
                    '<input type="text" name="es_custom_value[]"/>' +
                    '<input type="hidden" name="es_custom_key[]" value="' + value + '"/>' +
                    '<a href="#" class="js-es-remove-custom"><span class="es-sprite es-sprite-close"></span></a>' +
                    '</div>' +
                    '</div>';

                $('.es-fields-custom__wrap').append( content ).sortable();
            }

            $('[name=es-custom-field]').val('');

            return false;
        } );

        $( '.es-fields-custom__wrap' ).sortable();

        $(document).on('click', '.js-es-remove-custom', function() {
            $(this).closest('.property-data-field, .es-field-custom').remove();

            return false;
        });

        // Reinit metabox map after change coordinates.
        $('#es-latitude-input, #es-longitude-input').change(esInitMetaboxMap);

        if (addressInput) {
            var autocomplete = new google.maps.places.Autocomplete(addressInput);

            // Reinit map when address changed.
            google.maps.event.addListener(autocomplete, 'place_changed', function() {
                var result = autocomplete.getPlace();

                if (result != 'undefined') {
                    var location = result.geometry.location;
                    $('#es-latitude-input').val(location.lat());
                    $('#es-longitude-input').val(location.lng());

                    esInitMetaboxMap();
                }
            })
        }

        var $mapTaxonomyField = $('.js-es-map-taxonomy-field');
        var $mapTermsField = $('.js-es-map-terms-field');
        var $mapSettingWrapper = $('.es-markers-list');
        var $mapItems = $('.es-markers-list__item');

        $mapItems.each(function() {
            var $el = $(this);
            var color = $el.find('.js-es-item-color').val();

           $el.find('.es-marker-icon').css({color: color});
        });

        $mapTaxonomyField.change(function() {
            var $el = $(this);
            var action = $mapTaxonomyField.data('action');

            $mapTermsField.find('[value!=""]').remove();

            if (action && $el.val()) {
                $.get( ajaxurl, {taxonomy: $el.val(), action: action}, function(response) {
                    response = JSON.parse(response);
                    if (response.items) {
                        for (var i in response.items) {
                            $mapTermsField.append('<option value="' + response.items[i].term_id + '">' + response.items[i].name + '</option>');
                            $mapTermsField.removeProp('disabled');
                        }
                    } else {
                        $mapTermsField.prop('disabled', 'disabled');
                    }
                } );
            }
        });

        $mapTermsField.change(function() {
            var $el = $(this);

            if ($el.val() && Estatik.settings.markers) {
                var $row = $('.es-markers-list__item-' + $el.val());
                if ($row.length) {
                    $row.animate({opacity: 0.1}, 300, function() {
                        $row.animate({opacity: 1}, 300)
                    });
                } else {
                    var template = '<li class="es-markers-list__item es-markers-list__item-' + $el.val() + '">' +
                        '<div class="item-label">' + $el.find(':checked').html() + '</div>';

                    template += '<div class="item-content">' +
                        '<div class="content-markers">';

                    for(var i in Estatik.settings.markers) {
                        var marker = Estatik.settings.markers[i];
                        marker = $(marker).css('color', Estatik.settings.default_marker.color);
                        template += '<div class="marker">' + marker[0].outerHTML +
                            '<input class="js-item-marker-field" type="radio" name="es_settings[map][' + $el.val() + '][icon]" value="' + i + '">';

                        template += '</div>'
                    }

                    template += '</div>';

                    template += '<div class="content-color"><input type="color" class="js-es-item-color" name="es_settings[map][' + $el.val() + '][color]" ' +
                        'value="' + Estatik.settings.default_marker.color + '"/></div>';

                    template += '</div>';

                    template += '<div class="item-remove">' +
                        '<a href="" class="es-item-remove"><span class="es-sprite es-sprite-close"></span></a>' +
                        '</div>';

                    template += '</li>';

                    $mapSettingWrapper.append(template);
                }
            }
        });

        $($mapSettingWrapper).on('click', '.es-item-remove', function() {
            var $el = $(this);
            $el.closest('.es-markers-list__item').animate({opacity: 0}, 500, function() {
                $(this).remove();
            });

            return false;
        });

        $($mapSettingWrapper).on('change', '.js-item-marker-field', function() {
            var $el = $(this);
            $el.closest('.es-markers-list__item').find('.marker').removeClass('marker__active');
            $el.closest('.marker').addClass('marker__active')
        });

        $($mapSettingWrapper).on('input', '.js-es-item-color', function() {
            var $el = $(this);

            $el.closest('.es-markers-list__item').find('.es-marker-icon').each(function() {
                $(this).css('color', $el.val());
            })
        });

        var file_frame;

        $(document).on('click', 'a.gallery-image-add', function(e) {

            e.preventDefault();

            if (file_frame) file_frame.close();

            file_frame = wp.media.frames.file_frame = wp.media({
                title: $(this).data('uploader-title'),
                button: {
                    text: $(this).data('uploader-button-text')
                },
                multiple: false
            });

            file_frame.on('select', function() {
                var selection = file_frame.state().get('selection');

                selection.map(function(attachment, i) {
                    var url;
                    attachment = file_frame.state().get('selection').first().toJSON();

                    if (attachment.sizes['thumbnail'].url) {
                        url = attachment.sizes['thumbnail'].url;
                    } else if (attachment.sizes['thumbnail'].url) {
                        url = attachment.sizes['thumbnail'].url;
                    } else {
                        url = attachment.icon;
                    }

                    if ($('#es-media-list').hasClass('es-attachment-logo')) {
                        $('#es-media-list').html("<li><input type='hidden' name='es_settings[logo_attachment_id]' value='" + attachment.id + "'>" +
                            "<div class='image-preview-wrap'><a href='#' class='remove-image'><i class='fa fa-times-circle' aria-hidden='true'></i></a>" +
                            "<img src='" + url + "' class='image-preview'></div></li>");
                    } else {
                        $('#es-media-list').html("<li><input type='hidden' name='es_agent[profile_attachment_id]' value='" + attachment.id + "'>" +
                            "<div class='image-preview-wrap'><a href='#' class='remove-image'><i class='fa fa-times-circle' aria-hidden='true'></i></a>" +
                            "<img src='" + url + "' class='image-preview'></div></li>");
                    }
                });
            });

            file_frame.open();
        });

        var $parent_menu_item = $( '#toplevel_page_es_dashboard' );

        if ( $parent_menu_item.find( '.current' ).length ) {
            $parent_menu_item.addClass('wp-has-current-submenu');
            $parent_menu_item.children().addClass('wp-has-current-submenu');

            $( '#menu-users' ).addClass('wp-not-current-submenu')
                .removeClass('wp-has-current-submenu')
                .find('a')
                .removeClass('wp-has-current-submenu');
        }

        $unitsTooltipFields.on('change click focus', function () {
            var $content = $(this).closest('.es-field__content');
            var val = $content.find('input').val();
            var unit = $content.find('select').val();

            $('.tooltipstered').tooltipster({}).tooltipster('close');

            $.post(ajaxurl, {val: val, unit: unit, action: 'es_calculate_units'}, function(response) {
                response = response || {};

                if (response.status) {
                    $content.find('select').tooltipster({
                        contentAsHTML: true,
                        theme: 'tooltipster-borderless',
                        side: ['right'],
                        debug: false
                    }).tooltipster('content', response.content).tooltipster('open');
                }
            }, 'json');
        }).mouseleave(function() {
            $('.tooltipstered').tooltipster({}).tooltipster('close');
        });

    });
})(jQuery);
