( function( $ ) {
    'use strict';

    /**
     * Update fields order.
     * @param $section string Section machine name.
     */
    function updateSectionFieldsOrder( $section ) {
        var order = [];

        $section.find( 'li' ).each( function() {
            order.push( $( this ).data('machine-name') );
        } );

        /**
         * Ajax data for sending.
         * @type {{action: string, section, order: Array}}
         */
        var requestData = {
            action: 'es_fbuilder_change_fields_order',
            section: $section.data( 'machine-name' ),
            order: order
        };

        $.post( ajaxurl, requestData, function( response ) {
            console.log( response );
        } );
    }

    /**
     * Update fields order.
     * @param $list string Section machine name.
     */
    function updateSectionsOrder( $list ) {
        var order = [];

        $list.find( 'li' ).each( function() {
            order.push( $( this ).data('machine-name') );
        } );

        /**
         * Ajax data for sending.
         * @type {{action: string, section, order: Array}}
         */
        var requestData = {
            action: 'es_fbuilder_change_sections_order',
            order: order
        };

        $.post( ajaxurl, requestData, function( response ) {
            console.log( response );
        } );
    }

    function initDraggableElements() {
        $( '.es-list__sortable' ).sortable( {
            items: "li:not([data-machine-name=es-info])",
            connectWith: '.es-list__sortable',
            cursorAt: { top: 0, left: 0 },
            cancel: "li span",
            update: function( event, ui ) {
                var $el = $( this );

                if ( $el.hasClass('js-es-list__sections') ) {
                    updateSectionsOrder( $el );
                } else {
                    updateSectionFieldsOrder( $el.closest( 'li' ) );
                }
            }
        } );

        $( '.js-es-sortable' ).sortable();
    }

    $( function() {

        var $tooltipsterRetsAvailable = $( '.js-es__available-tooltipster--rets' );

        var toolTipsterConfig = {
            contentAsHTML: true,
            theme: 'tooltipster-borderless',
            side: ['right'],
            debug: false,
            // content: Estatik.tr.searchAvailable,
            interactive: true
        };

        var cloneyaConfig = {
            cloneThis: '.es-field',
            cloneButton	: '.clone',
            deleteButton: '.delete'
        };

        // Initialize sortable lists.
        initDraggableElements();

        $tooltipsterRetsAvailable.tooltipster( toolTipsterConfig ).tooltipster( 'content', Estatik.tr.retsAvailable );

        $( '.es-clone__wrap' ).cloneya( cloneyaConfig );

        $( '.js-es-load-options-fields' ).change( function() {
            var type = $( this ).val();

            if ( type ) {
                $.post( ajaxurl, { action: 'es_fbuilder_load_field_options', type: type }, function( response ) {
                    $( '.js-es-fbuilder__field-options' ).html( response );
                    $( '.es-clone__wrap' ).cloneya( cloneyaConfig );

                    $('.es-switch-input', $( '.js-es-fbuilder__field-options' )).esCheckbox({
                        labelTrue: Estatik.tr.yes,
                        labelFalse: Estatik.tr.no
                    });

                    // Re-Initialize sortable lists.
                    initDraggableElements();
                } );
            }
        } );
    } );
} )( jQuery );
