<?php

/**
 * Class Es_Data_Manager_Label_Item
 */
class Es_Data_Manager_Label_Item extends Es_Data_Manager_Term_Item
{
    /**
     * Es_Data_Manager_Label_Item constructor.
     * @param array $options
     */
    public function __construct(array $options = array() )
    {
        $this->_template_path = '/admin/templates/data-manager/label-item.php';
        parent::__construct( 'es_labels', $options );
    }

    /**
     * Overridden save term method.
     *
     * @return void.
     */
    public static function save()
    {
        // Nonce field name.
        $nonce = 'es_add_data_manager_label';

        // If valid ajax request.
        if ( self::is_valid_ajax() && ! empty( $_REQUEST[ $nonce ] ) && wp_verify_nonce( $_REQUEST[ $nonce ], $nonce ) ) {
            // Insert term to the taxonomy.
            $result = wp_insert_term( $_POST['item_name'], $_POST['taxonomy'] );

            // If term successfully added.
            if ( is_array( $result ) ) {
                update_term_meta( $result['term_id'], 'es_color', es_get_default_label_color() );
                $response = array( 'message' => __( 'Item is successfully created.', 'es-plugin' ),
                    'status' => 'success',
                    'item' => $result['term_taxonomy_id'],
                    'content' => self::get_item_markup( $result['term_id'] )
                );

                // If something was wrong and we have an wp error.
            } else if ( $result instanceof WP_Error ) {
                $response = array( 'message' => $result->get_error_message(), 'status' => 'error' );

                // Another crash error.
            } else {
                $response = array( 'message' => __( 'Item doesn\'t create. Please contact support.', 'es-plugin' ), 'status' => 'warning' );
            }
            // If request isn't valid.
        } else {
            $response = array( 'message' => __( 'Invalid ajax request.', 'es-plugin' ), 'status' => 'error' );
        }

        wp_die( json_encode( $response ) );
    }

    /**
     * @param $term_id
     * @return bool|string
     */
    private static function get_item_markup($term_id) {
        $term = get_term($term_id);
        $colors = Es_Property::get_label_colors();
        $term_color = get_term_meta( $term_id, 'es_color', true );

        if ( ! $term ) return false;

        ob_start(); ?>

        <li><label><?php echo $term->name; ?></label>

        <?php if ( ! empty( $colors ) ) : ?>
            <div class="es-data-manager-colors-wrap">
                <?php foreach ( $colors as $key => $color ) : ?>
                    <input type="radio"
                           <?php checked( $term_color, $color ); ?>
                           id="es_label_color-<?php echo $term->term_id . $key; ?>"
                           name="es_label_color[<?php echo $term->term_id; ?>]"
                           data-action="es_ajax_data_manager_label_color"
                           class="es-radio-color es-radio-color-<?php echo str_replace('#', '', $color); ?>"
                           value="<?php echo $color; ?>"
                           data-id="<?php echo $term->term_id; ?>"/>
                    <label for="es_label_color-<?php echo $term->term_id . $key; ?>"></label>
                <?php endforeach; ?>
            </div>
        <?php endif; ?>

        <a href="#" class="es-item-remove js-item-remove"
           data-id="<?php echo $term->term_id; ?>"
           data-action="es_ajax_data_manager_remove_term"><span class="es-sprite es-sprite-close"></span></a>
        </li>

        <?php return ob_get_clean();
    }

    /**
     * Set color for label.
     */
    public static function change_color()
    {
        $color = update_term_meta( $_POST['id'], 'es_color', $_POST['color'] );

        if ( ! $color || $color instanceof WP_Error) {
            $response = array( 'message' => __( 'Item doesn\'t updated. Please contact support.', 'es-plugin' ), 'status' => 'error' );
        } else {
            $response = array( 'message' => __( 'Color successfully changed.', 'es-plugin' ), 'status' => 'success' );
        }

        wp_die( json_encode( $response ) );
    }
}
