<?php

/**
 * Class Es_Agent_List_Page
 */
class Es_Agent_List_Page
{
    /**
     * @var int
     */
    static $counter = 0;

    /**
     * @return void
     * @static
     */
    public static function init()
    {
        $_ = new self();

        if ( self::is_agents_list_page() ) {
            $_->filters();
            $_->actions();
        }
    }

    /**
     * @return void
     */
    public function filters()
    {
        add_filter( 'admin_body_class', array( $this, 'add_body_class' ) );
        add_filter( 'user_row_actions', array( $this, 'row_actions' ), 10, 2 );
        add_filter( 'views_users', array( $this, 'views_users' ), 10, 2 );
        add_action('restrict_manage_users', array( $this, 'users_filter' ) );
        // Global js variables.
        add_filter( 'es_global_js_variables', array( $this, 'add_js_variables' ), 10, 1 );
        add_filter( 'pre_get_users', array( $this, 'filter_handler' ) );
    }

    /**
     * @return void
     */
    public function actions()
    {
        add_action( 'manage_users_columns', array( $this, 'add_columns'), 10, 2 );
        // Enqueue styles for our page.
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_styles' ) );
        // Enqueue scripts for our page.
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
        // Add action for render custom columns values.
        add_action( 'manage_users_custom_column' , array( $this, 'add_columns_values' ), 10, 3 );
        add_action( 'init', array( $this, 'agent_actions' ) );
    }

    /**
     * Do actions from agents list page.
     *
     * @return void
     */
    public function agent_actions()
    {
        $action = ! empty( $_GET['es-action'] ) ? $_GET['es-action'] : null;
        $users = ! empty( $_GET['users'] ) ? $_GET['users'] : null;

        if ( ! $users ) return ;

        if ( ! function_exists( 'wp_delete_user' ) ) {
            // Include helper wordpress functions for saving agent.
            include ( ABSPATH . 'wp-admin/includes/user.php' );
        }

        switch ( $action ) {
            case 'delete':
                foreach ( $users as $user_id ) wp_delete_user( $user_id );
                break;

            case 'activate':
                foreach ( $users as $user_id ) {
                    $agent = es_get_agent( $user_id );
                    $agent->change_status( $agent::STATUS_ACTIVE );
                }
                break;

            case 'deactivate':
                foreach ( $users as $user_id ) {
                    $agent = es_get_agent( $user_id );
                    $agent->change_status( $agent::STATUS_DISABLED );
                }
                break;
        }

        wp_redirect( apply_filters( 'es_agent_actions_redirect', es_admin_agents_uri(), $action ) ); exit;
    }

    /**
     * Create handler for post filtering.
     *
     * @param WP_User_Query $query
     */
    public function filter_handler( $query )
    {
        // Get filter data.
        $filter = filter_input( INPUT_GET, 'es_agent_filter', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );

        if ( ! empty( $filter ) ) {
            $meta_query = array();

            if ( ! empty ( $filter['company'] ) ) {
                $meta_query[] = array( 'key' => 'es_agent_company', 'value' => $filter['company'] );
            }

            if ( ! empty( $filter['rating'] ) ) {
                $meta_query[] = array( 'key' => 'es_agent_rating', 'value' => $filter['rating'] );
            }

            if ( ! empty( $filter['name'] ) ) {
                $query->set( 'search', '*' . $filter['name'] . '*' );
                $query->set( 'search_columns', array( 'user_login', 'user_email', 'user_nicename', 'display_name', 'first_name', 'last_name' ) );
                $meta_query[] = array( 'key' => 'es_agent_name', 'value' => $filter['name'] );
            }

            if ( $meta_query ) {
                $query->set( 'meta_query', $meta_query );
            }
        }
    }

    /**
     * Add user filter to the users list page.
     *
     * @param string $position
     */
    public function users_filter( $position )
    {
        $filter_path = apply_filters( 'es_admin_agent_list_filter_path', ES_ADMIN_TEMPLATES . 'agents/filter.php' );

        if ( file_exists( $filter_path ) && $position == 'top' ) {
            $filter = filter_input( INPUT_GET, 'es_agent_filter', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
            $filter_rating = ! empty( $filter['rating'] ) ? $filter['rating'] : null;
            $filter_company = ! empty( $filter['company'] ) ? $filter['company'] : null;

            include ( $filter_path );
        }
    }

    /**
     * Add column values for agents table.
     *
     * @param $output
     * @param $column
     * @param $user_id
     * @return int|string
     */
    public function add_columns_values( $output, $column, $user_id )
    {
        $agent = es_get_agent( $user_id );

        switch ( $column ) {
            case 'id':
                return $user_id;

            case 'company':
                return $agent->company;

            case 'property_qty':
                return $agent->property_qty;

            case 'rating':
                return '<div class="es-list-rating" data-rating="' . $agent->rating . '"></div>';

            case 'status':
                if ( $agent::STATUS_ACTIVE == $agent->status ) {
                    return '<span style="color: green">' . __( 'Active', 'es-plugin' ) . '</span>';
                } else {
                    return '<span style="color: red">' . __( 'Disabled', 'es-plugin' ) . '</span>';
                }

            default:
                return $output;
        }
    }

    /**
     * Register and enqueue scripts for our page.
     *
     * @return void
     */
    public function enqueue_scripts()
    {
        wp_register_script( 'es-admin-agents-list-style', ES_ADMIN_CUSTOM_SCRIPTS_URL . 'agent-list.js', array ( 'jquery', 'es-select2-script' ) );
        wp_enqueue_script( 'es-admin-agents-list-style' );
    }

    /**
     * Add JS variables for this page.
     *
     * @param $data
     * @return mixed
     */
    public function add_js_variables( $data ) {
        if ( current_user_can( 'activate_plugins' ) ) {
            $data['html']['logo'] = es_get_logo();
        }
        $data['settings']['rating'] = Es_Agent::get_rating_list();

        return $data;
    }

    /**
     * @return array
     */
    public function views_users()
    {
        return array();
    }

    /**
     * Customize users table row actions.
     *
     * @param $actions
     * @param $user
     * @return mixed
     */
    public function row_actions( $actions, $user )
    {
        $actions = array();

        // Customize edit link.
        $edit_link = apply_filters( 'es_agent_edit_link', 'admin.php?page=es_agent&id=' . $user->ID, $user );

        $actions['edit'] = '<a href="' . $edit_link . '">
            <i class="fa fa-pencil" aria-hidden="true"></i></a>';

        // Customize remove link.
        $actions['delete'] = "<a class='es-delete-user' href='" . wp_nonce_url( "admin.php?page=es_agent&delete=" . $user->ID, 'es-delete-user', 'es-delete-user' ) . "'>
            <i class='fa fa-trash' aria-hidden='true'></i></a>";

        return $actions;
    }

    /**
     * @param $columns
     * @return array
     */
    public function add_columns($columns)
    {
        unset( $columns['role'], $columns['posts'] );
        // Add user ID column.
        $columns = static::push_column( array( 'id' => __( 'Agent ID', 'es-plugin' ) ),  $columns, 1 );
        $columns = static::push_column( array( 'company' => __( 'Company', 'es-plugin' ) ),  $columns, 5 );
        $columns = static::push_column( array( 'property_qty' => __( 'Properties Q-ty', 'es-plugin' ) ),  $columns, 6 );
        $columns = static::push_column( array( 'rating' => __( 'Rating', 'es-plugin' ) ),  $columns, 7 );
        $columns = static::push_column( array( 'status' => __( 'Status', 'es-plugin' ) ),  $columns, 8 );

        return $columns;
    }

    /**
     * Add custom body class for users agents page.
     *
     * @param array $classes
     * @return string
     */
    public function add_body_class( $classes )
    {
        return $classes . 'es-agent-list-page';
    }

    /**
     * Check if current page is agents list.
     *
     * @return bool
     */
    public static function is_agents_list_page()
    {
        return is_admin() && ! empty( $_GET['role'] ) && $_GET['role'] == Es_Agent::get_role_name();
    }

    /**
     * Register and enqueue styles for our page.
     *
     * @return void
     */
    public function enqueue_styles()
    {
        wp_register_style( 'es-admin-user-list-style', ES_ADMIN_CUSTOM_STYLES_URL . 'agent-list.css' );
        wp_enqueue_style( 'es-admin-user-list-style' );
        wp_enqueue_style( 'jquery-ui' );
    }

    /**
     * Helper function for pushing new columns.
     *
     * @param array $column
     *    name => label array column.
     * @param array &$list
     *    Current array of all columns.
     * @param $index
     *    Index of pushed element.
     *
     * @return array
     *    Array with pushed column by index.
     */
    protected static function push_column( $column, $list, $index )
    {
        return array_merge( array_slice( $list, 0, $index ), $column, array_slice( $list,$index ) );
    }
}
