<?php

/**
 * Class Es_Agent_Profile
 */
class Es_Agent_Profile
{
    /**
     * Initialize agent profile page.
     *
     * @static
     * @return void
     */
    public static function init()
    {
        $_ = new self();
        $_->actions();
    }

    /**
     * Initialize scripts and styles of agent profile page.
     *
     * @return void
     */
    public function actions()
    {
        add_action( 'admin_enqueue_scripts', array( $this , 'enqueue_scripts' ) );
        add_action( 'init', array( $this, 'save' ) );
        add_action( 'init', array( $this, 'remove_user' ) );
    }

    /**
     * Remove user action.
     *
     * @return void
     */
    public function remove_user()
    {
        if ( ! empty( $_GET['es-delete-user'] ) && wp_verify_nonce( $_GET['es-delete-user'], 'es-delete-user' ) ) {
            $id = $_REQUEST['delete'];

            if ( ! function_exists( 'wp_delete_user' ) ) {
                // Include helper wordpress functions for saving agent.
                include ( ABSPATH . 'wp-admin/includes/user.php' );
            }

            wp_delete_user( $id );

            wp_redirect( es_admin_agents_uri() );
            exit;
        }
    }

    /**
     * Add profile agent page scripts.
     *
     * @return void
     */
    public function enqueue_scripts()
    {
        wp_register_script( 'es-admin-agents-list-style', ES_ADMIN_CUSTOM_SCRIPTS_URL . 'agent.js', array ( 'jquery' ) );
        wp_enqueue_script( 'es-admin-agents-list-style' );
    }

    /**
     * Render agent profile page.
     *
     * @return void
     */
    public static function render()
    {
        $template = apply_filters( 'es_agent_profile_template_path', ES_ADMIN_TEMPLATES . 'agents/profile.php' );

        $id = ! empty( $_GET['id'] ) ? $_GET['id'] : 0;

        $agent = new Es_Agent( $id );
        $user = $agent->get_entity();

        include $template;
    }

    /**
     * Save Agent info.
     *
     * @return void
     */
    public function save()
    {
        if ( ! empty( $_POST['es_save_agent'] ) && wp_verify_nonce( $_POST['es_save_agent'], 'es_save_agent' ) ) {

            // Include helper wordpress functions for saving agent.
            if ( ! function_exists( 'edit_user' ) ) {
                include ( ABSPATH . 'wp-admin/includes/user.php' );
            }

            // Set confirmed password variable.
            $_POST['pass2'] = $_POST['pass1'];
            $_POST['nickname'] = $_POST['user_login'];

            // Get user ID.
            $id = ! empty( $_POST['user_id'] ) ? $_POST['user_id'] : 0;

            do_action( 'es_before_save_agent', $id );

            // Save user.
            $id = edit_user( $id );

            if ( ! empty( $id ) && ! $id instanceof WP_Error ) {
                $agent = es_get_agent( $id );
                $post_data = filter_input( INPUT_POST, 'es_agent', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
                $fields = Es_Agent::get_fields();

                foreach ( $fields as $key => $field ) {
                    $value = isset( $post_data[$key] ) ? $post_data[$key] : $agent->{$key};
                    $agent->save_field_value( $key, $value );
                }

                $agent->change_status( $agent::STATUS_ACTIVE );

                if ( ! empty( $post_data['subscription'] ) ) {
                    es_subscription_subscribe_user( $post_data['subscription'], $id );
                }

                $custom_data_key = filter_input( INPUT_POST, 'es_custom_key', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
                $custom_data_value = filter_input( INPUT_POST, 'es_custom_value', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );

                delete_user_meta( $id, 'es_custom_data' );

                if ( ! empty( $custom_data_key ) ) {
                    foreach ( $custom_data_key as $index => $key ) {
                        $value = ! empty( $custom_data_value[ $index ] ) ? $custom_data_value[ $index ] : '';

                        if ( ! empty( $key ) && ! empty( $value ) ) {
                            add_user_meta( $id, 'es_custom_data', array( $key => $value ), false );
                        }
                    }
                }

                if ( empty( $post_data['profile_attachment_id'] ) ) {
                    $agent->delete_field_value( 'profile_attachment_id' );
                }

                $msg = new Es_Messenger( 'es_message' );

                $msg->set_message( __( 'Agent successfully saved', 'es-plugin' ), 'success' );

                do_action( 'es_after_save_agent', $id );

                wp_redirect( apply_filters( 'es_redirect_after_save_agent', 'admin.php?page=es_agent&id=' . $id, $id ) );

                exit;

            } else {
                if  ( ! empty( $id->errors ) ) {
                    $logger = new Es_Messenger( 'es_message' );

                    foreach ( $id->errors as $error_list ) {
                        if ( ! $error_list ) continue;

                        foreach ( $error_list as $error ) {
                            $logger->set_message( $error, 'error' );
                        }
                    }
                } else {
                    $msg = new Es_Messenger( 'es_message' );
                    $msg->set_message( __( 'Agent doesn\'t save', 'es-plugin' ), 'error' );
                }
            }
        }
    }
}
