<?php

/**
 * Class Es_Import_Page
 */
class Es_Import_Page
{
    /**
     * Initialize import page.
     *
     * @return void
     */
    public static function init()
    {
        include( ES_PLUGIN_PATH . '/admin/classes/import/class-import.php' );

        add_action( 'wp_ajax_es_import_properties', array( __CLASS__, 'import_handler' ) );

        if ( ! empty( $_GET['page'] ) && $_GET['page'] == 'es_import' ) {
            add_action( 'admin_enqueue_scripts', array( __CLASS__, 'enqueue_scripts' ) );
        }
    }

    /**
     * Enqueue scripts for import page.
     *
     * @return void
     */
    public static function enqueue_scripts()
    {
        $custom = 'admin/assets/js/custom/';

        wp_register_script( 'es-import-script', ES_PLUGIN_URL . $custom . 'import.js', array (
            'jquery',
            'es-progress-script',
            'jquery-ui-sortable',
            'es-admin-scroll-script'
        ) );

        wp_enqueue_script( 'es-import-script' );

        // Share plugin JS variables for locations script.
        wp_localize_script( 'es-import-script', 'Estatik', Estatik::register_js_variables() );

        $vendor = 'admin/assets/css/vendor/';

        wp_register_style( 'es-scroll-style', ES_PLUGIN_URL . $vendor . 'jquery.mCustomScrollbar.css' );
        wp_enqueue_style( 'es-scroll-style' );
    }

    /**
     * Render import page.
     *
     * @return void
     */
    public static function render()
    {
        $storage = new Es_Session_Storage( 'import' );
        $storage->clear_all();
        $template = apply_filters( 'es_import_template_path', ES_ADMIN_TEMPLATES . 'import/import.php' );
        include_once( $template );
    }

    /**
     * Import handler. Called by ajax using long pooling.
     *
     * @return void
     */
    public static function import_handler()
    {
        // Disable time limit for long-term operations.
        set_time_limit(0);
        ini_set('max_execution_time', 0);

        $import_data = ! empty( $_POST['es_import'] ) ? $_POST['es_import'] : $_POST;

        $import_data = wp_parse_args(
            $import_data,
            array(
                'num' => 0,
                'index' => 0,
                'action' => 'es_import_properties',
                'file' => ! empty( $_FILES['file'] ) ? $_FILES['file'] : null,
                'progress' => 0,
                'skip_first' => 0,
                'imported' => 0,
            )
        );

        if ( ! empty( $import_data['fields'] ) && is_array( $import_data['fields'] ) ) {
            update_option( 'es_import_fields', $import_data['fields'] );
        }

        $import_class_instance = apply_filters( 'es_import_class_name', 'Es_Import' );

        /** @var Es_Import $import */
        $import = new $import_class_instance( $import_data['file'], $import_data );

        if ( ! empty( $_FILES['file'] ) ) {
            // Prepare class for import.
            $import->prepare();

            try {
                $numItems = $import->get_num_items();
            } catch ( Exception $e ) {
                $numItems = 0;
            }

            // If import file is empty.
            if ( ! $numItems ) {
                $import->get_logger()->set_message( __( 'File is empty or incorrect file format.', 'es-plugin' ), 'error' );
                $import_data['done'] = 1;

                // if file is incorrect.
            } else if ( ! $import->get_file() ) {
                $import->get_logger()->set_message( __( 'File isn\'t uploaded. Check permissions for upload directory.', 'es-plugin' ), 'error' );
                $import_data['done'] = 1;

                // If all is ok, set import data.
            } else {
                $import_data['num'] = $import->get_num_items();
                $import_data['file'] = $import->get_file();

                $skip = $import->get_option( 'skip_first', 0 );
                $import_data['index'] = $skip ? 1 : 0;
            }
        } else if ( $import_data['imported'] < $import_data['num'] ) {
            // Import one line from file using line number.
            $import->one( $import_data['index'] );
            // Set progress in percent for response array.
            $import_data['progress'] = ( ( $import_data['imported'] + 1 ) * 100 ) / $import_data['num'];

            // Change line for reading for next executing script.
            $import_data['index'] = $import_data['index'] + 1;
            $import_data['imported'] = $import_data['imported'] + 1;
        } else {
            $import_data['done'] = 1;
            $import_data['progress'] = 100;
        }

        if ( $import_data['progress'] >= 100 ) {
            $import->get_logger()->set_message( __( 'Done!', 'es-plugin' ), 'success' );
            $import_data['done'] = 1;
            $import_data['progress'] = 100;
        }

        unset( $import_data['fields'] );

        $response = array_merge( $import_data, array( 'messages' => $import->get_logger()->get_messages() ) );

        // Send ajax response.
        wp_die( json_encode(
            apply_filters( 'es_import_response', $response )
        ) );
        exit;
    }
}
