<?php

/**
 * Class Es_Settings_Page
 */
class Es_Settings_Page extends Es_Object
{
    /**
     * @inheritdoc
     */
    public function actions()
    {
        add_action( 'wp_ajax_es_get_map_taxonomy_terms', array( $this, 'get_map_taxonomy_terms' ) );
        add_action( 'es_settings_save', array( $this, 'change_agent_permissions' ), 10, 3 );
    }

    /**
     * @return void
     */
    public function get_map_taxonomy_terms()
    {
        $taxonomy = filter_input( INPUT_GET, 'taxonomy' );

        if ( taxonomy_exists( $taxonomy ) ) {
            $items = get_terms( array(
                'taxonomy' => $taxonomy,
                'hide_empty' => false,
            ) );

            if ( ! empty( $items ) ) {
                $response = array( 'items' => $items );
            } else {
                $response = array( 'status' => 0, 'message' => __( 'The Taxonomy is empty.', 'es-plugin' ) );
            }
        } else {
            $response = array( 'status' => 0, 'message' => __( 'The Taxonomy doesn\'t exist', 'es-plugin' ) );
        }

        wp_die( wp_json_encode( $response ) );
    }

    /**
     * Render settings page content.
     *
     * @return void
     */
    public static function render()
    {
        $template = apply_filters( 'es_settings_template_path', ES_ADMIN_TEMPLATES . 'settings/settings.php' );
        include_once( $template );
    }

    /**
     * Return tabs of the settings page.
     *
     * @return array
     */
    public static function get_tabs()
    {
        return apply_filters( 'es_settings_get_tabs', array(
            'general' => array(
                'label' => __( 'General', 'es-plugin' ),
                'template' => ES_ADMIN_TEMPLATES . 'settings/general-tab.php'
            ),
            'layouts' => array(
                'label' => __( 'Layouts', 'es-plugin' ),
                'template' => ES_ADMIN_TEMPLATES . 'settings/layouts-tab.php'
            ),
            'currency' => array(
                'label' => __( 'Currency', 'es-plugin' ),
                'template' => ES_ADMIN_TEMPLATES . 'settings/currency-tab.php'
            ),
            'map' => array(
                'label' => __( 'Map view', 'es-plugin' ),
                'template' => ES_ADMIN_TEMPLATES . 'settings/map-tab.php'
            ),
            'sharing' => array(
                'label' => __( 'Sharing', 'es-plugin' ),
                'template' => ES_ADMIN_TEMPLATES . 'settings/sharing-tab.php'
            ),
        ) );
    }

    /**
     * Save settings action.
     *
     * @return void
     */
    public static function save()
    {
        if ( isset( $_POST['es_save_settings'] ) && wp_verify_nonce( $_POST['es_save_settings'], 'es_save_settings' ) ) {

            /** @var Es_Settings_Container $es_settings */
            global $es_settings;

            // Filtering and preparing data for save.
            $data = apply_filters( 'es_before_save_settings_data', $_POST['es_settings'] );

            // Before save action.
            do_action( 'es_before_settings_save', $data );

            $es_settings->save( $data );

            // After save action.
            do_action( 'es_after_settings_save', $data );
        }
    }

    /**
     * Change capabilities for user role on listing_publishing setting field change.
     * @see do_action( 'es_save_settings' )
     *
     * @return void
     */
    public function change_agent_permissions( $key, $value, $prev_value )
    {
        if ( $value == $prev_value || 'listings_publishing' != $key || es_subscription_is_enabled() ) return;

        $cap = apply_filters( 'es_automatic_publishing_permission_name', 'publish_es_properties' );
        $role = Es_Agent::get_role_name();

        if ( $value == 'automatic' ) {
            es_subscription_change_role_caps( ES_RESTORE_CAPS, $role, array( $cap ) );
        } else {
            es_subscription_change_role_caps( ES_REMOVE_CAPS, $role, array( $cap ) );

            // Query for users.
            $args = array( 'role' => Es_Agent::get_role_name() );
            $users = get_users( $args );

            if ( $users ) {
                foreach ( $users as $user ) {
                    es_subscription_change_user_caps( $user->ID, ES_REMOVE_CAPS, array( $cap ) );
                }
            }
        }
    }
}
