<?php

/**
 * Class Es_Property_Map_Widget
 */
class Es_Property_Map_Widget extends Es_Widget
{
    /**
     * Es_Property_Map_Widget constructor.
     */
    public function __construct()
    {
        parent::__construct( 'es_property_map_widget', __( 'Estatik Map View', 'es-plugin' ) );
    }

    /**
     * Return layouts of this widget.
     *
     * @return array
     */
    public static function get_layouts()
    {
        return apply_filters( 'es_get_property_map_widget_layouts', array( 'vertical', 'horizontal' ) );
    }

    /**
     * @return mixed
     */
    public function generate_map_shortcode( $instance = array() )
    {
        global $es_settings;

        $content = '';
        $data_filter = array();

        $width = ! empty( $instance['width'] ) ? $instance['width'] : $es_settings->map_width;
        $height = ! empty( $instance['height'] ) ? $instance['height'] : $es_settings->map_height;
        $limit = ! empty( $instance['limit'] ) ? $instance['limit'] : $es_settings->properties_per_map;
        $layout = ! empty( $instance['layout'] ) ? $instance['layout'] : 'vertical';

        if ( ! empty( $instance['filter_data'] ) ) {
            foreach ( $instance['filter_data'] as $tid ) {
                $term = get_term( $tid );
                if ( $term && strlen( $term->taxonomy ) > 3 ) {
                    $data_filter[ substr( $term->taxonomy, 3 ) ][] = $term->name;
                }
            }

            if ( is_array( $data_filter ) && ! empty( $data_filter ) ) {
                foreach ( $data_filter as $taxonomy => $slugs ) {
                    $content .= ' ' . $taxonomy . '="' . implode( ',', $slugs ) . '"';
                }
            }
        }

        $content .= ' limit="' . $limit . '"';
        $content .= ' classes="es-map-' . $layout . '"';
        $content .= ' width="' . $width . '" height="' . $height . '"';
        return apply_filters( 'es_generate_map_shortcode', '[es_property_map' . $content . ']' );
    }

    /**
     * @return array
     */
    public static function get_filter_fields_data()
    {
        /** @var Es_Settings_Container $es_settings */
        global $es_settings;

        $data = array();

        $taxonomies = $es_settings::get_setting_values( 'taxonomies' );

        if ( ! empty( $taxonomies ) ) {
            foreach ( $taxonomies as $name => $taxonomy ) {
                if ( taxonomy_exists( $name ) ) {
                    $taxonomy = get_taxonomy( $name );

                    $data[ $taxonomy->label ] = get_terms( array( 'taxonomy' => $taxonomy->name, 'hide_empty' => false ) );
                }
            }
        }

        return apply_filters( 'es_get_property_map_filter_fields_data', $data );
    }

    /**
     * Function for register widget.
     *
     * @return void
     */
    public static function register()
    {
        register_widget( 'Es_Property_Map_Widget' );
    }

    /**
     * @return string
     */
    protected function get_widget_template_path()
    {
        return ES_PLUGIN_PATH . '/admin/templates/widgets/es-property-map.php';
    }

    /**
     * @return string
     */
    protected function get_widget_form_template_path()
    {
        return ES_PLUGIN_PATH . '/admin/templates/widgets/es-property-map-form.php';
    }
}

add_action( 'widgets_init', array( 'Es_Property_Map_Widget', 'register' ) );
