<?php

/**
 * Class Es_Request_Widget
 */
class Es_Request_Widget extends Es_Widget
{
	/**
	 * @var int
	 */
	const SEND_ADMIN = 1;
	/**
	 * @var int
	 */
	const SEND_AGENT = 2;
	/**
	 * @var int
	 */
	const SEND_ADMIN_AGENT = 3;

	/**
	 * @inheritdoc
	 */
	public function __construct()
	{
		parent::__construct( 'es_request_widget' , __( 'Estatik Request Form', 'es-plugin' ) );
	}

	/**
	 * Return list of "send_to" selectbox field.
	 *
	 * @return mixed
	 */
	public static function get_send_to_list()
	{
		return apply_filters( 'es_request_widget_get_send_to_list', array(
			self::SEND_ADMIN       => __( 'Admin', 'es-plugin' ),
			self::SEND_AGENT       => __( 'Agent', 'es-plugin' ),
			self::SEND_ADMIN_AGENT => __( 'Admin & Agent', 'es-plugin' ),
		) );
	}

	/**
	 * Overridden render checker.
	 *
	 * @param $instance
	 * @return bool
	 */
	public static function can_render($instance)
	{
		return get_post_type() == Es_Property::get_post_type_name() && is_single();
	}

	/**
	 * Function for register widget.
	 *
	 * @return void
	 */
	public static function register()
	{
		register_widget( 'Es_Request_Widget' );
	}

	/**
	 * Submit request widget handler.
	 *
	 * @return void
	 */
	public static function submit_form()
	{
        $response = array();
		$nonce = 'es_request_send';

		if ( ! empty( $_POST[ $nonce ] ) && wp_verify_nonce( $_POST[ $nonce ], $nonce ) ) {
            $post = ! empty( $_POST['post_id'] ) ? get_post( $_POST['post_id'] ) : null;
            $check = false;

            if ( isset( $_POST['g-recaptcha-response'] ) ) {
                if ( ! empty( $_POST['g-recaptcha-response'] ) ) {
                    global $es_settings;
                    $secret = $es_settings->recaptcha_secret_key;

                    $verifyResponse = file_get_contents('https://www.google.com/recaptcha/api/siteverify?secret='.$secret.'&response='.$_POST['g-recaptcha-response']);

                    if ( ! empty( $verifyResponse ) ) {
                        $responseData = json_decode( $verifyResponse );

                        if ( ! empty( $responseData->success ) ) {
                            $check = true;
                        }
                    }
                }
            } else {
                $check = true;
            }

            if ( ! $check ) {
                $response = array( 'status' => 'error', 'message' => __( 'Invalid captcha.', 'es-plugin' ) );
            }

			if ( ! empty( $post->post_type ) && $post->post_type == Es_Property::get_post_type_name() && $check ) {
                if ( $emails = static::get_emails( $_POST['send_to'], $post ) ) {
                    $property = es_get_property( $post->ID );
                    $email = $_POST['email'];
                    $name = $_POST['name'];
                    $tel = $_POST['tel'];
                    $message = $_POST['message'];

                    $subject = apply_filters( 'es_request_form_email_subject', $_POST['subject'] );
                    $message = apply_filters( 'es_request_form_email_message', "
                        " . __( 'Name', 'es-plugin' ) . ": " . $name . "\n
                        " . __( 'Email', 'es-plugin' ) . ": " . $email . "\n
                        " . __( 'Phone', 'es-plugin' ) . ": " . $tel . "\n\n
                        " . __( 'Property ID', 'es-plugin' ) . ": " . $post->ID . "\n
                        " . __( 'Property Link', 'es-plugin' ) . ": " . get_permalink( $post->ID ) . "\n
                        " . __( 'Property Address', 'es-plugin' ) . ": " . $property->address . "\n\n
                        " . __( 'Request', 'es-plugin' ) . ": " . $message . "\n
                    " );

                    if ( wp_mail( $emails, $subject, $message ) ) {
                        $response = array( 'status' => 'success', 'message' => __( 'Thank you for your message! We will contact you as soon as we can.', 'es-plugin' ) );
                    } else {
                        $response = array( 'status' => 'error', 'message' => __( 'Your message wasn\'t sent. Please, contact support.', 'es-plugin' ) );
                    }
                }
			} else {
                $response = ! empty( $response ) ? $response : array( 'status' => 'error', 'message' => __( 'Incorrect post.', 'es-plugin' ) );
            }

            $response = apply_filters( 'es_request_form_response', $response );

            $template = apply_filters( 'es_request_form_response_template_path', ES_PLUGIN_PATH . '/templates/partials/request-form-response.php' );

            ob_start();
            include ( $template );

            wp_die( json_encode( ob_get_clean() ) );
		}
	}

    /**
     * Return emails for sending.
     *
     * @param $type
     * @param $post
     * @return mixed
     */
    protected static function get_emails( $type, WP_Post $post )
    {
        $emails = array();
        
        if ( $type == self::SEND_ADMIN || $type == self::SEND_ADMIN_AGENT ) {
            $emails[] = get_option('admin_email');
        }

        if ( $type == self::SEND_AGENT || $type == self::SEND_ADMIN_AGENT ) {
            $author_id = $post->post_author;
            $user = new WP_User( $author_id );

            if ( in_array( Es_Agent::get_role_name(), $user->roles ) && ! empty( $user->user_email ) ) {
                $emails[] = $user->user_email;
            }
        }

        return apply_filters( 'es_request_form_get_emails', $emails, $type, $post );
    }

	/**
	 * @inheritdoc
	 */
	protected function get_widget_template_path()
	{
		return ES_PLUGIN_PATH . '/admin/templates/widgets/es-request-widget.php';
	}

	/**
	 * @return string
	 */
	protected function get_widget_form_template_path()
	{
		return ES_PLUGIN_PATH . '/admin/templates/widgets/es-request-widget-form.php';
	}
}

add_action( 'widgets_init', array( 'Es_Request_Widget', 'register' ) );
add_action( 'wp_ajax_es_request_send', array( 'Es_Request_Widget', 'submit_form' ) );
add_action( 'wp_ajax_nopriv_es_request_send', array( 'Es_Request_Widget', 'submit_form' ) );