<?php

/**
 * Class Es_Search_Widget
 */
class Es_Search_Widget extends Es_Widget
{
    /**
     * @inheritdoc
     */
    public function __construct()
    {
        parent::__construct( 'es_search_widget' , __( 'Estatik Search', 'es-plugin' ), array(
            'description' => __( 'Estatik Search is advanced search widget developed for Estatik plugins.', 'es-plugin' )
        ) );
	    add_action( 'es_widget_es_search_widget_page_access_block', array( $this, 'search_page' ), 11, 1 );
    }

	/**
	 * @param $instance
	 */
    public function search_page( $instance ) {
	    $instance['page_id'] = ! empty( $instance['page_id'] ) ? $instance['page_id'] : null;
    	$pages[] = __( 'WP Search Page', 'es-plugin' );
	    $post_pages = get_pages();

	    if ( $post_pages ) {
		    foreach ( $post_pages as $page ) {
			    $pages[ $page->ID ] = $page->post_title;
		    }
	    } ?>

	    <p>
		    <label for="<?php echo esc_attr( $this->get_field_id( 'page_id' ) ); ?>">
			    <?php _e( 'Search page:', 'es-plugin' ); ?>
		    </label>
		    <select name="<?php echo esc_attr( $this->get_field_name( 'page_id' ) ); ?>" class="widefat">
			    <?php foreach ( $pages as $id => $page ) : ?>
				    <option <?php selected( $id, $instance['page_id'] ); ?> value="<?php echo $id; ?>"><?php echo $page; ?></option>
			    <?php endforeach; ?>
		    </select>
	    </p><?php
    }

    /**
     * Return layouts of this widget.
     *
     * @return array
     */
    public static function get_layouts()
    {
        return apply_filters( 'es_get_search_widget_layouts', array( 'horizontal' => __( 'Horizontal', 'es-plugin' ), 'vertical' => __( 'Vertical', 'es-plugin' ) ) );
    }

    /**
     * Return search fields.
     *
     * @return array
     */
    public static function get_widget_fields()
    {
        $taxonomies = Es_Taxonomy::get_taxonomies_list();
        $key = array_search( 'es_labels', $taxonomies );

        if ( $key ) unset( $taxonomies[ $key ] );

        return apply_filters( 'es_get_widget_fields', array_merge( array(
            'address',
            'price',
            'bedrooms',
            'bathrooms',
            'floors',
            'area',
            'lot_size',
            'country',
            'state',
            'city',
            'street',
            'post_author',
        ), $taxonomies ) );
    }

    /**
     * Function for register widget.
     *
     * @return void
     */
    public static function register()
    {
        register_widget( 'Es_Search_Widget' );
    }

    /**
     * Render field for search widget.
     *
     * @param $name
     * @return string|boolean
     */
    public static function render_field( $name ) {
        $property = es_get_property( null );

        // If input data is empty.
        if ( empty( $name ) || ! $field = $property::get_field_info( $name ) ) return false;

        // Field content string;
        $content = null;
        // Current field value.
        $value = isset( $_GET['es_search'][ $name ] ) ? $_GET['es_search'][ $name ] : null;
        // Field options.
        $options = ' ';

        if ( empty( $field['options']['id'] ) ) {
            $field['options']['id'] = empty( $field['search_range_mode'] ) ?
                'es-search-' . $name . '-input' : null;
        }

        if ( isset( $field['options']['required'] ) ) {
            unset( $field['options']['required'] );
        }

        // Set value as data attribute for ajax fields.
        if ( ! empty( $value ) ) {
            $field['options']['data-value'] = $value;
            $field['options']['value'] = $value;
        } else {
			$field['options']['value'] = '';
		}

	    if ( empty( $field['options']['value'] ) && ! empty( $field['default_value'] ) ) {
		    $field['options']['value'] = $field['default_value'];
	    }

        $field['type'] = ! empty( $field['type'] ) ? $field['type'] : null;

        if ( ! empty( $field['type'] ) && ( 'date' == $field['type'] || 'datetime' == $field['type'] ) ) {
            $field['options']['data-date-field'] = 1;
        }

        if ( ! empty ( $field['values_callback'] ) ) {
            $field['values'] = call_user_func( $field['values_callback'][0], $field['values_callback'][1] );
        }

        if ( ! empty( $field['options'] ) ) {
            if ( ! empty( $field['search_range_mode'] ) ) {
                unset( $field['options']['value'] );
            }

            foreach ( $field['options'] as $key => $option ) {
                if ( is_array( $option ) ) continue;
                $options .= $key . '="' . $option . '" ';
            }
        }

        // Generate label if empty.
        if (isset($field['label']) && $field['label'] == FALSE) {
            $field['label'] = '';
        }
        else {
            $label = ! empty( $field['label'] ) ? $field['label'] : __( Es_Html_Helper::generate_label( $name ), 'es-plugin' );
            $field['label'] = '<div class="es-field__label">
                <label for="' . $field['options']['id'] . '">
                ' . $label . '
                </label></div>';
        }

        if ( ! empty( $field['search_range_mode'] ) ) {
            $field['type'] = 'text';
        }

        $class_unit = null;

        if ( ! empty( $field['units'] ) ) {
            $class_unit = 'es-field__wrap--units';
        }

        if ( empty( $field['template'] ) ) {
            $content .= '<div class="es-field__wrap ' . $class_unit . '">';
        }

        $multiple = ! empty( $field['options']['multiple'] ) ? '[]' : '';

        switch ( $field['type'] ) {
            case 'list':
                $content .= '<select name="es_search[' . $name . ']' . $multiple . '" ' . $options .'>';

                if ( ! empty( $field['prompt'] ) ) {
                    $content .= '<option value="">' . $field['prompt'] . '</option>';
                } else if ( ! empty( $field['fbuilder'] ) ) {
                    $content .= '<option value="">' . __( 'Choose value', 'es-plugin' ) . '</option>';
                }

                if ( ! empty( $field['values'] ) ) {
                    foreach ( $field['values'] as $value => $label ) {
                        $values = ! empty( $field['options']['value'] ) && is_array( $field['options']['value'] ) ?
                            $field['options']['value'] : array();

                        $selected = is_array( $value ) ?
                            selected( true, in_array( $value, $values ), false ) :
                            selected( $value, $field['options']['value'], false );

                        if ( is_array( $field['options']['value'] ) && ! $selected ) {
                            if ( in_array( $value, $field['options']['value'] ) ) {
                                $selected = selected( true, true, false );
                            }
                        }

                        if ( is_array( $field['options']['value'] ) && ! $selected ) {
                            if ( in_array( $value, $field['options']['value'] ) ) {
                                $selected = selected( true, true, false );
                            }
                        }

                        $content .= '<option value="' . $value . '" ' . $selected . '>' . $label . '</option>';
                    }
                }

                $content .= '</select>';
                break;

            case 'radio':
            case 'checkbox':
                $content .= '<input type="' . $field['type'] . '" name="es_search[' . $name . ']" ' . $options . ' ' . checked( $value, $field['options']['value'], false ) . '/>';
                break;

            case 'autocomplete':
                $content .= '<div class="es-autocomplete-wrap js-autocomplete-wrap">';
                $content .= '<input type="text" name="es_search[' . $name . ']" ' . $options .' data-action="' . $field['autocomplete_action'] . '"/>';
                $content .= '<div class="es-autocomplete-result"></div>';
                $content .= '</div>';
                break;

            default:
                if ( ! empty( $field['search_range_mode'] ) ) {
                    $min = ! empty( $value['min'] ) ? $value['min'] : '';
                    $max = ! empty( $value['max'] ) ? $value['max'] : '';
                    $content .= '<div class="es-field__range"><input type="' . $field['type'] . '" placeholder="' . __( 'min', 'es-plugin' ) . '" name="es_search[' . $name . '][min]" ' . $options .' value="' . $min . '"/>';
                    $content .= '<input type="' . $field['type'] . '" placeholder="' . __( 'max', 'es-plugin' ) . '" name="es_search[' . $name . '][max]" ' . $options .' value="' . $max . '"/>';
                } else {
                    $content .= '<input type="' . $field['type'] . '" name="es_search[' . $name . ']" ' . $options .'/>';
                }
        }

        if ( ! empty( $field['units'] ) ) {
            $content .= self::render_field( $field['units'] );
        }

        if ( ! empty( $field['search_range_mode'] ) ) {
            $content .= '</div>';
        }

        if ( empty( $field['template'] ) ) {
            $content .= '</div>';
        }

        $content = $field['label'] . $content;

        return apply_filters( 'es_search_render_field', $content, $field, $name );
    }

    /**
     * Return location items for search widget.
     *
     * @return void
     */
    public static function get_location_items()
    {
        $response = null;

        if ( !empty( $_POST['status'] ) && $_POST['status'] == 'initialize' ) {
            $response = ES_Address_Components::get_component_list( $_POST['type'] );
        } else if ( !empty( $_POST['status'] ) && $_POST['status'] == 'dependency' ) {
            $response = Es_Search_Location::get_related_location( $_POST['type'], $_POST['val'] );
        }

        $response = apply_filters( 'es_search_get_location_items_response', $response );

        wp_die( json_encode( $response ) );
    }

	/**
	 * @param $query WP_Query
	 * @param $data array
	 */
    public static function build_query( $query, $data ) {

        if ( $data ) {
            $property = es_get_property( null );

            foreach ( $data as $field => $value ) {
                if ( empty( $value ) ) continue;

                $field_info = $property::get_field_info( $field );
                $field_key = $property->get_entity_prefix() . $field;

                if ( ! empty( $field_info['skip_search'] ) ) continue;

                switch ( $field ) {
                    case 'country':
                        $meta_query[] = array(
                            'key' => '_address_component_country',
                            'value' => $value,
                            'compare' => '=',
                        );
                        break;

                    case 'neighborhood':
                        $meta_query[] = array(
                            'key' => '_address_component_neighborhood',
                            'value' => $value,
                            'compare' => '=',
                        );
                        break;

                    case 'city':
                        $meta_query[] = array(
                            'key' => '_address_component_locality',
                            'value' => $value,
                            'compare' => '=',
                        );
                        break;

                    case 'state':
                        $meta_query[] = array(
                            'key' => '_address_component_administrative_area_level_1',
                            'value' => $value,
                            'compare' => '=',
                        );
                        break;

                    case 'street':
                        $meta_query[] = array(
                            'key' => '_address_component_route',
                            'value' => $value,
                            'compare' => '=',
                        );
                        break;

                    case 'address':
                        // Create array from address string using delimiters.
                        if ( ! empty( $value ) ) {

                            $ids = $property::find_by_address( $value );

                            if ( ! empty( $ids ) ) {
                                $query->set( 'post__in', $ids );
                            } else {
                                $query->set( 'post__in', array( -1 ) );
                            }
                        }
                        break;

                    case 'featured':
                    case 'hot':
                    case 'foreclosure':
                    case 'open_house':
                        $meta_query[] = array(
                            'key' => $property->get_entity_prefix() . $field,
                            'value' => $value,
                            'compare' => '=',
                        );
                        break;

                    case 'post_author':
                        if ( ! empty( $value ) ) $query->set( 'author__in', $value );
                        break;

                    default:
                        if ( $field == 'price' && ( ! empty( $value['min'] ) || ! empty( $value['max'] ) ) ) {
                            $meta_query[] = array(
                            	'relation' => 'OR',
	                            array( 'compare' => '=', 'key' => $property->get_entity_prefix() . 'call_for_price', 'value' => '0' ),
	                            array( 'compare' => '=', 'key' => $property->get_entity_prefix() . 'call_for_price', 'value' => null ),
	                            array( 'compare' => 'NOT EXISTS', 'key' => $property->get_entity_prefix() . 'call_for_price' ),
                            );
                        }

                        if ( ! empty( $field_info['units'] ) ) {
                            if ( ! empty( $data[ $field_info['units'] ] ) ) {
                                $field_key = $property->get_entity_prefix() . $field . '_' . $data[ $field_info['units'] ];
                            }
                        }

                        if ( ! empty( $field_info['search_range_mode'] ) ) {

                            switch ( $field_info['type'] ) {
                                case 'date':
                                    $type = 'DATE';
                                    break;

                                case 'datetime':
                                    $type = 'DATETIME';
                                    break;

                                default:
                                    $type = 'NUMERIC';
                            }

                            if ( ! empty($value['min'] ) ) {
                                $meta_query[] = array(
                                    'key' => $field_key,
                                    'value' => $field_info['type'] != 'number' ? $value['min'] : (int) $value['min'],
                                    'compare' => '>=',
                                    'type' => $type,
                                );
                            }

                            if ( ! empty( $value['max'] ) ) {
                                $meta_query[] = array(
                                    'key' => $field_key,
                                    'value' => $field_info['type'] != 'number' ? $value['max'] : (int) $value['max'],
                                    'compare' => '<=',
                                    'type' => $type,
                                );
                            }
                        } else if ( ! empty( $field_info['system_type'] ) && 'taxonomy' == $field_info['system_type'] ) {
                            $tax_query['relation'] = 'AND';
                            $tax_query[] = array(
                                'taxonomy' => $field,
                                'field' => 'id',
                                'terms' => $value,
                                'operator' => ! empty( $field_info['search_operator'] ) ? $field_info['search_operator'] : 'AND'
                            );
                        } else {
                            $meta_query[] = array(
                                'key' => $field_key,
                                'value' => $value,
                            );
                        }
                }
            }

            // If not only relation key exists in tax array.
            if ( ! empty( $tax_query ) && count( $tax_query ) > 1 ) {
                $query->set( 'tax_query', $tax_query );
            }

            if ( ! empty( $meta_query ) ) {
                $query->set( 'meta_query', $meta_query );
            }
        }

        return $query;
    }

    /**
     * Customize standard search query.
     *
     * @param WP_Query $query
     * @return void
     */
    public static function pre_get_posts( $query )
    {
        // If query is search.
        if ( ! empty( $_GET['es_search'] ) && is_array( $_GET['es_search'] ) && $query->is_search && ! is_admin() ) {

            $property = es_get_property( null );
            $query->set( 'post_type', $property::get_post_type_name() );

            static::build_query( $query, $_GET['es_search'] );
        }
    }

    /**
     * Return addresses for autocomplete field.
     *
     * @return void
     */
    public static function address_autocomplete_handler()
    {
        $response = '';
        if ( ! empty( $_POST['s'] ) ) {
            global $wpdb;
            $addresses = array();
            $query = $_POST['s'];
            $num_to_show = apply_filters( 'es_autocomplete_addresses_to_show', 10 );

            $results_adresses = $wpdb->get_col( "SELECT meta_value 
                FROM $wpdb->postmeta 
                WHERE meta_key = 'es_property_address' 
                AND LOWER(`meta_value`) 
                LIKE '%" . strtolower( $query ) . "%'" );

            $prepare = $wpdb->prepare(
                "SELECT long_name, type FROM {$wpdb->prefix}address_components
                       WHERE long_name LIKE '%s' OR short_name LIKE %s",
                array( '%' . $query . '%', '%' . $query . '%' )
            );

            $results_addresses_components = $wpdb->get_results( $prepare );

            if ( $results_adresses ) {
                foreach ( $results_adresses as $address ) {
                    if ( count( $addresses ) >= $num_to_show ) break;
                    $addresses[] = array( 'text' => $address, 'label' => __( 'Address', 'es-plugin' ), 'type' => 'address' );
                }
            }

            if ( $results_addresses_components ) {
                foreach ( $results_addresses_components as $addresses_component ) {
                    $type = ES_Address_Components::get_label_by_type( $addresses_component->type );
                    $addresses[] = array( 'text' => $addresses_component->long_name, 'label' => $type, 'type' => $addresses_component->type );
                }
            }

            if ( ! empty( $addresses ) ) {
                $response .= '<ul class="autocomplete-dropdown-list">';
                foreach ( $addresses as $address ) {
                    if ( $num_to_show <=0 ) break;
                    $response .= '<li data-content="' . $address['text'] . '"><sub class="es-adddress-type es-address-type-' . $address['type'] . '">' . $address['label'] . '</sub>' . $address['text'] . '</li>';
                    $num_to_show--;
                }
                $response .= '</ul>';
            }
        }

        $response = apply_filters( 'address_autocomplete_response', $response );

        wp_die( $response );
    }

    /**
     * @inheritdoc
     */
    protected function get_widget_template_path()
    {
        return ES_PLUGIN_PATH . '/admin/templates/widgets/es-search-widget.php';
    }

    /**
     * @return string
     */
    protected function get_widget_form_template_path()
    {
        return ES_PLUGIN_PATH . '/admin/templates/widgets/es-search-widget-form.php';
    }
}

add_action( 'widgets_init', array( 'Es_Search_Widget', 'register' ) );
add_action( 'wp_ajax_nopriv_es_get_location_items', array( 'Es_Search_Widget', 'get_location_items' ) );
add_action( 'wp_ajax_es_get_location_items', array( 'Es_Search_Widget', 'get_location_items' ) );
add_action( 'pre_get_posts', array( 'Es_Search_Widget', 'pre_get_posts' ), 20 );
add_action( 'wp_ajax_es_address_autocomplete', array( 'Es_Search_Widget', 'address_autocomplete_handler' ), 20 );
add_action( 'wp_ajax_nopriv_es_address_autocomplete', array( 'Es_Search_Widget', 'address_autocomplete_handler' ), 20 );
