<?php

/**
 * @var Es_Settings_Container $es_settings
 */

?>

<div class="es-manage-markers-wrap">
    <h2><?php _e( 'Set map markers', 'es-plugin' ); ?></h2>

    <div class="es-manage-markers-inner">
        <select name="" id="" class="js-es-map-taxonomy-field" data-action="es_get_map_taxonomy_terms">
            <option value="">-- <?php _e( 'Select taxonomy', 'es-plugin' ); ?> --</option>

            <?php if ( $taxonomies = $es_settings::get_setting_values( 'taxonomies' ) ) : ?>
                <?php foreach ( $taxonomies as $taxonomy => $label ) : ?>
                    <option value="<?php echo $taxonomy; ?>"><?php echo $label; ?></option>
                <?php endforeach; ?>
            <?php endif; ?>
        </select>

        <select name="" id="" class="js-es-map-terms-field" disabled>
            <option value="">-- <?php _e( 'Select term', 'es-plugin' ); ?> --</option>
        </select>

        <ul class="es-markers-list">
            <?php if ( ( $items = $es_settings->map ) && ( $markers = $es_settings::get_setting_values( 'markers' ) )  ) : ?>
                <?php if ( ! empty( $items ) && is_array( $items ) ) : ?>
                    <?php foreach ( $items as $term_id => $item ) : ?>
                        <?php if ( ! term_exists( $term_id ) ) continue; ?>
                        <?php $term = get_term( $term_id ); $taxonomy = get_taxonomy( $term->taxonomy ); ?>

                        <li class="es-markers-list__item es-markers-list__item-<?php echo $term_id; ?>">
                            <div class="item-label"><?php echo $term->name; ?></div>
                            <div class="item-content">
                                <div class="content-markers">
                                    <?php foreach ( $markers as $key => $marker ) : ?>
                                        <div class="marker<?php echo $item['icon'] == $key ? ' marker__active' : ''; ?>">
                                            <?php echo $marker; ?>
                                            <input class="js-item-marker-field" <?php checked( $item['icon'], $key ); ?> type="radio" name="es_settings[map][<?php echo $term_id; ?>][icon]" value="<?php echo $key; ?>">
                                        </div>
                                    <?php endforeach; ?>
                                </div>
                                <div class="content-color">
                                    <input type="color" class="js-es-item-color" name="es_settings[map][<?php echo $term_id; ?>][color]"
                                           value="<?php echo $item['color']; ?>"/>
                                </div>
                            </div>
                            <div class="item-remove">
                                <a href="" class="es-item-remove"><span class="es-sprite es-sprite-close"></span></a>
                            </div>
                        </li>
                    <?php endforeach; ?>
                <?php endif; ?>
            <?php endif; ?>
        </ul>
    </div>
    <h2><?php _e( 'Extra settings', 'es-plugin' ); ?></h2>

    <?php echo Es_Html_Helper::render_settings_field( __( 'Width', 'es-plugin' ), 'es_settings[map_width]', 'text', array(
        'value' => $es_settings->map_width,
        'min' => '1',
    ) );

    echo Es_Html_Helper::render_settings_field( __( 'Height', 'es-plugin' ), 'es_settings[map_height]', 'text', array(
        'value' => $es_settings->map_height,
        'min' => '1',
    ) );

    echo Es_Html_Helper::render_settings_field( __( 'Number of listings to show', 'es-plugin' ), 'es_settings[properties_per_map]', 'number', array(
        'value' => $es_settings->properties_per_map,
        'min' => '1',
    ) );

    echo Es_Html_Helper::render_settings_field( __( 'Zoom', 'es-plugin' ), 'es_settings[map_zoom]', 'number', array(
        'value' => $es_settings->map_zoom,
        'min' => '0',
        'max' => '20',
    ) ); ?>

    <div class="es-field">
        <div class="es-field__label"></div>
        <div class="es-field__content">
            <sub><?php _e( 'If zoom field is empty then google map calculate zoom automatically.', 'es-plugin' ); ?></sub>
        </div>
    </div>


</div>
<div class="es-map-description">
    <h2><?php _e( 'Shortcodes', 'es-plugin' ); ?></h2>
    <p><?php _e( 'Please create pages with shortcodes below to display map on your website', 'es-plugin' ); ?></p>

    <ul class="es-shortcode-info-list">

        <li>
            <div class="shortcode-name">[es_property_map show="all"]</div>
            <div class="shortcode-description">- <?php _e( 'All properties', 'es-plugin'); ?></div>
        </li>

        <li>
            <div class="shortcode-name">[es_property_map type="your type"]</div>
            <div class="shortcode-description">- <?php _e( 'Properties by type', 'es-plugin'); ?></div>
        </li>

        <li>
            <div class="shortcode-name">[es_property_map category="your category"]</div>
            <div class="shortcode-description">- <?php _e( 'Properties by category', 'es-plugin'); ?></div>
        </li>

        <li>
            <div class="shortcode-name">[es_property_map status="your status"]</div>
            <div class="shortcode-description">- <?php _e( 'Properties by status', 'es-plugin'); ?></div>
        </li>

        <li>
            <div class="shortcode-name">[es_property_map rent_period="your period"]</div>
            <div class="shortcode-description">- <?php _e( 'Properties by rent period', 'es-plugin'); ?></div>
        </li>

        <li>
            <div class="shortcode-name">[es_property_map limit=20]</div>
            <div class="shortcode-description">- <?php _e( 'Num properties for display', 'es-plugin'); ?></div>
        </li>

        <li>
            <div class="shortcode-name">[es_property_map ids="1,2,3,4,5"]</div>
            <div class="shortcode-description">- <?php _e( 'Specific properties', 'es-plugin'); ?></div>
        </li>

        <li>
            <div class="shortcode-name">[es_property_map address="your address string"]</div>
            <div class="shortcode-description">- <?php _e( 'Specific properties', 'es-plugin'); ?></div>
        </li>
</div>