<?php

/**
 * @var array $instance
 * @var Es_Property_Map_Widget $this
 * @var Es_Settings_Container $es_settings
 */

wp_enqueue_script('es-select2-script');

global $es_settings;

$title         = ! empty( $instance['title'] )        ? $instance['title']          : null;
$width         = ! empty( $instance['width'] )        ? $instance['width']          : $es_settings->map_width;
$height        = ! empty( $instance['height'] )       ? $instance['height']         : $es_settings->map_height;
$layout        = ! empty( $instance['layout'] )       ? $instance['layout']         : null;
$filter_data   = ! empty( $instance['filter_data'] )  ? $instance['filter_data']    : array();
$limit         = ! empty( $instance['limit'] )        ? $instance['limit']          : $es_settings->properties_per_map;

$filter_data = ! empty( $filter_data[0] ) && is_array( $filter_data[0] ) ? $filter_data[0] : $filter_data;

?>

<div class="es-widget-wrap">
    <p>
        <label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php _e( 'Map Title:', 'es-plugin' ); ?></label>
        <input type="text" class="widefat" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" value="<?php echo $title; ?>"/>
    </p>

    <?php if ( $data = $this::get_filter_fields_data() ) : ?>
        <select data-placeholder="<?php _e( '-- Select parameters for filtering --', 'es-plugin' ); ?>"
                style="width: 100%"
                multiple class="js-select2-tags"
                name="<?php echo esc_attr( $this->get_field_name( 'filter_data[]' ) ); ?>"
                id="<?php echo esc_attr( $this->get_field_id( 'filter_data' ) ); ?>">

            <?php foreach ( $data as $group => $items ) : ?>
                <?php if ( empty( $items ) ) continue; ?>

                <optgroup label="<?php echo $group; ?>">
                    <?php foreach ( $items as $value => $item ) : ?>
                        <?php if ( $item instanceof WP_Term) : ?>
                            <option <?php selected( in_array( $item->term_id , $filter_data ) ); ?> value="<?php echo $item->term_id; ?>"><?php echo $item->name; ?></option>
                        <?php else : ?>
                            <option <?php selected( in_array( $item->term_id , $filter_data ) ); ?> value="<?php echo $value; ?>"><?php echo $item; ?></option>
                        <?php endif; ?>
                    <?php endforeach; ?>
                </optgroup>

            <?php endforeach; ?>
        </select>
    <?php endif; ?>

    <?php if ( $layouts = $this::get_layouts() ) : ?>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'layout' ) ); ?>"><?php _e( 'Layouts', 'es-plugin' ); ?>:</label>
            <select name="<?php echo esc_attr( $this->get_field_name( 'layout' ) ); ?>" class="widefat">
                <?php foreach ( $layouts as $field ) : ?>
                    <option <?php selected( $field, $layout ); ?> value="<?php echo $field; ?>"><?php echo Es_Html_Helper::generate_label( $field ); ?></option>
                <?php endforeach; ?>
            </select>
        </p>
    <?php endif; ?>

    <p>
        <label for="<?php echo esc_attr( $this->get_field_id( 'width' ) ); ?>"><?php _e( 'Width', 'es-plugin' ); ?>: </label>
        <input name="<?php echo esc_attr( $this->get_field_name( 'width' ) ); ?>" type="text" class="widefat" value="<?php echo $width; ?>">
    </p>

    <p>
        <label for="<?php echo esc_attr( $this->get_field_id( 'height' ) ); ?>"><?php _e( 'Height', 'es-plugin' ); ?>: </label>
        <input name="<?php echo esc_attr( $this->get_field_name( 'height' ) ); ?>" type="text" class="widefat" value="<?php echo $height; ?>">
    </p>

    <p>
        <label for="<?php echo esc_attr( $this->get_field_id( 'limit' ) ); ?>"><?php _e( 'Limit', 'es-plugin' ); ?>: </label>
        <input name="<?php echo esc_attr( $this->get_field_name( 'limit' ) ); ?>" type="number" min="1" class="widefat" value="<?php echo $limit; ?>">
    </p>

    <?php do_action( 'es_widget_' . $this->id_base . '_page_access_block', $instance ); ?>
</div>
