(function($) {
    'use strict';

    $(function() {
        var mapBlocks = $('.es-properties-map');

        $(window).on('resize', function() {
            mapBlocks.each(function() {
                var $this = $(this);

                if ($this.width() > 360) {
                    if ($this.hasClass('es-map-vertical')) {
                        $this.removeClass('es-map-vertical');
                    }
                } else {
                    if (!$this.hasClass('es-map-vertical')) {
                        $this.addClass('es-map-vertical');
                    }
                }
            });
        });

        $(window).trigger('resize');

        if (mapBlocks.length && Estatik.properties && EsGoogleMap) {
            for (var mapID in Estatik.properties) {
                var mapElement = document.getElementById('es-property-map-' + mapID);

                if (mapElement) {

                    var instance = new EsGoogleMap(mapElement, null, null, Estatik.settings.map_zoom).init();
                    var bounds = new google.maps.LatLngBounds();
                    var loc;

                    if ( Estatik.properties[mapID].length ) {

                        var isSetCenter = false;

                        for (var i in Estatik.properties[mapID]) {
                            var item = Estatik.properties[mapID][i];
                            var m, color;

                            if (item.lat && item.lon) {
                                item.lon = parseFloat(item.lon);
                                item.lat = parseFloat(item.lat);

                                loc =  new google.maps.LatLng(item.lat, item.lon);
                                bounds.extend(loc);

                                if ( Estatik.settings.map_zoom && ! isSetCenter ) {
                                    instance.instance.setCenter( loc );
                                    isSetCenter = true;
                                }

                                if (!item.marker.content) {
                                    m = $(Estatik.settings.default_marker.marker);
                                    color = Estatik.settings.default_marker.color;
                                } else {
                                    m = $(item.marker.content);
                                    color = item.marker.color;
                                }

                                m.css('color', color);

                                item.marker = m[0].outerHTML;

                                var marker = instance.setCustomMarker(item.marker, {lat: item.lat, lng: item.lon});

                                // Create property map overlay.
                                if (item.template) {
                                    var infoWindow = new google.maps.InfoWindow({
                                        maxWidth: 340
                                    });

                                    google.maps.event.addListener(marker, 'click', (function(marker, content) {
                                        return function() {
                                            infoWindow.setContent(content);
                                            infoWindow.open(instance.instance, marker);

                                            $('.gm-style-iw').next().css({'height': '0px'}); //remove arrow bottom inforwindow
                                            $('.gm-style-iw').prev().html(''); //remove style default inforwindows
                                        }
                                    })(marker, item.template));

                                    $(document).on('click', '.es-overlay__close', function() {
                                        infoWindow.close();
                                        return false;
                                    })
                                }
                            }

                            if ( bounds && ! Estatik.settings.map_zoom ) {
                                instance.instance.fitBounds(bounds);
                                instance.instance.panToBounds(bounds);
                            }
                        }
                    } else {
                        if (navigator.geolocation) {
                            navigator.geolocation.getCurrentPosition(function (position) {
                                var initialLocation = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
                                instance.instance.setCenter(initialLocation);
                            }, function() {
                                $( mapElement ).parent().hide();
                            });
                        } else {
                            $( mapElement ).parent().hide();
                        }
                    }
                }
            }
        }
    });
})(jQuery);
