<?php

/**
 * Class Estatik
 */
class Estatik
{
    /**
     * Plugin instance.
     *
     * @var Estatik
     */
    protected static $_instance;
    /**
     * Plugin version.
     *
     * @var string
     */
    protected static $_version = '3.4.0';

    /**
     * Estatik constructor.
     */
    protected function __construct()
    {
        $this->activation();
        $this->deactivation();
        $this->set_global_vars();
        $this->actions();
        $this->init();
        $this->set_global_vars();
        $this->filters();
        $this->set_global_vars();
    }

    /**
     * Return plugin version.
     *
     * @return string
     */
    public static function getVersion()
    {
        return static::$_version;
    }

    /**
     * Return plugin instance.
     *
     * @return Estatik
     */
    protected static function getInstance()
    {
        return is_null( static::$_instance ) ? new Estatik() : static::$_instance;
    }

    /**
     * Initialize plugin.
     *
     * @return void
     */
    public static function run()
    {
        self::class_loader();
        static::$_instance = static::getInstance();
    }

    /**
     * Return registered image size.
     *
     * @return array
     */
    public static function get_image_sizes()
    {
        return array(
            'thumbnail' => array( 100, 100, true ),
            'es-image-size-archive' => array( 536, 370, true ),
            'es-agent-size' => array( 190, 250, true ),
        );
    }

    /**
     * Execute on plugin activate action.
     *
     * @return void
     */
    public function activation()
    {
        $sizes = self::get_image_sizes();

        foreach ( $sizes as $name => $data ) {
            if ( ! has_image_size( $name ) )
                add_image_size( $name, $data[0], $data[1], $data[2] );
        }
    }

    /**
     * Execute on plugin deactivate action.
     *
     * @return void
     */
    public function deactivation() {}

    /**
     * Add plugin actions.
     *
     * @return void
     */
    public function actions()
    {
        add_action( 'wp', array( $this, 'set_global_vars' ) );
        add_action( 'init', array( $this, 'register_post_types' ) );
	    add_action( 'init', array( $this, 'register_taxonomies' ) );
	    add_action( 'init', array( $this, 'install_after_update' ) );
        add_action( 'init', array( $this, '_migration' ) );
        add_action( 'admin_menu', array( $this, 'register_admin_pages' ) );
        add_action( 'es_after_content', array( $this, 'powered' ) );
        add_action( 'es_shortcode_list_after', array( $this, 'powered' ) );
        add_action( 'es_shortcode_after', array( $this, 'powered' ) );
        add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
        add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_styles' ) );
        add_action( 'admin_init', array( $this, 'admin_restrict_access' ) );
        add_action( 'init', array( $this, 'admin_bar_access' ) );
        add_action( 'init', array( $this, 'login' ) );

        add_action( 'save_post', array( 'Es_Property', 'save' ), 10, 2 );

        add_action( 'save_post', array( 'Es_Subscription', 'save' ), 10, 2 );
        add_action( 'before_delete_post', array( 'Es_Property', 'delete' ) );
        add_action( 'init', array( 'Es_Settings_Page', 'save' ) );

        add_action( 'login_enqueue_scripts', array( __CLASS__, 'es_admin_logo' ) );

        add_action( 'pre_get_posts', function( $query ) {
            $user = wp_get_current_user();
            if ( ! empty( $user ) && in_array( Es_Agent::get_role_name(), $user->roles ) && is_admin() ) {
                /** @var WP_Query $query */
                $query->set( 'author', get_current_user_id() );
            }
        } );

        add_action( 'post_edit_form_tag' , function() {
            echo ' enctype="multipart/form-data"';
        } );
    }

	public function install_after_update() {
		if ( ! self::was_activated() ) {
			self::install();
		}
	}

    /**
     * Add filters to the wp functionality.
     *
     * @return void
     */
    public function filters() {
        if ( ! is_admin() ) {
            add_filter( 'body_class', array( $this, 'body_class' ) );
        }
        add_filter( 'image_resize_dimensions', array( $this, 'thumbnail_upscale' ), 10, 6 );
        add_filter( 'authenticate', array( $this, 'authenticate' ), 30, 1 );
        add_filter( 'lostpassword_url', array( $this, 'lostpassword_url'), 10, 1 );
    }

    /**
     * @param $url
     *
     * @return mixed
     */
    public function lostpassword_url( $url )
    {
        global $es_settings;

        $restore_pwd_page_id = $es_settings->reset_password_page_id;

        if ( $restore_pwd_page_id && ( $post = get_post( $restore_pwd_page_id ) ) ) {
            $url = get_permalink( $post );
        }

        return $url;
    }

    /**
     * Add class to the body.
     *
     * @param $classes
     * @return array
     */
    public function body_class( $classes )
    {
        global $es_settings;

        return array_merge( $classes, array( 'es-theme-' . $es_settings->theme_style ) );
    }

    /**
     * Crop image fix.
     *
     * @param $default
     * @param $orig_w
     * @param $orig_h
     * @param $new_w
     * @param $new_h
     * @param $crop
     * @return array|null
     */
    public function thumbnail_upscale( $default, $orig_w, $orig_h, $new_w, $new_h, $crop ){
        if ( !$crop ) return null; // let the wordpress default function handle this

        $aspect_ratio = $orig_w / $orig_h;
        $size_ratio = max($new_w / $orig_w, $new_h / $orig_h);

        $crop_w = round($new_w / $size_ratio);
        $crop_h = round($new_h / $size_ratio);

        $s_x = floor( ($orig_w - $crop_w) / 2 );
        $s_y = floor( ($orig_h - $crop_h) / 2 );

        return array( 0, 0, (int) $s_x, (int) $s_y, (int) $new_w, (int) $new_h, (int) $crop_w, (int) $crop_h );
    }

    /**
     * Initialize entity classes using specific conditions.
     *
     * @return void
     */
    public function init()
    {
        global $es_settings, $session_storage;

        $session_storage = new Es_Session_Storage( 'estatik_cache' );
        $session_storage->clear_all();

        $es_settings = new Es_Settings_Container();

        Es_Subscription_Init::run();

        // Initialize classes for admin panel.
        if ( is_admin() ) {

            // Initialize admin properties list page.
            if ( isset( $_GET['post_type'] ) && $_GET['post_type'] == Es_Property::get_post_type_name() ) {
                Es_Property_List_Page::init();
            }

            // Initialize update pro page.
            if ( isset( $_GET['page'] ) && $_GET['page'] == 'es_pro' ) {
                Es_Upgrade_Pro_Page::init();
            }

	        if ( isset( $_GET['page'] ) && $_GET['page'] == 'es_agent' ) {
		        Es_Agent_Profile::init();
	        }

			// Initialize dashboard page.
			if ( isset( $_GET['page'] ) && $_GET['page'] == 'es_dashboard' ) {
				Es_Dashboard_Page::init();
			}

            Es_Property_Metabox::init();
            Es_Agent_List_Page::init();
            Es_Settings_Page::init();
            Es_Data_Manager_Page::init();
            Es_Fields_Builder_Page::init();
        } else {
            // Initialize template loader class.
            Es_Template_Loader::init();
            Es_Archive_Sorting::init();
            Es_Property_Archive_Page::init();
            Es_Property_Single_Page::init();
        }

        Es_Prop_Management_Init::init();
        Es_Import_Page::init();
        Es_Agent::init();
        Es_Shortcodes::init();
        Es_Migration_Page::init();
        Es_Manage_Agents::init();
        Es_FBuilder::init();
    }

    /**
     * Install action. Triggering on plugin activation.
     *
     * @return void
     */
    public static function install()
    {
        global $wpdb;

        $table_name = $wpdb->prefix . 'address_components';
        $charset_collate = '';

        if ( ! empty ( $wpdb->charset ) )
            $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";

        if ( ! empty ( $wpdb->collate ) )
            $charset_collate .= " COLLATE $wpdb->collate";

        $sql = 'CREATE TABLE IF NOT EXISTS ' . $table_name . ' (
            id int(11) NOT NULL AUTO_INCREMENT,
            long_name VARCHAR(255),
            short_name VARCHAR(255),
            `type` VARCHAR(255) NOT NULL,
            `locale` VARCHAR(10),
            PRIMARY KEY (id)
        ); ' . $charset_collate . ';';

        register_taxonomy( 'es_labels', Es_Property::get_post_type_name() );

        $labels = es_get_standard_label_names();

        if ( taxonomy_exists( 'es_labels' ) && ! empty( $labels ) ) {
            foreach ( $labels as $color => $label ) {
                if ( ! term_exists( $label ) ) {
                    $term = wp_insert_term( $label, 'es_labels' );
                    if ( ! empty( $term['term_id'] ) ) {
                        update_term_meta( $term['term_id'], 'es_color', $color );
                    }
                }
            }
        }

        update_option( 'es_thumbnail_attachment_id', '' );

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );

        // If plugin was not activated before.
        if ( ! self::was_activated() ) {

            // Get roles and capabilities for manage entities.
            if ( $caps = self::get_roles_capabilities() ) {
                foreach ( $caps as $role_name => $capabilities ) {
                    $role = get_role( $role_name );

                    // If agent role isn't created then create it.
                    if ( ! $role && $role_name == Es_Agent::get_role_name() ) {
                        $role = add_role( $role_name, __( 'Estatik Agent', 'es-plugin' ) );
                    }

                    // Set capabilities for role.
                    foreach ( $capabilities as $capability ) {
                        if ( ! $role->has_cap( $capability ) ) {
                            $role->add_cap( $capability );
                        }
                    }
                }
            }
        }

        // Set option that plugin is activated.
        self::set_activated();

        do_action( 'es_plugin_after_install' );
    }

    /**
     * @return mixed
     */
    public static function get_roles_capabilities()
    {
        $caps = array(
            'administrator' => array(
                'es_manage_properties',
                'read_es_property',
                'edit_es_property',
                'create_es_properties',
                'edit_es_properties',
                'edit_published_es_properties',
                'delete_published_es_properties',
                'delete_es_property',
                'publish_es_properties',
                'edit_others_es_properties',
                'read_private_es_properties',
                'delete_es_properties',
                'delete_private_es_properties',
                'delete_others_es_properties',
                'edit_private_es_properties',
                'es_manage_agent_subscription_plans',
                'es_manage_dashboard',
            ),
            Es_Agent::get_role_name() => array(
                'es_manage_properties',
                'create_es_properties',
                'publish_es_properties',
                'read_es_property',
                'edit_es_property',
                'edit_es_properties',
                'edit_published_es_properties',
                'delete_published_es_properties',
                'delete_es_property',
                'read',
                'delete_es_properties',
                'upload_files',
                'level_2',
            )
        );

        $roles = array_keys( $caps );

        if ( $taxonomies = Es_Taxonomy::get_taxonomies_list() ) {
            foreach ( $taxonomies as $taxonomy ) {
                $taxonomy = new Es_Taxonomy( $taxonomy );
                foreach ( $roles as $role ) {
                    $caps[ $role ] = array_merge( $caps[ $role ], array_values( $taxonomy->get_default_capabilities() ) );

                    if ( $role == Es_Agent::get_role_name() ) {
                        unset(
                            $caps[ $role ][ $taxonomy->get_capability_name( 'edit_terms' ) ],
                            $caps[ $role ][ $taxonomy->get_capability_name( 'delete_terms' ) ]
                        );
                    }
                }
            }
        }

        return apply_filters( 'es_get_roles_capabilities', $caps );
    }

    /**
     * Load plugin files.
     *
     * @return void
     */
    public static function class_loader()
    {
        $files = apply_filters( 'es_class_loader', array(
            ES_PLUGIN_PATH . '/classes/class-object.php',
            ES_PLUGIN_PATH . '/admin/classes/class-search-location.php',
            ES_PLUGIN_PATH . '/admin/classes/class-property-metabox.php',
            ES_PLUGIN_PATH . '/classes/class-post-duplicate.php',
            ES_PLUGIN_PATH . '/classes/class-archive-sorting.php',
            ES_PLUGIN_PATH . '/classes/class-entity.php',
            ES_PLUGIN_PATH . '/classes/class-repository.php',
            ES_PLUGIN_PATH . '/classes/pages/class-property-single-page.php',
            ES_PLUGIN_PATH . '/classes/pages/class-property-archive-page.php',
            ES_PLUGIN_PATH . '/classes/class-manage-agents.php',

            ES_PLUGIN_PATH . '/management/classes/class-prop-management-init.php',

            ES_PLUGIN_PATH . '/classes/shortcodes/class-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-my-listing-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-featured-props-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-latest-props-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-cheapest-props-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-expensive-props-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-category-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-property-map-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-agent-register-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-agent-list-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-property-slideshow-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-location-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-state-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-country-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-city-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-agent-listing-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-labels-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-features-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-single-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-login-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-restore-password-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-search-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-map-search-shortcode.php',
            ES_PLUGIN_PATH . '/management/classes/shortcodes/class-prop-management-shortcode.php',
            ES_PLUGIN_PATH . '/classes/shortcodes/class-search-form-shortcode.php',
            ES_PLUGIN_PATH . '/classes/class-shortcodes.php',
            ES_PLUGIN_PATH . '/classes/class-session-storage.php',
            ES_PLUGIN_PATH . '/admin/interfaces/es-messenger-interface.php',
            ES_PLUGIN_PATH . '/admin/classes/widgets/class-widget.php',
            ES_PLUGIN_PATH . '/admin/classes/widgets/class-search-widget.php',
            ES_PLUGIN_PATH . '/admin/classes/widgets/class-request-widget.php',
            ES_PLUGIN_PATH . '/admin/classes/widgets/class-property-map-widget.php',
            ES_PLUGIN_PATH . '/admin/classes/widgets/class-property-slideshow-widget.php',
            ES_PLUGIN_PATH . '/admin/classes/class-data-manager-item.php',
            ES_PLUGIN_PATH . '/admin/classes/class-taxonomy.php',
            ES_PLUGIN_PATH . '/admin/classes/class-data-manager-term-item.php',
            ES_PLUGIN_PATH . '/admin/classes/class-data-manager-label-item.php',
            ES_PLUGIN_PATH . '/admin/classes/class-data-manager-currency-item.php',
            ES_PLUGIN_PATH . '/admin/classes/class-estatik-upgrade.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-dashboard-page.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-settings-page.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-data-manager-page.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-fields-builder-page.php',
            ES_PLUGIN_PATH . '/admin/classes/migration/class-property-migration.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-migration-page.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-property-list-page.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-upgrade-pro-page.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-agent-list.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-agent-profile-page.php',
            ES_PLUGIN_PATH . '/admin/classes/pages/class-import-page.php',
            ES_PLUGIN_PATH . '/admin/classes/class-messenger.php',
            ES_PLUGIN_PATH . '/admin/classes/class-fbuilder-helper.php',
            ES_PLUGIN_PATH . '/admin/classes/class-fbuilder.php',
            ES_PLUGIN_PATH . '/classes/class-settings-container.php',
            ES_PLUGIN_PATH . '/classes/class-html-helper.php',
            ES_PLUGIN_PATH . '/classes/class-user.php',
            ES_PLUGIN_PATH . '/classes/class-post.php',
            ES_PLUGIN_PATH . '/classes/class-agent.php',
            ES_PLUGIN_PATH . '/classes/class-property.php',
            ES_PLUGIN_PATH . '/classes/class-address-components.php',
            ES_PLUGIN_PATH . '/classes/class-template-loader.php',
            ES_PLUGIN_PATH . '/subscriptions/classes/class-subscription-init.php',
            ES_PLUGIN_PATH . '/functions.php',
        ) );

        foreach ( $files as $file ) {
            if ( file_exists( $file ) ) {
                include $file;
            }
        }
    }

    /**
     * Enqueue scripts for frontend part.
     *
     * @return void
     */
    public function enqueue_scripts()
    {
        $custom = 'assets/js/custom/';

        $adminVendor = 'admin/assets/js/vendor/';
        $vendor = 'assets/js/vendor/';

        global $es_settings;

        $language = es_get_locale();

		wp_register_script( 'es-rating-admin-script', ES_PLUGIN_URL . $adminVendor . 'jquery.star-rating-svg.js', array ( 'jquery' ) );

        wp_register_script( 'es-select2-script', ES_PLUGIN_URL . $adminVendor . 'select2.min.js', array ( 'jquery' ) );

        // Base front script.
        wp_register_script(
            'es-front-script',
            ES_PLUGIN_URL . $custom . 'front.js',
            array ( 'jquery', 'es-dropdown-script', 'es-select2-script', 'jquery-ui-datepicker' )
        );

        // Custom dropDown script.
	    wp_register_script(
	        'es-dropdown-script',
            ES_PLUGIN_URL . $custom . 'es-dropdown.js',
            array ( 'jquery' )
        );

        if ( $es_settings->google_api_key ) {
            // Google map wrapper.
            wp_register_script(
                'es-admin-map-script',
                ES_PLUGIN_URL . $custom . 'map.js',
                array( 'es-admin-googlemap-api' )
            );

            // Google map API.
            wp_register_script(
                'es-admin-googlemap-api',
                'https://maps.googleapis.com/maps/api/js?key=' . $es_settings->google_api_key . '&libraries=places&language=' . $language,
                array(),
                false,
                true
            );

            wp_register_script(
                'es-map-tab-script',
                ES_PLUGIN_URL . $custom . 'map-tab.js',
                array ( 'jquery', 'es-admin-map-script' )
            );

            wp_enqueue_script( 'es-map-tab-script' );
        }

        wp_register_script( 'es-magnific-script', ES_PLUGIN_URL . $vendor . 'jquery.magnific-popup.min.js', array ( 'jquery' ) );

        // Script for dependent location dropDowns in search widget.
        wp_register_script(
            'es-front-locations-script',
            ES_PLUGIN_URL . $custom . 'locations.js',
            array ( 'jquery', 'es-dropdown-script' )
        );

		wp_register_script( 'es-slick-script', ES_PLUGIN_URL . $adminVendor . 'slick.min.js', array ( 'jquery' ) );

        wp_enqueue_script( 'es-front-locations-script' );

        // Share plugin JS variables for locations script.
        wp_localize_script( 'es-front-locations-script', 'Estatik', static::register_js_variables() );
    }

    /**
     * Enqueue styles for frontend part.
     *
     * @return void
     */
    public function enqueue_styles()
    {
        $vendor = 'assets/css/vendor/';
        $custom = 'assets/css/custom/';
		$admin_vendor = 'admin/assets/css/vendor/';

        wp_register_style( 'es-select2-style', ES_PLUGIN_URL . $admin_vendor . 'select2.min.css' );
        wp_enqueue_style( 'es-select2-style' );

        // Register base styles for the plugin.
        wp_register_style( 'es-front-style', ES_PLUGIN_URL . $custom . 'front.css' );
        wp_enqueue_style( 'es-front-style' );

        // Register font awesome.
        wp_register_style( 'es-font-awesome', ES_PLUGIN_URL . $vendor . 'font-awesome.min.css' );
        wp_enqueue_style( 'es-font-awesome' );


        wp_register_style('jquery-ui', '//code.jquery.com/ui/1.12.1/themes/base/jquery-ui.css');
        wp_enqueue_style( 'jquery-ui' );

		wp_register_style( 'es-slick-style', ES_PLUGIN_URL . $admin_vendor . 'slick.css' );

        // Register base styles for the plugin.
        wp_register_style( 'es-magnific-style', ES_PLUGIN_URL . $vendor . 'magnific-popup.css' );
        wp_enqueue_style( 'es-magnific-style' );


        wp_register_style(
            'es-google-open-sans-form',
            'https://fonts.googleapis.com/css?family=Open+Sans:300,400" rel="stylesheet'
        );

        wp_enqueue_style( 'es-google-open-sans-form' );
    }

    /**
     * Enqueue admin scripts.
     *
     * @return void
     */
    public function admin_enqueue_scripts()
    {
        global $es_settings;

        $custom = 'admin/assets/js/custom/';
        $vendor = 'admin/assets/js/vendor/';
        $custom_main = 'assets/js/custom/';

        $language = es_get_locale();

        wp_register_script(
            'es-admin-scroll-script', ES_PLUGIN_URL . $vendor . 'jquery.mCustomScrollbar.concat.min.js',
            array( 'jquery' ),
            false,
            $in_footer = false
        );

        wp_register_script(
            'es-data-manager-script', ES_PLUGIN_URL . $custom . 'jquery-data-manager.js',
            array ( 'jquery', 'es-popup-script' )
        );

        wp_register_script(
            'es-cloneya-script', ES_PLUGIN_URL . $vendor . 'jquery-cloneya.min.js',
            array ( 'jquery' )
        );

        wp_register_script(
            'es-datetime-picker', ES_PLUGIN_URL . $vendor . 'jquery.datetimepicker.min.js',
            array ( 'jquery' )
        );

        wp_register_script( 'es-admin-script', ES_PLUGIN_URL . $custom . 'admin.js', array (
            'jquery', 'es-popup-script', 'es-data-manager-script', 'es-slick-admin-script', 'jquery-ui-tabs',
            'es-admin-scroll-script', 'es-admin-map-script', 'es-rating-admin-script', 'es-select2-script', 'jquery-ui-sortable',
            'es-datetime-picker',
        ) );

        wp_enqueue_script( 'es-admin-script' );

        wp_register_script( 'es-select2-script', ES_PLUGIN_URL . $vendor . 'select2.min.js', array ( 'jquery' ) );
        wp_register_script( 'es-progress-script', ES_PLUGIN_URL . $vendor . 'jquery.progress.js', array ( 'jquery' ) );

        wp_register_script( 'es-checkbox-script', ES_PLUGIN_URL . $custom . 'es-checkboxes.js', array ( 'jquery' ) );
        wp_enqueue_script( 'es-checkbox-script' );

        wp_register_script( 'es-tooltipster-script', ES_PLUGIN_URL . $vendor . 'tooltipster.bundle.min.js', array ( 'jquery' ) );
        wp_enqueue_script( 'es-tooltipster-script' );

        wp_register_script( 'es-popup-script', ES_PLUGIN_URL . $custom . 'es-popup.js', array ( 'jquery' ) );

        // Register slider for dashboard page.
        wp_register_script( 'es-slick-admin-script', ES_PLUGIN_URL . $vendor . 'slick.min.js', array ( 'jquery' ) );
        wp_register_script( 'es-rating-admin-script', ES_PLUGIN_URL . $vendor . 'jquery.star-rating-svg.js', array ( 'jquery' ) );

        wp_localize_script( 'es-admin-script', 'Estatik', static::register_js_variables() );

        wp_register_script(
            'es-admin-map-script', ES_PLUGIN_URL . $custom_main . 'map.js',
            array( 'es-admin-googlemap-api' ),
            false
        );

        wp_register_script(
            'es-admin-googlemap-api',
            'https://maps.googleapis.com/maps/api/js?key=' . $es_settings->google_api_key . '&libraries=places&language='.$language,
            array(),
            false
        );

        wp_register_script(
            'es-admin-map-tab-script',
            ES_PLUGIN_URL . $custom_main . 'map-tab.js',
            array ( 'jquery', 'es-admin-map-script' )
        );

        wp_enqueue_script( 'es-admin-map-tab-script' );
    }

    /**
     * Enqueue admin styles.
     *
     * @return void
     */
    public function admin_enqueue_styles()
    {
        $vendor = 'admin/assets/css/vendor/';
        $vendor_main = 'assets/css/vendor/';
        $custom = 'admin/assets/css/custom/';

        wp_register_style( 'es-tooltipster-style', ES_PLUGIN_URL . $vendor . 'tooltipster.bundle.min.css' );
        wp_enqueue_style( 'es-tooltipster-style' );

        wp_register_style( 'es-tooltipster-theme-style', ES_PLUGIN_URL . $vendor . 'tooltipster-sideTip-borderless.min.css' );
        wp_enqueue_style( 'es-tooltipster-theme-style' );

        wp_register_style( 'es-select2-style', ES_PLUGIN_URL . $vendor . 'select2.min.css' );
        wp_enqueue_style( 'es-select2-style' );

        wp_register_style( 'es-rating-style', ES_PLUGIN_URL . $vendor . 'star-rating-svg.css' );
        wp_enqueue_style( 'es-rating-style' );

        wp_register_style( 'es-font-awesome', ES_PLUGIN_URL . $vendor_main . 'font-awesome.min.css' );
        wp_enqueue_style( 'es-font-awesome' );

        wp_register_style( 'es-datetime-picker-css', ES_PLUGIN_URL . $vendor . 'jquery.datetimepicker.css' );
        wp_enqueue_style( 'es-datetime-picker-css' );

        wp_register_style( 'jquery-ui', '//ajax.googleapis.com/ajax/libs/jqueryui/1.8/themes/base/jquery-ui.css' );

        wp_register_style( 'es-admin-style', ES_PLUGIN_URL . $custom . 'admin.css' );
        wp_enqueue_style( 'es-admin-style' );

        wp_register_style( 'es-checkboxes-style', ES_PLUGIN_URL . $custom . 'es-checkboxes.css' );
        wp_enqueue_style( 'es-checkboxes-style' );

        wp_register_style( 'es-popup-style', ES_PLUGIN_URL . $custom . 'es-popup.css' );
        wp_enqueue_style( 'es-popup-style' );

        wp_register_style(
            'es-google-open-sans-form',
            'https://fonts.googleapis.com/css?family=Open+Sans:300,400" rel="stylesheet'
        );

        wp_enqueue_style( 'es-google-open-sans-form' );
    }

    /**
     * Register global javascript
     *
     * @return array
     */
    public static function register_js_variables()
    {
        global $es_settings;

	    if ( empty( $es_settings ) ) {
		    $es_settings = new Es_Settings_Container();
	    }

        return apply_filters( 'es_global_js_variables', array(
            'tr' => array(
                'remove_image' => __( 'Remove image', 'es-plugin' ),
                'remove' => __( 'Remove', 'es-plugin' ),
                'yes' => __( 'Yes', 'es-plugin' ),
                'no' => __( 'No', 'es-plugin' ),
                'ok' => __( 'Ok', 'es-plugin' ),
                'select_location' => __( 'Select location', 'es-plugin' ),
                'sorting' => __( 'Sort by', 'es-plugin' ),
                'confirmDeleting' => __( 'Are you sure you want to delete this item?', 'es-plugin' ),
                'retsAvailable' => sprintf( __( 'The RETS integration feature <br>is available in %s version.', 'es-plugin' ),
                    '<a target="_blank" href="https://estatik.net/product/estatik-premium-rets/">' . __( 'Premium', 'es-plugin' ) . '</a>' ),
                'color' => __( 'Color', 'es-plugin' ),
                'icons' => __( 'Icons', 'es-plugin' ),
                'replace_photo' => __( 'Replace photo', 'es-plugin' ),
            ),
            'settings' => array(
                'pluginUrl' => ES_PLUGIN_URL,
                'layout' => $es_settings->listing_layout,
                'dateFormat' => $es_settings->date_format,
                'map_zoom' => $es_settings->map_zoom,
                'dateTimeFormat' => $es_settings->date_format . ' H:i',
                'responsive' => array(
                    'es-layout-list' => array( 'min' => 660, 'max' =>  999999 ),
                    'es-layout-2_col' => array( 'max' => 640, 'min' =>  0 ),
                    'es-layout-3_col' => array( 'min' => 620, 'max' => 660 ),
                ),
                'markers' => $es_settings::get_setting_values( 'markers' ),
                'default_marker' => array(
                    'color' => $es_settings->marker_color,
                    'marker' => $es_settings->get_label( 'markers', $es_settings->markers )
                ),
            ),
            'widgets' => array(
                'search' => array(
                    'initPriority' => array(
                        Es_Search_Location::LOCATION_COUNTRY_TYPE => array(
                            Es_Search_Location::LOCATION_STATE_TYPE
                        ),

                        Es_Search_Location::LOCATION_STATE_TYPE   => array(
                            Es_Search_Location::LOCATION_CITY_TYPE,
                        ),

                        Es_Search_Location::LOCATION_CITY_TYPE    => array(
                            Es_Search_Location::LOCATION_STREET_TYPE,
                            Es_Search_Location::LOCATION_NEIGHBORHOOD_TYPE,
                        ),

                        Es_Search_Location::LOCATION_STREET_TYPE  => array(
                            Es_Search_Location::LOCATION_NEIGHBORHOOD_TYPE,
                        ),

                        Es_Search_Location::LOCATION_NEIGHBORHOOD_TYPE => array(),
                    ),
                )
            ),
            'ajaxurl' => admin_url( 'admin-ajax.php' ),
        ) );
    }

    /**
     * @param $user
     * @return WP_Error
     */
    public function authenticate( $user )
    {
        if ( $user && ! is_wp_error( $user ) && in_array( Es_Agent::get_role_name(), $user->roles ) ) {
            $agent = es_get_agent( $user->ID );

            if ( $agent->status != Es_Agent::STATUS_ACTIVE ) {
                $user = new WP_Error( 'es_deactivated_user', __( 'Your account is not approved, please contact admin of this website to get approved.', 'es-plugin' ) );
            }
        }

        return $user;
    }

    /**
     * @return void
     */
    public function login()
    {
        $nonce = 'es-login';

        if ( ! empty( $_REQUEST[ $nonce ] ) && wp_verify_nonce( $_REQUEST[ $nonce ], $nonce ) ) {
            global $user;
            $messenger = new Es_Messenger( 'login' );
            $creds = array();
            $creds['user_login'] = $_POST['log'];
            $creds['user_password'] =  $_POST['pwd'];
            $creds['remember'] = true;
            $user = wp_signon( $creds, is_ssl() );
            if ( is_wp_error( $user ) ) {
                $messenger->set_message( $user->get_error_message(), 'error' );
                wp_redirect( ! empty( $_POST['redirect'] ) ? $_POST['redirect'] : $_POST['_wp_http_referer'] ); die;
            }
            if ( ! is_wp_error( $user ) ) {
                $agent = es_get_agent( $user->ID );

                if ( in_array( $agent::get_role_name(), $user->roles ) && $agent::STATUS_DISABLED == $agent->status ) {
                    wp_logout();
                    $messenger->clean_container();
                    $messenger->set_message( __( 'Account disabled.', 'es-plugin' ), 'error' );
                    wp_redirect( $_POST['redirect'] ); die;
                } else {
                    wp_redirect( ! empty( $_POST['redirect_to'] ) && $user->has_cap( 'delete_es_properties' )
                        ? $_POST['redirect_to'] : home_url() ); die;
                }
            }
        }
    }

    /**
     * Register admin pages.
     */
    public function register_admin_pages()
    {
        $imagesPath = 'admin/assets/images/';

        $parent = 'es_dashboard';
        $function = array( 'Es_Dashboard_Page', 'render' );

        add_menu_page(
            __( 'Estatik', 'es-plugin' ),
            __( 'Estatik', 'es-plugin' ),
            'es_manage_dashboard',
            $parent,
            $function,
            ES_PLUGIN_URL . $imagesPath .'es_menu_icon.png',
            '20.7'
        );

        add_submenu_page(
            'es_dashboard',
            __( 'Dashboard', 'es-plugin' ),
            __( 'Dashboard', 'es-plugin' ),
            'es_manage_dashboard',
            'es_dashboard',
            array( 'Es_Dashboard_Page', 'render' )
        );

        add_submenu_page(
            $parent,
            __( 'My listings', 'es-plugin' ),
            __( 'My listings', 'es-plugin' ),
            'es_manage_properties',
            es_admin_property_list_uri()
        );

        add_submenu_page(
            $parent,
            __( 'Add new property', 'es-plugin' ),
            __( 'Add new property', 'es-plugin' ),
            'es_manage_dashboard',
            es_admin_property_add_uri()
        );

        add_submenu_page(
            'es_dashboard',
            __( 'Data Manager', 'es-plugin' ),
            __( 'Data Manager', 'es-plugin' ),
            'manage_options',
            'es_data_manager',
            array( 'Es_Data_Manager_Page', 'render' )
        );

        add_submenu_page(
            'es_dashboard',
            __( 'Agents', 'es-plugin' ),
            __( 'Agents', 'es-plugin' ),
            'manage_options',
            es_admin_agents_uri() );

        add_submenu_page(
            'es_dashboard',
            __( 'Add New Agent', 'es-plugin' ),
            __( 'Add New Agent', 'es-plugin' ),
            'manage_options',
            'es_agent',
            array( 'Es_Agent_Profile', 'render' )
        );

        add_submenu_page(
            'es_dashboard',
            __( 'CSV Import', 'es-plugin' ),
            __( 'CSV Import', 'es-plugin' ),
            'manage_options',
            'es_import',
            array( 'Es_Import_Page', 'render' )
        );

        if ( es_subscription_is_enabled() ) {
            add_submenu_page(
                'es_dashboard',
                __( 'Subscriptions', 'es-plugin' ),
                __( 'Subscriptions', 'es-plugin' ),
                'manage_options',
                'edit.php?post_type=es_subscription'
            );

            add_submenu_page(
                'es_dashboard',
                __( 'Add subscription', 'es-plugin' ),
                __( 'Add subscription', 'es-plugin' ),
                'manage_options',
                'post-new.php?post_type=es_subscription'
            );

            add_submenu_page(
                'es_dashboard',
                __( 'Orders', 'es-plugin' ),
                __( 'Orders', 'es-plugin' ),
                'manage_options',
                'edit.php?post_type=es_order'
            );
        }

        add_submenu_page(
            'es_dashboard',
            __( 'Fields builder', 'es-plugin' ),
            __( 'Fields builder', 'es-plugin' ),
            'manage_options',
            'es_fbuilder',
            array( 'Es_Fields_Builder_Page', 'render' )
        );

        add_submenu_page(
            'es_dashboard',
            __( 'Settings', 'es-plugin' ),
            __( 'Settings', 'es-plugin' ),
            'manage_options',
            'es_settings',
            array( 'Es_Settings_Page', 'render' )
        );

        if ( ! es_migration_already_executed() && es_need_migrate() ) {
            add_submenu_page(
                'es_dashboard',
                __( 'Migration', 'es-plugin' ),
                __( 'Migration', 'es-plugin' ),
                'manage_options',
                'es_migration',
                array( 'Es_Migration_Page', 'render' )
            );
        }
    }

	/**
	 * Display powered by phrase.
     *
     * @return void
	 */
	public function powered() {
		global $es_settings;

		if ( $es_settings->powered_by_link ) {
            $template = apply_filters( 'es_powered_template', ES_TEMPLATES . 'powered.php' );
			include( $template );
		}
	}

    /**
     * Initialize plugin global vars.
     *
     * @return void
     */
    public function set_global_vars()
    {
        global $post, $es_settings, $es_property;

        // Set global plugin settings array.
        $es_settings = new Es_Settings_Container();

        // Set global property object.
        if ( ! empty( $post->post_type ) && Es_Property::get_post_type_name() == $post->post_type && is_single() ) {
            $es_property = es_get_property( $post->ID );
        }
    }

    /**
     * Register plugin post types.
     *
     * @return void
     */
    public function register_post_types()
    {
        $args = array(
            'label' => __( 'Property', 'es-plugin' ),
            'labels' => array(
                'name' => __( 'My listings', 'es-plugin' ),
            ),
            'public' => true,
            'show_in_menu' => false,
            'has_archive' => true,
            'supports' => array( 'title', 'editor', 'author', 'excerpt' ),
            'rewrite' => array(
                'slug' => 'property',
            ),
            'capability_type'     => array( 'es_property','es_properties' ),
            'capabilities' => array(
                'create_posts' => 'create_es_properties',
            ),
            'map_meta_cap' => true
        );

        register_post_type( Es_Property::get_post_type_name(), $args );

        flush_rewrite_rules();
    }

    /**
     * Register plugin taxonomies.
     *
     * @return void
     */
    public function register_taxonomies()
    {
        $tax = new Es_Taxonomy( 'es_category' );

        register_taxonomy( 'es_category', Es_Property::get_post_type_name(), apply_filters( 'es_category_taxonomy_args', array(
            'labels' => array(
                'name' => __( 'Category', 'es-plugin' ),
                'singular_name' => __( 'Category', 'es-plugin' ),
            ),
            'capabilities' => $tax->get_default_capabilities(),
        ) ) );

        $tax = new Es_Taxonomy( 'es_status' );

        register_taxonomy( 'es_status', Es_Property::get_post_type_name(), apply_filters( 'es_status_taxonomy_args', array(
            'labels' => array(
                'name' => __( 'Status', 'es-plugin' ),
                'singular_name' => __( 'Status', 'es-plugin' ),
            ),
            'capabilities' => $tax->get_default_capabilities(),
        ) ) );

        $tax = new Es_Taxonomy( 'es_type' );

        register_taxonomy( 'es_type', Es_Property::get_post_type_name(), apply_filters( 'es_type_taxonomy_args', array(
            'labels' => array(
                'name' => __( 'Type', 'es-plugin' ),
                'singular_name' => __( 'Type', 'es-plugin' ),
            ),
            'capabilities' => $tax->get_default_capabilities(),
        ) ) );

        $tax = new Es_Taxonomy( 'es_feature' );

        register_taxonomy( 'es_feature', Es_Property::get_post_type_name(), apply_filters( 'es_feature_taxonomy_args', array(
            'labels' => array(
                'name' => __( 'Features', 'es-plugin' ),
                'singular_name' => __( 'Feature', 'es-plugin' ),
            ),
            'capabilities' => $tax->get_default_capabilities(),
        ) ) );

        $tax = new Es_Taxonomy( 'es_rent_period' );

        register_taxonomy( 'es_rent_period', Es_Property::get_post_type_name(), apply_filters( 'es_rent_period_args', array(
            'labels' => array(
                'name' => __( 'Rent period', 'es-plugin' ),
                'singular_name' => __( 'Rent period', 'es-plugin' ),
            ),
            'capabilities' => $tax->get_default_capabilities(),
        ) ) );

        $tax = new Es_Taxonomy( 'es_amenities' );

        register_taxonomy( 'es_amenities', Es_Property::get_post_type_name(), apply_filters( 'es_amenities_taxonomy_args', array(
            'labels' => array(
                'name' => __( 'Amenities', 'es-plugin' ),
                'singular_name' => __( 'Amenities', 'es-plugin' ),
            ),
            'capabilities' => $tax->get_default_capabilities(),
        ) ) );

        $tax = new Es_Taxonomy( 'es_labels' );

        register_taxonomy( 'es_labels', Es_Property::get_post_type_name(), apply_filters( 'es_labels_args', array(
            'labels' => array(
                'name' => __( 'Labels', 'es-plugin' ),
                'singular_name' => __( 'Label', 'es-plugin' ),
            ),
            'show_ui' => false,
            'capabilities' => $tax->get_default_capabilities(),
        ) ) );
    }

    /**
     * Disable admin access for agent role.
     *
     * @return void
     */
    function admin_restrict_access() {
        if( defined( 'DOING_AJAX' ) && DOING_AJAX ) return;

        $user = wp_get_current_user();

        if ( $user instanceof WP_User && in_array( Es_Agent::get_role_name(), $user->roles ) ) {
            wp_die( __( 'Sorry, you are not allowed to access this page.' ) );
        }
    }

    /**
     * Hide admin bar for agents.
     *
     * @return void
     */
    function admin_bar_access() {
        if( defined( 'DOING_AJAX' ) && DOING_AJAX ) return;

        $user = wp_get_current_user();

        if ( ! empty( $user->roles ) && in_array( Es_Agent::get_role_name(), $user->roles ) ) {
            show_admin_bar( false );
        }
    }

    /**
     * Get is plugin was activated before.
     *
     * @return mixed
     */
    public static function was_activated()
    {
        return get_option( 'es_plugin_was_activated', 0 );
    }

    /**
     * Set plugin as activated and set plugin version via wp options.
     *
     * @return void
     */
    public static function set_activated()
    {
        update_option( 'es_plugin_was_activated', 1 );
        update_option( 'es_plugin_version', self::getVersion() );
    }

    /**
     * Customize admin logo.
     *
     * @return void
     */
    public static function es_admin_logo()
    {
        global $es_settings;

        if ( $es_settings->logo_attachment_id && ( $image = wp_get_attachment_image_url( $es_settings->logo_attachment_id, 'full') ) ) : ?>

            <style type="text/css">
                body.login div#login h1 a {
                    background-image: url(<?php echo $image; ?>);
                    background-size: 100% auto;
                    background-position: center center;
                    width: 100%;
                    height: 100%;
                }

                body.login div#login h1 {
                    width: <?php echo $es_settings->logo_attachment_width; ?>;
                    height: <?php echo $es_settings->logo_attachment_height; ?>;
                    margin: 0 auto;
                }
            </style>

        <?php endif;
    }

    /**
     * Migrate new settings for estatik plugin.
     *
     * @return void
     */
    public function _migration()
    {
        global $wpdb;

        $charset_collate = '';

        if ( ! empty ( $wpdb->charset ) )
            $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";

        if ( ! empty ( $wpdb->collate ) )
            $charset_collate .= " COLLATE $wpdb->collate";

        if ( ! function_exists( 'dbDelta' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        }

        /**
         * START MIGRATIONS FLOW.
         */

        if ( ! get_option( 'es_migration_0' ) ) {
	        // An array of Field names
	        $existing_columns = $wpdb->get_col("DESC {$wpdb->prefix}address_components", 0);

			if ( ! empty( $existing_columns ) ) {
				if ( ! in_array( 'locale', $existing_columns ) ) {
					$wpdb->query( "ALTER TABLE {$wpdb->prefix}address_components ADD locale VARCHAR(10)" );
				}

				update_option( 'es_migration_0', true );
			}
        }

        if ( ! get_option( 'es_migration_1' ) ) {
            $wpdb->query( "UPDATE {$wpdb->prefix}address_components SET locale='" . es_get_locale() . "'" );

            // Resave address components.
            $type =  Es_Property::get_post_type_name();
            $posts_to_update = get_posts( array('post_type' => $type, 'posts_per_page' => -1 ) );
            foreach ( $posts_to_update as $update_this_post ){
                $property = es_get_property( $update_this_post->ID );
                $address_data = ES_Address_Components::get_google_components( $property->address );
                if ( ! empty( $address_data->status  ) && $address_data->status == 'OK' ) {
                    $res = $address_data->results[0];
                    if ( ! empty( $res->address_components ) ) {
                        $save_data['address_components'] = json_encode( $res->address_components );
                    }
                }

                if ( ! empty( $save_data['address_components'] ) ) {
                    $data =  json_decode( $save_data['address_components'] );
                    $property->save_address_components( $data );
                    if ( ! empty( $data ) && is_array( $data ) ) {
                        foreach ( $data as $item ) {
                            ES_Address_Components::save_component( $item );
                        }
                    }
                }
            }
            update_option( 'es_migration_1', true );
        }

        if ( ! get_option( 'es_migration_5' ) ) {

            $sql = 'CREATE TABLE IF NOT EXISTS ' . $wpdb->prefix . 'fbuilder_fields (
                `id` INT(11) NOT NULL AUTO_INCREMENT,
                `label` VARCHAR(255) NOT NULL,
                `machine_name` VARCHAR(255) NOT NULL,
                `options` TEXT,
                `type` VARCHAR(20) NOT NULL,
                `tab` VARCHAR(40) NOT NULL,
                `entity` VARCHAR(255) NOT NULL,
                `formatter` VARCHAR(255),
                `values` TEXT,
                `section` TEXT,
                `rets_support` INT(1),
                `search_support` INT(1),
                PRIMARY KEY (id)
            ); ' . $charset_collate . ';';

            dbDelta( $sql );

            update_option( 'es_migration_5', true );
        }

        if ( ! get_option( 'es_migration_3' ) ) {

            $sql = 'CREATE TABLE IF NOT EXISTS ' . $wpdb->prefix . 'fbuilder_fields_order (
                `id` INT(11) NOT NULL AUTO_INCREMENT,
                `section_machine_name` VARCHAR(255) NOT NULL,
                `field_machine_name` VARCHAR(255) NOT NULL,
                `order` INT(11),
                PRIMARY KEY (id)
            ); ' . $charset_collate . ';';

            dbDelta( $sql );

            $sql = 'CREATE TABLE IF NOT EXISTS ' . $wpdb->prefix . 'fbuilder_sections (
                `id` INT(11) NOT NULL AUTO_INCREMENT,
                `label` VARCHAR(255) NOT NULL,
                `machine_name` VARCHAR(255) NOT NULL,
                PRIMARY KEY (id)
            ); ' . $charset_collate . ';';

            dbDelta( $sql );

            $sql = 'CREATE TABLE IF NOT EXISTS ' . $wpdb->prefix . 'fbuilder_sections_order (
                `id` INT(11) NOT NULL AUTO_INCREMENT,
                `section_machine_name` VARCHAR(255) NOT NULL,
                `order` INT(11),
                PRIMARY KEY (id)
            ); ' . $charset_collate . ';';

            dbDelta( $sql );

            update_option( 'es_migration_3', true );
        }

        if ( ! get_option( 'es_migration_6' ) ) {
            $wpdb->query( "ALTER TABLE {$wpdb->prefix}fbuilder_fields ADD search_range_mode INT(1)" );
            $wpdb->query( "ALTER TABLE {$wpdb->prefix}fbuilder_fields ADD show_thumbnail INT(1)" );
            $wpdb->query( "ALTER TABLE {$wpdb->prefix}fbuilder_sections ADD show_tab INT(1)" );
            update_option( 'es_migration_6', true );
        }

        if ( ! get_option( 'es_migration_7634' ) ) {
            $wpdb->query("ALTER TABLE {$wpdb->prefix}fbuilder_fields ADD import_support INT(1)");
            update_option( 'es_migration_7634', true );
        }

        if ( ! get_option( 'es_migration_32451' ) ) {
            $agent = es_get_agent();

            $users = get_users( array(
                'role' => $agent::get_role_name(),
            ) );

            if ( $users ) {
                /** @var WP_User $user */
                foreach ( $users as $user ) {
                    $agent = es_get_agent( $user->ID );
                    $agent->save_field_value( 'status', $agent::STATUS_ACTIVE );
                }
            }

            update_option( 'es_migration_32451', true );
        }

        if ( ! get_option( 'es_migration_32452345865' ) ) {
            $agents = es_get_users_list( array(
                'role' => Es_Agent::get_role_name(),
            ) );

            if ( $agents ) {
                foreach ( $agents as $id => $agent ) {
	                $query = new WP_Query( array(
		                'post_type' => Es_Property::get_post_type_name(),
		                'posts_per_page' => -1,
		                'fields' => array( 'ids' ) ,
	                    'author' => $id,
                    ) );

	                $posts = $query->get_posts();
	                $count = count( $posts );
	                $agent = es_get_agent( $id );
	                $agent->save_field_value( 'property_qty', $count );
                }
            }
	        update_option( 'es_migration_32452345865', true );
        }

	    /**
	     * Field builder fix.
	     */
        if ( ! get_option( 'es_migration_fb_fix_1' ) ) {
            $fields = Es_FBuilder_Helper::get_entity_fields( 'property', null, true );

            if ( $fields ) {
                $fields = array_filter( $fields, function( $var ) {
                    return ! empty( $var['section'] );
                } );

                foreach ( $fields as $id => $field ) {
                    if ( ! empty( $field ) ) {
                        $check_field = $wpdb->get_var( "SELECT ID FROM " . $wpdb->prefix . "fbuilder_fields_order WHERE field_machine_name = '{$id}'" );

                        if ( ! $check_field ) {
	                        $wpdb->insert( $wpdb->prefix . 'fbuilder_fields_order', array(
		                        'section_machine_name' => 'es-info',
		                        'field_machine_name' => $id,
	                        ) );
                        }
                    }
                }
            }

            update_option( 'es_migration_fb_fix_1', true );
        }
    }
}
