<?php

/**
 * Class Es_Manage_Agents
 */
class Es_Manage_Agents extends Es_Object
{
    /**
     * @inheritdoc
     */
    public function actions()
    {
        add_action( 'init', array( $this, 'restore_password' ) );
        add_action( 'init', array( $this, 'check_active_state' ) );
        add_action( 'register_new_user', array( $this, 'send_new_user_notifications'), 1 );
        add_action( 'edit_user_created_user', array( $this, 'send_new_user_notifications' ), 1, 2 );
        add_action( 'init', array( $this, 'register' ) );
        add_action( 'admin_notices', array( $this, 'admin_notices' ) );
        add_action( 'wp_logout', array( $this, 'logout_message' ) );
        add_action( 'es_agent_change_status', array( $this, 'disable_properties' ), 10, 3 );
        add_action( 'es_agent_change_status', array( $this, 'enable_properties' ), 10, 3 );

        if ( empty( $_REQUEST[ 'es-login' ] ) ) {
            add_action( 'wp_login', array( $this, 'user_login' ), 1, 2 );
        }

        parent::actions();
    }

    /**
     * Logout unactive user.
     *
     * @return void
     */
    public function check_active_state()
    {
        if ( is_user_logged_in() ) {
            $agent = es_get_agent();

            if ( in_array( $agent::get_role_name(), $agent->get_entity()->roles ) && ! $agent->status ) {
                wp_logout();
            }
        }
    }

    /**
     * After login check to see if user account is disabled.
     *
     * @param string $user_login
     * @param object $user
     */
    public function user_login( $user_login, $user = null ) {

        if ( ! $user ) {
            $user = get_user_by( 'login', $user_login );
        }
        if ( !$user ) {
            // not logged in - definitely not disabled
            return;
        }
        // Get user meta

        $agent = es_get_agent( $user->ID );

        $active = $agent->status;

        // Is the use logging in disabled?
        if ( ! $active && in_array( $agent::get_role_name(), $user->roles ) ) {
            // Clear cookies, a.k.a log user out
            wp_clear_auth_cookie();

            // Build login URL and then redirect
            $login_url = site_url( 'wp-login.php', 'login' );
            $login_url = add_query_arg( 'disabled', '1', $login_url );
            wp_redirect( $login_url );
            exit;
        }
    }

    /**
     * Show a notice to users who try to login and are disabled
     *
     * @since 1.0.0
     * @param string $message
     * @return string
     */
    public function user_login_message( $message ) {

        // Show the error message if it seems to be a disabled user
        if ( isset( $_GET['disabled'] ) && $_GET['disabled'] == 1 )
            $message =  '<div id="login_error">' . __( 'Account disabled.', 'es-plugin' ) . '</div>';

        return $message;
    }

    /**
     * @inheritdoc
     */
    public function filters()
    {
        add_filter( 'retrieve_password_message', array( $this, 'retrieve_password_message' ), 10, 4 );
        add_filter( 'login_message', array( $this, 'user_login_message' ) );
        parent::filters();
    }

    /**
     * Customize WP standard reset password message.
     *
     * @param $message
     * @param $key
     * @param $user_login
     * @param $user_data
     * @return string
     */
    public function retrieve_password_message( $message, $key, $user_login, $user_data )
    {
        global $es_settings;
        $pid = $es_settings->reset_password_page_id;

        if ( user_can( $user_data, Es_Agent::get_role_name() ) && ! empty( $pid ) && get_post( $pid ) ) {
            global $es_settings;

            $message = __('Someone has requested a password reset for the following account:') . "\r\n\r\n";
            $message .= network_home_url( '/' ) . "\r\n\r\n";
            $message .= sprintf(__('Username: %s'), $user_login) . "\r\n\r\n";
            $message .= __('If this was a mistake, just ignore this email and nothing will happen.') . "\r\n\r\n";
            $message .= __('To reset your password, visit the following address:') . "\r\n\r\n";
            $message .= '<' . add_query_arg( array( '_key' => $key, '_login' => rawurlencode($user_login) ),
                    get_the_permalink( $es_settings->reset_password_page_id ) ) . ">\r\n";
        }

        return $message;
    }

    /**
     * Add admin notices.
     *
     * @return void
     */
    public function admin_notices()
    {
        global $es_settings;
        $notice_type = 'notice-warning';

        // If registration is enabled and registration page doesn't exist.
        if ( get_option( 'users_can_register' ) && empty( $es_settings->registration_page_id ) ) : ?>
            <div class="es-notice notice <?php echo $notice_type; ?> is-dismissible">
                <p>
                    <b><?php _e( 'Estatik', 'es-plugin' ); ?></b>:
                    <?php echo sprintf(
                        __( 'Please, create a registration page with <b>[es_register]</b> shortcode and set it on %s page.', 'es-plugin' ),
                        '<a href="' . es_admin_settings_uri() . '" target="_blank">' . __( 'Estatik General settings' ) . '</a>'
                    ); ?>
                </p>
            </div>
        <?php endif;

        // If login page doesn't exist.
        if ( empty( $es_settings->login_page_id ) ) : ?>
            <div class="es-notice notice <?php echo $notice_type; ?> is-dismissible">
                <p>
                    <b><?php _e( 'Estatik', 'es-plugin' ); ?></b>:
                    <?php echo sprintf(
                        __( 'Please, create a login page with <b>[es_login]</b> shortcode and set it on %s page.', 'es-plugin' ),
                        '<a href="' . es_admin_settings_uri() . '" target="_blank">' . __( 'Estatik General settings' ) . '</a>'
                    ); ?>
                </p>
            </div>
        <?php endif;

        // If reset pwd page doesn't exist.
        if ( empty( $es_settings->reset_password_page_id ) ) : ?>
            <div class="es-notice notice <?php echo $notice_type; ?> is-dismissible">
                <p>
                    <b><?php _e( 'Estatik', 'es-plugin' ); ?></b>:
                    <?php echo sprintf(
                        __( 'Please, create a password reset page with <b>[es_reset_pwd]</b> shortcode and set it on %s page.', 'es-plugin' ),
                        '<a href="' . es_admin_settings_uri() . '" target="_blank">' . __( 'Estatik General settings' ) . '</a>'
                    ); ?>
                </p>
            </div>
        <?php endif;
    }

    /**
     * Register new agent action.
     *
     * @return void
     */
    public function register()
    {
        $nonce = 'es_agent_registration';

        if ( ! empty( $_POST[ $nonce ] ) && wp_verify_nonce( $_POST[ $nonce ], $nonce ) && ! empty( $_POST[ 'es_agent' ] ) ) {

            // Messages after registration.
            $logger = new Es_Messenger( 'es_agent_register' );

            if ( isset( $_POST['g-recaptcha-response'] ) ) {
                if ( ! empty( $_POST['g-recaptcha-response'] ) ) {
                    global $es_settings;
                    $secret = $es_settings->recaptcha_secret_key;

                    $verifyResponse = file_get_contents('https://www.google.com/recaptcha/api/siteverify?secret='.$secret.'&response='.$_POST['g-recaptcha-response']);

                    if ( ! empty( $verifyResponse ) ) {
                        $responseData = json_decode( $verifyResponse );

                        if ( ! empty( $responseData->success ) ) {
                            $valid_request = true;
                        } else {
                            $logger->set_message( __( 'Captcha validation failed.', 'es-plugin' ), 'error' );
                            $valid_request = false;
                        }
                    } else {
                        $logger->set_message( __( 'Can\'t connect to google services. Please, contact the support.', 'es-plugin' ), 'error' );
                        $valid_request = false;
                    }
                } else {
                    $logger->set_message( __( 'Invalid recaptcha.', 'es-plugin' ), 'error' );
                    $valid_request = false;
                }
            } else {
                $valid_request = true;
            }

            if ( $valid_request ) {
                // Get agent class.
                $agent = es_get_agent();
                // Register new agent.
                $registered = $agent->register( $_POST[ 'es_agent' ] );

                if ( ! is_wp_error( $registered ) ) {
                    $agent = es_get_agent( $registered->getID() );
                    $agent->change_status( $agent::STATUS_ACTIVE );

                    $logger->set_message( __( 'You are registered at our website. <br>Your username and password were sent to your email address.', 'es-plugin' ), 'success' );
                } else if ( is_wp_error( $registered ) ) {
                    $logger->set_message( $registered->get_error_message(), 'error' );
                } else {
                    $logger->set_message( __( 'You are not registered to our website. <br>Please, contact a support.', 'es-plugin' ), 'error' );
                }
            }

            wp_redirect( $_POST['_redirect'] ); die;
        }
    }

    /**
     * Set success message on user logout.
     *
     * @return void
     */
    public function logout_message() {
        $messenger = new Es_Messenger( 'login' );
        $messenger->clean_container();
        $messenger->set_message( __( 'You are logged out.', 'es-plugin' ), 'success' );
    }

    /**
     * Disable agent properties when agent change status to DISABLED.
     *
     * @param Es_Agent $agent
     * @param int $status
     * @param int $old_status
     */
    public function disable_properties( $agent, $status, $old_status ) {
        if ( $status == $old_status ) return;

        $property = es_get_property( null );

        if ( $agent::STATUS_DISABLED == $status ) {
            $posts = get_posts( array(
                'post_type' => $property::get_post_type_name(),
                'post_status' => 'publish',
                'post_author' => $agent->getID(),
                'author' => $agent->getID(),
                'posts_per_page' => -1,
            ) );

            if ( $posts ) {
                $unpublished_posts = array();

                foreach ( $posts as $post ) {
                    wp_update_post( array( 'ID' => $post->ID, 'post_status' => 'draft' ) );
                    $unpublished_posts[] = $post->ID;
                }

                $agent->save_field_value( 'unpublished_properties', $unpublished_posts );
            }
        }
    }

    /**
     * Enable agent properties when agent change status to ACTIVE.
     *
     * @param Es_Agent $agent
     * @param int $status
     * @param int $old_status
     */
    public function enable_properties( $agent, $status, $old_status ) {
        if ( $status == $old_status ) return;

        if ( $agent::STATUS_ACTIVE == $status ) {
            $post_ids = $agent->unpublished_properties;

            if ( $post_ids ) {
                foreach ( $post_ids as $post_id ) {
                    $post_status = get_post_status( $post_id );
                    if ( $post_status && $post_status != 'publish' ) {
                        wp_update_post( array(
                            'post_status' => 'publish',
                            'ID' => $post_id,
                        ) );
                    }
                }
            }
        }
    }

    /**
     * Handles sending password retrieval email to user.
     *
     * METHOD COPIED FROM wp-login.php BECAUSE WHEN FILE IS INCLUDED - HE SENDS HEADERS.
     *
     * @return bool|WP_Error True: when finish. WP_Error on error
     */
    function retrieve_password() {
        $errors = new WP_Error();

        if ( empty( $_POST['user_login'] ) ) {
            $errors->add('empty_username', __('<strong>ERROR</strong>: Enter a username or email address.'));
        } elseif ( strpos( $_POST['user_login'], '@' ) ) {
            $user_data = get_user_by( 'email', trim( wp_unslash( $_POST['user_login'] ) ) );
            if ( empty( $user_data ) )
                $errors->add('invalid_email', __('<strong>ERROR</strong>: There is no user registered with that email address.'));
        } else {
            $login = trim($_POST['user_login']);
            $user_data = get_user_by('login', $login);
        }

        /**
         * Fires before errors are returned from a password reset request.
         *
         * @since 2.1.0
         * @since 4.4.0 Added the `$errors` parameter.
         *
         * @param WP_Error $errors A WP_Error object containing any errors generated
         *                         by using invalid credentials.
         */
        do_action( 'lostpassword_post', $errors );

        if ( $errors->get_error_code() )
            return $errors;

        if ( empty( $user_data ) ) {
            $errors->add('invalidcombo', __('<strong>ERROR</strong>: Invalid username or email.'));
            return $errors;
        }

        // Redefining user_login ensures we return the right case in the email.
        $user_login = $user_data->user_login;
        $user_email = $user_data->user_email;
        $key = get_password_reset_key( $user_data );

        if ( is_wp_error( $key ) ) {
            return $key;
        }

        $message = __('Someone has requested a password reset for the following account:') . "\r\n\r\n";
        $message .= network_home_url( '/' ) . "\r\n\r\n";
        $message .= sprintf(__('Username: %s'), $user_login) . "\r\n\r\n";
        $message .= __('If this was a mistake, just ignore this email and nothing will happen.') . "\r\n\r\n";
        $message .= __('To reset your password, visit the following address:') . "\r\n\r\n";
        $message .= '<' . network_site_url("wp-login.php?action=rp&key=$key&login=" . rawurlencode($user_login), 'login') . ">\r\n";

        if ( is_multisite() ) {
            $blogname = get_network()->site_name;
        } else {
            /*
             * The blogname option is escaped with esc_html on the way into the database
             * in sanitize_option we want to reverse this for the plain text arena of emails.
             */
            $blogname = wp_specialchars_decode(get_option('blogname'), ENT_QUOTES);
        }

        /* translators: Password reset email subject. 1: Site name */
        $title = sprintf( __('[%s] Password Reset'), $blogname );

        /**
         * Filters the subject of the password reset email.
         *
         * @since 2.8.0
         * @since 4.4.0 Added the `$user_login` and `$user_data` parameters.
         *
         * @param string  $title      Default email title.
         * @param string  $user_login The username for the user.
         * @param WP_User $user_data  WP_User object.
         */
        $title = apply_filters( 'retrieve_password_title', $title, $user_login, $user_data );

        /**
         * Filters the message body of the password reset mail.
         *
         * @since 2.8.0
         * @since 4.1.0 Added `$user_login` and `$user_data` parameters.
         *
         * @param string  $message    Default mail message.
         * @param string  $key        The activation key.
         * @param string  $user_login The username for the user.
         * @param WP_User $user_data  WP_User object.
         */
        $message = apply_filters( 'retrieve_password_message', $message, $key, $user_login, $user_data );

        if ( $message && !wp_mail( $user_email, wp_specialchars_decode( $title ), $message ) )
            wp_die( __('The email could not be sent.') . "<br />\n" . __('Possible reason: your host may have disabled the mail() function.') );

        return true;
    }

    /**
     * Reset password action.
     *
     * @return void
     */
    public function restore_password()
    {
        $nonce = 'es-restore-pwd';

        if ( ! empty( $_REQUEST[ $nonce ] ) && wp_verify_nonce( $_REQUEST[ $nonce ], $nonce ) ) {
            $messenger = new Es_Messenger( 'login' );

            if ( ! empty( $_POST['action'] ) && ! empty( $_POST['_key'] ) && ! empty( $_POST['_login'] ) && $_POST['action'] == 'fill_password' ) {
                $user = check_password_reset_key( $_POST['_key'], $_POST['_login'] );
                if ( $user instanceof WP_Error ) {
                    $messenger->set_message( $user->get_error_message(), 'error' );
                } else {
                    reset_password( $user, $_POST['pwd'] );
                    $messenger->set_message( __( 'Password successfully changed.', 'es-plugin' ), 'success' );
                }
            } else {

                add_filter( 'wp_die_handler', function() use ( $messenger ) {
                    $messenger->set_message( __( 'Email didn\'t sent. PHP Mail is doesn\'t work.' ) , 'error' );
                    wp_redirect( $_POST['redirect'] ); die;
                } );

                $result = $this->retrieve_password();

                if ( $result instanceof WP_Error ) {
                    $messenger->set_message( $result->get_error_message() , 'error' );
                } else {
                    $messenger->set_message( __( 'New password emailed to you.' ) , 'success' );
                }
            }

            wp_redirect( $_POST['redirect'] ); die;
        }
    }

    /**
     * @param null $user_id
     * @param string $notify
     */
    public function send_new_user_notifications( $user_id = null, $notify = 'both' ) {
        if ( user_can( $user_id, Es_Agent::get_role_name() ) ) {

            remove_action( 'register_new_user', 'wp_send_new_user_notifications' );

            global $wpdb, $wp_hasher;
            $user = get_userdata( $user_id );

            // The blogname option is escaped with esc_html on the way into the database in sanitize_option
            // we want to reverse this for the plain text arena of emails.
            $blogname = wp_specialchars_decode(get_option('blogname'), ENT_QUOTES);

            if ( 'user' !== $notify ) {
                $switched_locale = switch_to_locale( get_locale() );
                $message  = sprintf( __( 'New user registration on your site %s:' ), $blogname ) . "\r\n\r\n";
                $message .= sprintf( __( 'Username: %s' ), $user->user_login ) . "\r\n\r\n";
                $message .= sprintf( __( 'Email: %s' ), $user->user_email ) . "\r\n";

                @wp_mail( get_option( 'admin_email' ), sprintf( __( '[%s] New User Registration' ), $blogname ), $message );

                if ( $switched_locale ) {
                    restore_previous_locale();
                }
            }

            // `$deprecated was pre-4.3 `$plaintext_pass`. An empty `$plaintext_pass` didn't sent a user notification.
            if ( 'admin' === $notify || ( empty( $deprecated ) && empty( $notify ) ) ) {
                return;
            }

            // Generate something random for a password reset key.
            $key = wp_generate_password( 20, false );

            /** This action is documented in wp-login.php */
            do_action( 'retrieve_password_key', $user->user_login, $key );

            // Now insert the key, hashed, into the DB.
            if ( empty( $wp_hasher ) ) {
                $wp_hasher = new PasswordHash( 8, true );
            }
            $hashed = time() . ':' . $wp_hasher->HashPassword( $key );
            $wpdb->update( $wpdb->users, array( 'user_activation_key' => $hashed ), array( 'user_login' => $user->user_login ) );

            $switched_locale = switch_to_locale( get_user_locale( $user ) );

            $message = sprintf(__('Username: %s'), $user->user_login) . "\r\n\r\n";
            $message .= __('To set your password, visit the following address:') . "\r\n\r\n";

            global $es_settings;

            if ( $es_settings->reset_password_page_id && get_post( $es_settings->reset_password_page_id ) ) {
                $message .='<' . add_query_arg( array( '_key' => $key, '_login' => rawurlencode($user->user_login) ), get_permalink( $es_settings->reset_password_page_id ) ). ">\r\n\r\n";
            } else {
                $message .= '<' . network_site_url("wp-login.php?action=rp&key=$key&login=" . rawurlencode($user->user_login), 'login') . ">\r\n\r\n";
            }

            if ( $es_settings->login_page_id && get_post( $es_settings->login_page_id ) ) {
                $message .= get_permalink( $es_settings->login_page_id ). "\r\n";
            } else {
                $message .= wp_login_url() . "\r\n";
            }

            wp_mail($user->user_email, sprintf(__('[%s] Your username and password info'), $blogname), $message);

            if ( $switched_locale ) {
                restore_previous_locale();
            }
        }
    }
}
