<?php

/**
 * Class Es_Property_Map_Shortcode for [es_property_map] shortcode.
 */
class Es_Property_Map_Shortcode extends Es_My_Listing_Shortcode
{
    /**
     * @inheritdoc
     */
    public function get_shortcode_name()
    {
        return 'es_property_map';
    }

    /**
     * @inheritdoc
     */
    public function get_shortcode_default_atts()
    {
        global $es_settings;

        $default = array_merge( parent::get_shortcode_default_atts(), array(
            'show' => null,
            'limit' => $es_settings->properties_per_map,
            'width' => $es_settings->map_width,
            'height' => $es_settings->map_height,
            'classes' => '',
        ) );

        return $default;
    }

    /**
     * @inheritdoc
     */
    public function build_query_args( $atts )
    {
        $args = parent::build_query_args( $atts );

        if ( $atts['show'] == 'all') {
            $args['limit'] = -1;
            $args['posts_per_page'] = -1;
        } else {
            $args['posts_per_page'] = $atts['limit'];
        }

        unset( $args['paged'] );

        return $args;
    }

    /**
     * Render map and set js variables for executing this map.
     *
     * @param $query_args
     * @param $atts
     * @return string
     */
    protected function property_loop( $query_args, $atts )
    {
        if ( ! empty( $query_args ) ) {
            $query = new WP_Query( $query_args );
            $posts = $query->get_posts();
        }

        wp_register_script( 'es-property-map-shortcode', ES_PLUGIN_URL . 'assets/js/custom/shortcode-property-map.js',
            array( 'jquery', 'es-admin-googlemap-api' ), false, true );

        global $es_settings, $post;
        $uid = uniqid();

        $temp = $post;

        $variables = Estatik::register_js_variables();

        $counter = 0;
        $temp_taxonomy = null;

        foreach ( $this->get_tax_names() as $taxonomy ) {
            $name = apply_filters( 'es_property_map_tax_exclude_name', substr( $taxonomy, 3 ), $taxonomy );

            if ( ! empty( $atts[ $name ] ) ) {
                $counter++;
                $temp_taxonomy = $taxonomy;
            }
        }

        ob_start();

        $variables['properties'][ $uid ] = array();

        if ( ! empty( $posts ) ) : wp_enqueue_script( 'es-property-map-shortcode' ); ?>

            <?php foreach ( $posts as $post ) : $property = es_get_property( $post->ID ); $marker = self::get_marker( $post, array( $counter, $temp_taxonomy ) );

                if ( ! empty( $property->longitude ) && ! empty( $property->latitude ) && is_numeric( $property->longitude ) && is_numeric( $property->latitude ) ) {
                    $variables['properties'][ $uid ][] = array(
                        'template' => self::get_map_overlay_template( $post, array( 'marker' => $marker, 'default_color' => $es_settings->marker_color ) ),
                        'post' => $post,
                        'lat' => $property->latitude,
                        'lon' => $property->longitude,
                        'marker' => $marker,
                    );
                } ?>
            <?php endforeach; ?>

            <?php if ( ! empty( $variables['properties'][$uid] ) || ( empty( $variables['properties'][ $uid ] ) && is_ssl() ) ) : ?>
                <div class="es-widget-map-wrap">
                    <div class="es-properties-map <?php echo $atts['classes']; ?>"
                         id="es-property-map-<?php echo $uid; ?>"
                         style="width: <?php echo $es_settings->map_width; ?>;
                             height: <?php echo $es_settings->map_height; ?>"></div>
                </div>
            <?php endif; ?>
        <?php endif;

        if ( empty( $variables['properties'][ $uid ] ) && $this instanceof Es_Search_Map_Shortcode ) : ?>
            <p><?php _e( 'Nothing to display', 'es-plugin' ); ?></p>
        <?php endif;

        wp_localize_script( 'es-property-map-shortcode', 'Estatik', $variables );

        add_filter( 'es_global_js_variables', function() use ( $variables ) {
            return $variables;
        } );

        $post = $temp;

        return ob_get_clean();
    }

    /**
     * @return array
     */
    public static function get_marker( $post, $settings = array() )
    {
        global $es_settings;

        if ( ( $markers = $es_settings->map ) ) {
            if ( is_array( $markers ) && ! empty( $markers ) ) {
                foreach ( $markers as $term_id => $_marker ) {
                    if ( ! term_exists( $term_id ) ) continue;

                    $term = get_term( $term_id );

                    if ( ! empty( $settings[0] ) && $settings[0] == 1 && $settings[1] != $term->taxonomy ) continue;

                    if ( is_object_in_term( $post->ID, $term->taxonomy, $term_id ) ) {
                        $marker = array(
                            'color' => $_marker['color'],
                            'content' => $es_settings->get_label( 'markers', $_marker['icon'] ),
                        );
                        break;
                    }
                }
            }
        }

        return isset( $marker ) ? $marker : array();
    }

    /**
     * Return property overlay template for [es_property_map] shortcode.
     *
     * @param $post
     * @return bool|string
     */
    protected static function get_map_overlay_template( $post, $args ) {
        // Property overlay template path.
        $path = apply_filters( 'es_get_map_overlay_template', ES_PLUGIN_PATH . '/templates/partials/map-property-overlay.php' );
        // If file exists.
        if ( file_exists( $path ) ) {
            ob_start();
            include $path;
            return ob_get_clean();
        }
        return false;
    }
}
