<?php

/**
 * @file functions.php
 *
 * Implements functions for plugin.
 */

/**
 * Return current URL.
 *
 * @return string|void
 */
function es_get_current_url() {
    return set_url_scheme( 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'] );
}

/**
 * Return estatik plugin logo path.
 *
 * @return mixed
 */
function es_logo_url() {
    return apply_filters( 'es_logo_url', ES_PLUGIN_URL . '/admin/assets/images/logo.png' );
}

/**
 * Return url to the CSV sample file.
 *
 * @return string
 */
function es_get_sample_csv_url() {
    return apply_filters('es_sample_csv_url', ES_PLUGIN_URL . 'samples/sample.csv');
}

/**
 * Method for getting locale with WPML support.
 *
 * @return mixed
 */
function es_get_locale() {
    if ( ! empty( $_REQUEST['icl_post_language'] ) ) {
        return $_REQUEST['icl_post_language'];
    }

    if ( defined( 'ICL_LANGUAGE_CODE' ) ) {
        return ICL_LANGUAGE_CODE;
    }

    return ! empty( $_GET['lang'] ) ? $_GET['lang'] : str_replace( '_', '-', get_locale() );
}

/**
 * Return array of classes for list.
 *
 * @return mixed
 */
function es_get_list_classes() {
    global $es_settings;

    $list[] = 'es-listing';
    $list[] = 'es-layout-' . $es_settings->listing_layout;
    $list[] = 'hentry';

    $template = get_option( 'template' );

    if ( 'rectangulum' == $template ) {
        $list[] = 'entry-content';
    }

    return apply_filters( 'es_get_list_classes', $list);
}

/**
 * Display list class string.
 *
 * @return void
 */
function es_the_list_classes() {
    $classes = es_get_list_classes();
    echo ! empty( $classes ) ? implode(' ', $classes) : '';
}

/**
 * Return create property page uri.
 *
 * @return mixed
 */
function es_admin_property_add_uri() {
    return apply_filters( 'es_admin_property_list_uri', 'post-new.php?post_type=' . Es_Property::get_post_type_name() );
}

/**
 * Return create property page uri.
 *
 * @return mixed
 */
function es_admin_fields_builder_uri() {
    return apply_filters( 'es_admin_property_fields_builder_uri', 'admin.php?page=es_fbuilder' );
}

/**
 * Return data manager page uri.
 *
 * @return mixed
 */
function es_admin_data_manager_uri() {
    return apply_filters( 'es_admin_property_list_uri', 'admin.php?page=es_data_manager' );
}

/**
 * Return data manager page uri.
 *
 * @return mixed
 */
function es_admin_settings_uri() {
    return apply_filters( 'es_admin_property_list_uri', 'admin.php?page=es_settings' );
}

/**
 * Return Estatik admin listings URI.
 *
 * @return string
 */
function es_admin_property_list_uri() {
    return apply_filters( 'es_admin_property_list_uri', 'edit.php?post_type=' . Es_Property::get_post_type_name() );
}

/**
 * Return agents list uri.
 *
 * @return mixed
 */
function es_admin_agents_uri() {
    return apply_filters( 'es_admin_agents_uri', 'users.php?role=' . Es_Agent::get_role_name() );
}

/**
 * @retrun void
 */
function es_migration_set_executed() {
    update_option( 'es_migration_already_executed', true );
}

/**
 * @return bool
 */
function es_migration_already_executed() {
    return apply_filters( 'es_migration_already_executed', get_option( 'es_migration_already_executed' ) );
}

/**
 * @return array
 */
function es_need_migrate() {
    return Es_Property_Migration::get_prop_ids();
}

/**
 * Return estatik logo markup.
 *
 * @return string;
 */
function es_get_logo() {
    ob_start();

    do_action( 'es_before__logo' );
    echo "<div class='es-logo clearfix'><img src='" . es_logo_url() . "'><br>
            <span class='es-version'>" . __( 'Ver', 'es-plugin' ) . ". " . Estatik::getVersion() .  "</span></div>";
    do_action( 'es_after_logo');

    return ob_get_clean();
}

/**
 * Return property field value.
 *
 * @param $field
 * @param int $post
 * @return mixed|null
 */
function es_get_the_property_field( $field, $post = 0 ) {
    $post = get_post( $post );

    if ( ! empty( $post->ID ) ) {
        $es_property = es_get_property( $post->ID );

        $finfo = Es_Property::get_field_info( $field );

        if ( ! empty( $finfo['system_type'] ) && 'taxonomy' == $finfo['system_type'] && ! empty( $finfo['loop_callback'] ) ) {
            $result = call_user_func_array( $finfo['loop_callback']['callback'], $finfo['loop_callback']['args'] );
        } else {
            $result = $es_property->{$field};
        }

        if ( is_numeric( $result ) && ! empty( $finfo['type'] ) && $finfo['type'] != 'text' ) {
            $result = $result == (int) $result ? (int) $result : $result;
        }

        return apply_filters( 'es_get_the_' . $field, $result );
    }

    return null;
}

/**
 * Filter value for date added field.
 *
 * @return string
 */
function es_get_the_date_added() {
    return es_the_date('', '', false );
}
add_filter( 'es_get_the_date_added', 'es_get_the_date_added', 10 );

/**
 * Render property field.
 *
 * @param $field
 * @param string $before
 * @param string $after
 */
function es_the_property_field( $field, $before = '', $after = '' ) {
    $result = es_get_the_property_field( $field );

    echo ! empty( $result ) ? $before . $result . $after : null;
}

/**
 * Return bathrooms formatted string.
 *
 * @param int $post
 * @return null|string
 */
function es_get_the_formatted_bathrooms( $post = 0 ) {
    $value = es_get_the_property_field( 'bathrooms', $post );

    if ( ! empty( $value ) ) {
        // _n function doesn't work :(
        return $value == 1 ? sprintf( __( '%s bath', 'es-plugin' ), $value  ) :  sprintf( __( '%s baths', 'es-plugin' ), $value );
    }
}

/**
 * Display formatter bath string.
 *
 * @see es_get_the_formatted_bathrooms()
 *
 * @param string $before
 * @param string $after
 * @param bool $display_empty
 *
 * @return void|null|string
 */
function es_the_formatted_bathrooms( $before = '', $after = '', $display_empty = false ) {
    $value = es_get_the_formatted_bathrooms();

    if ( $display_empty || ( ! empty( $value ) ) ) {
        echo $before . $value . $after;
    }
}

/**
 * Return beds formatted string.
 *
 * @param int $post
 * @return null|string
 */
function es_get_the_formatted_bedrooms( $post = 0 ) {
    $value = es_get_the_property_field( 'bedrooms', $post );

    if ( ! empty( $value ) ) {
        // _n function doesn't work :(
        return $value == 1 ? sprintf( __( '%s bed', 'es-plugin' ), $value  ) :  sprintf( __( '%s beds', 'es-plugin' ), $value );
    }
}

/**
 * Display formatter beds string.
 *
 * @see es_get_the_formatted_bathrooms()
 *
 * @param string $before
 * @param string $after
 * @param bool $display_empty
 *
 * @return void|null|string
 */
function es_the_formatted_bedrooms( $before = '', $after = '', $display_empty = false ) {
    $value = es_get_the_formatted_bedrooms();

    if ( $display_empty || ( ! empty( $value ) ) ) {
        echo $before . $value . $after;
    }
}

/**
 * Display property title using {title_address} setting.
 *
 * @param string $before
 * @param string $after
 */
function es_the_title( $before = '', $after = '' ) {
    $result = es_get_the_title();
    echo ! empty( $result ) ? $before . $result . $after : null;
}

/**
 * Return address or title of the property using {title_address} setting.
 *
 * @param int $post
 * @return string
 */
function es_get_the_title( $post = 0 ) {
    global $es_settings;

    if ( $es_settings->title_address == 'address' ) {
        return es_get_the_property_field( 'address', $post );
    } else {
        return get_the_title( $post );
    }
}


/**
 *
 * the_title filter.
 * @param $title
 * @param $id
 * @return string
 */
function es_the_title_filter( $title, $id = null ) {
    global $es_settings;
    $post = get_post( $id );
    if ( is_single() && $post->post_type == Es_Property::get_post_type_name() ) {
        if ( $es_settings->title_address == 'address' ) {
            return es_get_the_property_field( 'address', $post );
        }
    }

    return $title;
}
add_filter( 'the_title', 'es_the_title_filter', 10, 2 );

/**
 * Display property address string.
 *
 * @param string $before
 * @param string $after
 */
function es_the_address( $before = '', $after = '' ) {
    global $es_settings;

    $result = es_get_the_property_field( 'address' );
    echo ! empty( $result ) && $es_settings->show_address ? $before . $result . $after : null;
}

/**
 * @param string $before
 * @param string $after
 * @param bool $echo
 * @param string $default_class
 * @return null|string
 */
function es_the_formatted_price( $before = '', $after = '', $echo = true, $default_class = 'es-price' ) {
    /** @var Es_Settings_Container $es_settings */
    global $es_settings;

    // Get property price.
    $price = es_get_the_property_field( 'price' );
    $call = es_get_the_property_field( 'call_for_price' );


    if ( $call && $es_settings->show_price ) {
        $result = '<span class="es-price">' . __( 'Call for price', 'es-plugin' ) . '</span>';

        $result = $before . apply_filters( 'es_the_formatted_price', $result, $price, $call ) . $after;

        if ( ! $echo ) {
            return $result;
        }

        echo $result;
        return;
    }

    // Get position of the currency.
    $position = $es_settings->currency_position;
    // Get currency name using currency code.
    $currency = $es_settings->get_label( 'currency', $es_settings->currency );
    // Get price format.
    $format = $es_settings->price_format;

    $price_temp = ! $price ? 0 : $price;

    $sup = ! empty( $format[0] ) ? $format[0] : null;
    $dec = ! empty( $format[1] ) ? $format[1] : null;

    $dec_num = $sup == ' ' ? 0 : 2;

    if ( $currency == 'RUB' ) {
        $currency = '<i class="fa fa-rub" aria-hidden="true"></i>';
    }

    $price_temp = number_format( $price_temp, $dec_num, $dec, $sup );
    $price_temp = $position == 'after' ? $price_temp . ' ' . $currency : $currency . ' ' . $price_temp;

    $result = '<span class="' . $default_class . '">' . $price_temp . '</span>';

    $formetted = ! empty( $price ) && $es_settings->show_price ?
        $before . apply_filters( 'es_the_formatted_price', $result, $price_temp ) . $after : null;

    if ( $echo ) {
        echo $formetted;
    } else {
        return $formetted;
    }
}

/**
 * @param $field
 * @param $formatter
 *
 * @param null $field_info
 * @return string
 */
function es_get_the_formatted_field( $field, $formatter ) {
    /** @var Es_Settings_Container $es_settings */
    global $es_settings;

    $result = es_get_the_property_field( $field );

    switch( $formatter ) {
        case 'price':
            if ( ! $result ) break;

            // Get position of the currency.
            $position = $es_settings->currency_position;
            // Get currency name using currency code.
            $currency = $es_settings->get_label( 'currency', $es_settings->currency );
            // Get price format.
            $format = $es_settings->price_format;

            $price_temp = ! $result ? 0 : $result;

            $sup = ! empty( $format[0] ) ? $format[0] : null;
            $dec = ! empty( $format[1] ) ? $format[1] : null;

            $dec_num = $sup == ' ' ? 0 : 2;

            if ( $currency == 'RUB' ) {
                $currency = '<i class="fa fa-rub" aria-hidden="true"></i>';
            }

            $price_temp = number_format( $price_temp, $dec_num, $dec, $sup );
            return $position == 'after' ? $price_temp . ' ' . $currency : $currency . ' ' . $price_temp;

            break;

        case 'area':
	        if ( ! $result ) break;
            $es_property = es_get_property( null );
            $fields = $es_property::get_fields();

            $unit = ! empty( $fields[ $field ]['units'] ) ? es_get_the_property_field( $fields[ $field ]['units'] ) : null;
            $unit = $unit ? $unit : $es_settings->unit;
            $unit = $unit ? $es_settings->get_label( 'unit', $unit ) : null;

            return  ! empty( $result ) ? $result . ' ' . $unit : null;

            break;

        case 'url':
            return "<a class='es-url-link es-url-link-{$field}' target='_blank' href='{$result}'>{$result}</a>";
            break;

        case 'file':
            $entity_field_info = Es_Property::get_field_info( $field );

            if ( $result && ( $attachment = wp_get_attachment_url( $result ) ) ) {
                $finfo = pathinfo( $attachment );

                if ( ! empty( $entity_field_info['show_thumbnail'] ) ) {
                    if ( ! function_exists( 'file_is_valid_image' ) ) {
                        require_once( ABSPATH . 'wp-admin/includes/image.php' );
                    }

                    if ( file_is_displayable_image( $attachment ) ) {
                        $file = '<a href="' . $attachment . '" class="js-magnific-gallery">' . wp_get_attachment_image( $result, 'es-image-size-archive' ) . '</a>';
                    } else {
                        $file = '<a href="' . $attachment . '" target="_blank">' . es_get_file_icon( $finfo['basename'] ) . '</a>';
                    }

                    return  array( 'markup' => sprintf("<li class='es-file__wrap'>
                                <div class='es-file__content'>%s</div>
                                <span class='es-file__label'><b>%s</b></span>
                            </li>",$file, ! empty( $entity_field_info['label'] ) ? $entity_field_info['label']  : null ) );
                } else {
                    $icon = es_get_file_icon( $finfo['basename'] );
                    $icon = $icon ? $icon . ' ' : null;
                    return $icon . '<a href="' . $attachment . '" target="_blank">' .$finfo['basename'] . '</a>';
                }
            }
            break;
    }

    return apply_filters( 'es_get_the_formatted_field', $result, $field, $formatter );
}

/**
 * Return property categories.
 *
 * @param string $before
 * @param string $sep
 * @param string $after
 * @param bool $echo
 * @return string|void
 */
function es_the_categories( $before = '', $sep = ', ', $after = '', $echo = true ) {
    ob_start();
    the_terms( 0, 'es_category', $before, $sep, $after );
    $result = ob_get_clean();

    if ( $echo ) {
        echo $result;
    } else {
        return $result;
    }
}

/**
 * Overridden the_content function.
 * For now the_content function uses for execute [es_single] shortcode.
 * Use this function instead of the_content.
 *
 * @param null $more_link_text
 * @param bool $strip_teaser
 */
function es_the_content( $more_link_text = null, $strip_teaser = false ) {
    $content = get_the_content( $more_link_text, $strip_teaser );

    /**
     * Filters the post content.
     *
     * @since 0.71
     *
     * @param string $content Content of the current post.
     */
    $content = apply_filters( 'es_the_content', $content );
    $content = str_replace( ']]>', ']]&gt;', $content );
    echo $content;
}

add_filter( 'es_the_content', 'wptexturize'                       );
add_filter( 'es_the_content', 'convert_smilies', 20               );
add_filter( 'es_the_content', 'wpautop'                           );
add_filter( 'es_the_content', 'shortcode_unautop'                 );
add_filter( 'es_the_content', 'prepend_attachment'                );
add_filter( 'es_the_content', 'wp_make_content_images_responsive' );

/**
 * @return mixed
 */
function es_get_standard_label_names() {
    return  apply_filters( 'es_install_standard_labels', array(
        '#00cbf0' => __( 'Featured', 'es-plugin' ),
        '#ff9600' => __( 'Hot', 'es-plugin'),
        '#2bbe0e' => __( 'Open House', 'es-plugin' ),
        '#9e9e9e' => __( 'Foreclosure', 'es-plugin' ),
    ) );
}

function es_get_default_label_color() {
    return apply_filters( 'es_default_label_color', '#9e9e9e' );
}

/**
 * @param $term_id
 * @param string $color
 * @return mixed|string
 */
function es_get_the_label_color(  $term_id, $color = '#9e9e9e' ) {
    $meta = get_term_meta( $term_id, 'es_color', true );
    return ! empty( $meta ) ? get_term_meta( $term_id, 'es_color', true ) : $color;
}

/**
 * Return property status list.
 *
 * @param string $before
 * @param string $sep
 * @param string $after
 * @param bool $echo
 * @return string
 */
function es_the_status_list( $before = '', $sep = ', ', $after = '', $echo = true ) {
    ob_start();
    the_terms( 0, 'es_status', $before, $sep, $after );
    $result = ob_get_clean();

    if ( $echo ) {
        echo $result;
    } else {
        return $result;
    }
}

/**
 * Return property status list.
 *
 * @param $before
 * @param $sep
 * @param $after
 * @return string
 */
function es_the_rent_period( $before = '', $sep = ', ', $after = '', $echo ) {
    ob_start();
    the_terms( 0, 'es_rent_period', $before, $sep, $after );
    $result = ob_get_clean();

    if ( $echo ) {
        echo $result;
    } else {
        return $result;
    }
}

/**
 * Return property status list.
 *
 * @param $before
 * @param $sep
 * @param $after
 */
function es_the_amenities( $before = '', $sep = ', ', $after = '' ) {
    the_terms( 0, 'es_amenities', $before, $sep, $after );
}

/**
 * Return list of terms of amenities taxonomy.
 *
 * @return array|false|WP_Error
 */
function es_get_the_amenities( $post = 0 ) {
    return get_the_terms( $post, 'es_amenities' );
}

/**
 * Get agent.
 *
 * @param null $user_id
 * @return Es_Agent
 */
function es_get_agent( $user_id = null ) {
    $user_id = $user_id instanceof WP_User ? $user_id->ID : $user_id;
    return apply_filters( 'es_get_agent', new Es_Agent( $user_id ? $user_id : get_current_user_id() ) );
}

/**
 * @param int $post
 * @return array|false|WP_Error
 */
function es_get_the_features( $post = 0 ) {
    return get_the_terms( $post, 'es_feature' );
}

/**
 * @param $id
 * @return mixed|Es_Order
 */
function es_get_order( $id ) {
    return apply_filters( 'es_get_order', new Es_Order( $id ) );
}

/**
 * Display list of property types.
 *
 * @param string $before
 * @param string $sep
 * @param string $after
 * @param bool $echo
 * @return void|string
 */
function es_the_types( $before = '', $sep = ', ', $after = '', $echo = true ) {
    ob_start();
    the_terms( 0, 'es_type', $before, $sep, $after );
    $result = ob_get_clean();

    if ( $echo ) {
        echo $result;
    } else {
        return $result;
    }
}

/**
 * Render property formatted area field with units.
 *
 * @param string $before
 * @param string $after
 * @param bool $echo
 *
 * @return void|string
 */
function es_the_formatted_area( $before = '', $after = '', $echo = true ) {
    /** @var Es_Settings_Container $es_settings */
    global $es_settings;

    $es_property = es_get_property( null );
    $fields = $es_property::get_fields();

    $result = es_get_the_property_field( 'area' );
    $unit = ! empty( $fields['area']['units'] ) ? es_get_the_property_field( $fields['area']['units'] ) : null;
    $unit = $unit ? $unit : $es_settings->unit;
    $unit = $unit ? $es_settings->get_label( 'unit', $unit) : null;

    $result = ! empty( $result ) ?
        $before . apply_filters( 'es_the_formatted_area', $result . ' ' . $unit, $result ) . $after : null;

    if ( $echo ) {
        echo $result;
    } else {
        return $result;
    }
}

/**
 * @param string $size
 * @param string $entity
 * @return string
 */
function es_get_default_thumbnail( $size = 'thumbnail', $entity = 'property' ) {
    global $es_settings;

    if ( $entity == 'property' ) {
        $meta = 'thumbnail_attachment_id';
        $thumbnail = ES_PLUGIN_URL . 'assets/images/thumbnail.png';
    } else {
        $meta = 'thumbnail_attachment_agent_id';
        $thumbnail = ES_PLUGIN_URL . 'assets/images/agent.png';
    }

    $attachment_id = $es_settings->{$meta};

    if ( ! $es_settings->{$meta} ) {
        if ( ! function_exists( 'wp_handle_upload' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
        }

        if ( ! function_exists( 'wp_generate_attachment_metadata' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/image.php' );
        }

        if ( ! function_exists( 'media_handle_upload' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/media.php' );
        }

        $upload_dir = wp_upload_dir();
        $file['name'] = basename( $thumbnail );
        $file['tmp_name'] = download_url( $thumbnail );

        $file = wp_handle_sideload( $file, array( 'test_form' => false ) );

        if ( empty( $file['error'] ) ) {
            $wp_filetype = wp_check_filetype( basename( $file['file'] ), null );

            $attachment = array(
                'guid' => $upload_dir['baseurl'] . ES_DS . _wp_relative_upload_path( $file['file'] ),
                'post_mime_type' => $wp_filetype['type'],
                'post_title' => preg_replace('/\.[^.]+$/', '', basename( $file['file'] ) ),
                'post_content' => '',
                'post_status' => 'inherit'
            );

            $attachment_id = wp_insert_attachment( $attachment, $file['file'] );

            $attach_data = wp_generate_attachment_metadata( $attachment_id,  get_attached_file( $attachment_id ) );
            wp_update_attachment_metadata( $attachment_id,  $attach_data );

            $es_settings->saveOne( $meta, $attachment_id );
            $attachment_id = $es_settings->{$meta};
        } else {
            $attachment_id = null;
        }
    }

    return wp_get_attachment_image( $attachment_id, $size );
}

/**
 * Render property formatted lot size field with units.
 *
 * @param string $before
 * @param string $after
 * @param bool $echo
 * @return mixed|null|string|void
 */
function es_the_formatted_lot_size( $before = '', $after = '', $echo = true ) {
    /** @var Es_Settings_Container $es_settings */
    global $es_settings;

    $es_property = es_get_property( null );
    $fields = $es_property::get_fields();

    $result = es_get_the_property_field( 'lot_size' );
    $unit = ! empty( $fields['lot_size']['units'] ) ? es_get_the_property_field( $fields['lot_size']['units'] ) : null;
    $unit = $unit ? $unit : $es_settings->unit;
    $unit = $unit ? $es_settings->get_label( 'unit', $unit ) : null;

    $result = ! empty( $result ) ?
        $before . apply_filters( 'es_the_formatted_lot_size', $result . ' ' . $unit, $result ) . $after : null;

    if ( $echo ) {
        echo $result;
    } else {
        return $result;
    }
}

/**
 * Return property map block.
 *
 * @return void
 */
function es_the_map() {
    global $es_property;

    $data = ! empty( $es_property->latitude ) && ! empty( $es_property->longitude ) ?
        "data-lat='{$es_property->latitude}' data-lon='{$es_property->longitude}'" : null;

    echo apply_filters( 'es_the_map', '<div id="es-google-map" ' . $data . ' style="width:100%; height:300px;"></div>' );
}

/**
 * Return property added date.
 *
 * @param string $before
 * @param string $after
 * @param bool $echo
 * @return string|void
 */
function es_the_date( $before = '', $after = '', $echo = true ) {
    global $es_settings;

    if ( $es_settings->date_added ) {
        if ( $echo ) {
            the_date( $es_settings->date_format, $before, $after, $echo );
        } else {
            return $before . get_the_date( $es_settings->date_format ) . $after;
        }
    }
}

/**
 * Return thumbnail url for empty image.
 *
 * @param string $size
 *
 * @return false|null|string
 */
function es_get_empty_image_url( $size = 'thumbnail' ) {
    $attachment_id = get_option( 'es_empty_image_attachment_id' );
    return ! empty( $attachment_id ) ? wp_get_attachment_image_url( $attachment_id, $size ) : null;
}

/**
 * Return thumbnail for empty image.
 *
 * @param string $size
 * @param string $attr
 *
 * @return null|string
 */
function es_get_empty_image( $size = 'thumbnail', $attr = '' ) {
    echo es_get_default_thumbnail( $size );
}

/**
 * Get property.
 *
 * @param $post_id
 * @return Es_Property
 */
function es_get_property( $post_id ) {
    return apply_filters( 'es_get_property', new Es_Property( $post_id ) );
}

/**
 * @param $parent_file
 * @return string
 */
function es_agent_menu_fix( $parent_file ){
    global $submenu_file;

    if ( $parent_file == 'users.php' && isset( $_GET['role'] ) ) {
        global $parent_file;
        $submenu_file = es_admin_agents_uri();
        $parent_file = 'es_dashboard';
    }
    return $parent_file;
}

add_filter( 'parent_file', 'es_agent_menu_fix', 10, 1 );


/**
 * @param $from
 * @param $to
 * @param $value
 *
 * @return int
 */
function es_prepare_unit( $from, $to, $value ) {

    // OH. MY. GOD. Switch in the switch. Perfect.

    switch ( $from ) {

        case 'hectares':
            switch ( $to ) {
                case 'sq_ft':
                    $value = $value * 107639.104;
                    break;

                case 'sq_m':
                    $value = $value * 10000;
                    break;

                case 'acres':
                    $value = $value * 2.4710538146717;
                    break;
            }
            break;

        case 'sq_ft':
            switch ( $to ) {
                case 'hectares':
                    $value = $value / 107639.104;
                    break;

                case 'sq_m':
                    $value = $value / 10;
                    break;

                case 'acres':
                    $value = $value / 43560;
                    break;
            }
            break;

        case 'sq_m':
            switch ( $to ) {
                case 'hectares':
                    $value = $value / 10000;
                    break;

                case 'sq_ft':
                    $value = $value * 10;
                    break;

                case 'acres':
                    $value = $value / 4046.8564300507887;
                    break;
            }
            break;

        case 'acres':
            switch ( $to ) {
                case 'hectares':
                    $value = $value / 2.4710538146717;
                    break;

                case 'sq_ft':
                    $value = $value * 43560;
                    break;

                case 'sq_m':
                    $value = $value * 4046.8564300507887;
                    break;
            }
            break;

        default:
            $value = false;
    }

    return apply_filters( 'es_prepare_unit', $value, $from, $to, $value );
}

/**
 * Return HTML based content for ajax action for calculate units.
 *
 * @return void
 */
function es_ajax_calculate_units() {

    if ( ! empty( $_POST['unit'] ) && ! empty( $_POST['val'] ) ) {
        /** @var $es_settings Es_Settings_Container */
        global $es_settings; $content = null;

        $template = "<b>{unit}: </b>{value}</br>";

        foreach ( $es_settings::get_setting_values('unit') as $key => $setting_value ) {
            $unit = es_prepare_unit( $_POST['unit'], $key, $_POST['val'] );

            if ( $unit === false) continue;

            $content .= strtr( $template, array(
                '{value}' => $unit,
                '{unit}' => $es_settings->get_label( 'unit', $key )
            ) );
        }

        wp_die( json_encode( array(
            'status' => true,
            'content' => $content,
        ) ) );
    }
}

add_action( 'wp_ajax_es_calculate_units', 'es_ajax_calculate_units' );

/**
 * Render property main thumbnail.
 *
 * @param string $size
 */
function es_the_post_thumbnail( $size = 'thumbnail' ) {
    global $post;

    $property = es_get_property( $post->ID );
    $images = $property->gallery;

    echo ! empty( $images[0] ) ? wp_get_attachment_image( $images[0], $size ) : null;
}

/**
 * Return registration page URL.
 *
 * @return string
 */
function es_get_register_url() {
    global $es_settings;
    $can_register = get_option( 'users_can_register' );
    $register_page_ID = $es_settings->registration_page_id;

    if ( $can_register && $register_page_ID && ( $_post = get_post( $register_page_ID ) ) ) {
        return get_permalink( $_post );
    }
}

/**
 * Render estatik message.
 *
 * @param $message
 * @param string $type
 */
function es_render_message( $message, $type = 'success' ) {
    switch ( $type ) {
        case 'error':
            $icon = '<i class="fa fa-times-circle-o" aria-hidden="true"></i>';
            break;
        case 'warning':
            $icon = '<i class="fa fa-exclamation-circle" aria-hidden="true"></i>';
            break;
        case 'success':
            $icon = '<i class="fa fa-check-circle-o" aria-hidden="true"></i>';
            break;
        default:
            $icon = '<i class="fa fa-check-circle-o" aria-hidden="true"></i>';
            break;
    }
    $message = $icon .  ' ' . $message;

    echo apply_filters( 'es_render_message', '<p class="es-message es-message-' . $type . '" >' . $message . '</p>', $message, $type );
}

/**
 * @return array
 */
function es_get_users_list( $args ) {
    $users = get_users( $args );
    $data = array();

    /** @var WP_User[] $users */
    if ( ! empty( $users ) ) {
        foreach ( $users as $user ) {
            $agent = es_get_agent( $user->ID );
            $data[ $user->ID ] = $agent->get_full_name() ? $agent->get_full_name() : $user->user_nicename;
        }
    }

    return $data;
}

/**
 * Return file icon markup.
 *
 * @param $url
 * @return string Font Awesome icon markup.
 */
function es_get_file_icon( $url ) {
    $finfo = pathinfo( $url );
    $icon = null;

    if ( ! empty( $finfo['extension'] ) ) {
        switch ( $finfo['extension'] ) {
            case 'pdf':
                $class = 'fa-file-pdf-o';
                break;

            case 'jpg':
            case 'jpeg':
            case 'png':
            case 'gif':
            case 'bmp':
                $class = 'fa-file-image-o';
                break;

            case 'txt':
            case 'doc':
            case 'docx':
                $class = 'fa-file-text-o';
                break;

            case 'xls':
                $class = 'fa-file-excel-o';
                break;

            default:
                $class = 'fa-file-o';
        }

        $icon = '<i class="fa %s" aria-hidden="true"></i>';

        $icon = sprintf( $icon, $class );
    }

    return apply_filters( 'es_get_file_econ', $icon, $url, $finfo );
}

/**
 * Render rating block.
 *
 * @param int $rating
 */
function es_render_rating( $rating ) {
    $img_en = ES_PLUGIN_URL . '/assets/images/star_rating.png';
    $img_ds = ES_PLUGIN_URL . '/assets/images/star_rating-d.png';

    if ( ! $rating ) return;

    echo '<span class="es-agent-rating">';

    for ($i = 1; $i <= 5; $i++) {
        $img = $i > $rating ? $img_ds : $img_en;
        printf( '<img src="%s" height="22" style="margin-top:3px;" width="auto"/>', $img );
    }

    echo '</span>';
}

/**
 * @param $query
 * @param array $args
 *
 * @return null|string
 */
function es_the_pagination( $query, $args = array() ) {
	$navigation = null;

	if ( get_query_var( 'paged' ) ) {
		$page_num = get_query_var( 'paged' );
	} elseif ( get_query_var( 'page' ) ) {
		$page_num = get_query_var( 'page' );
	} else {
		$page_num = 1;
	}

	$args = wp_parse_args( $args, array(
		'show_all'           => false,
		'end_size'           => 2,
		'mid_size'           => 2,
		'screen_reader_text' => ' ',
		'total'              => $query->max_num_pages,
		'current' => $page_num,
	) );

	$args = apply_filters( 'es_the_pagination_args', $args );

	$links = paginate_links( $args );

	if ( $links ) {
		$navigation = _navigation_markup( $links, 'pagination', $args['screen_reader_text'] );
	}

	return $navigation;
}

/**
 * Render google captcha function.
 *
 * @return void
 */
function es_render_recaptcha() {
    global $es_settings;
    $lang = es_get_locale();
    $siteKey = $es_settings->recaptcha_site_key;
    if ( ! empty( $siteKey ) ) :
    ?>
    <div class="es-recaptcha-wrapper">
        <div class="g-recaptcha" data-sitekey="<?php echo $siteKey;?>"></div>
        <script type="text/javascript"
                src="https://www.google.com/recaptcha/api.js?hl=<?php echo $lang;?>">
        </script>
        </div>
    <?php endif;
}
add_action( 'es_recaptcha', 'es_render_recaptcha' );

/**
 * @param $secret
 *
 * @return bool
 */
function es_validate_recaptcha( $secret = null ) {
	$check = false;

	if ( isset( $_REQUEST['g-recaptcha-response'] ) ) {
		if ( ! empty( $_REQUEST['g-recaptcha-response'] ) ) {
			global $es_settings;
			$secret = $secret ? $secret : $es_settings->recaptcha_secret_key;

			$verifyResponse = file_get_contents('https://www.google.com/recaptcha/api/siteverify?secret=' . $secret.'&response=' . $_REQUEST['g-recaptcha-response'] );

			if ( ! empty( $verifyResponse ) ) {
				$responseData = json_decode( $verifyResponse );

				if ( ! empty( $responseData->success ) ) {
					$check = true;
				}
			}
		} else {
			$check = false;
		}
	} else {
		$check = true;
	}

	return $check;
}
