<?php

/**
 * Class Es_Prop_Management_Init.
 */
class Es_Prop_Management_Init extends Es_Object
{
    /**
     * @inheritdoc
     */
    public static function init()
    {
        parent::init();
        self::class_loader();
    }

    /**
     * Load module classes.
     *
     * @return void
     */
    public static function class_loader()
    {
        $files = array(
            ES_PLUGIN_PATH . ES_DS . 'management' . ES_DS . 'classes' . ES_DS . 'table' . ES_DS . 'class-list-table.php',
            ES_PLUGIN_PATH . ES_DS . 'management' . ES_DS . 'classes' . ES_DS . 'table' . ES_DS . 'class-post-list-table.php',
        );

        $files = apply_filters( 'es_prop_management_class_loader', $files );

        foreach ( $files as $file ) {
            if ( ! file_exists( $file ) ) continue;

            include $file;
        }
    }

    /**
     * @inheritdoc
     */
    public function filters()
    {
        add_filter( 'es_shortcodes_list', array( $this, 'shortcodes' ), 10, 1 );
        add_filter( 'es_post_list_table_get_columns', array( $this, 'list_table_columns' ), 10, 2 );
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );
        add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
        add_action( 'es_list_table_value_col', array( $this, 'get_table_value' ), 10, 3 );
        add_action( 'es_after_property_metabox_front_tab_content', array( 'Es_Property_Metabox', 'render_admin_map' ) );
        add_action( 'init', array( $this, 'save_property' ) );
        add_action( 'init', array( $this, 'manage_actions' ) );
    }

    /**
     * Enqueue scripts for management module.
     *
     * @return void
     */
    public function enqueue_scripts()
    {
        $custom = ES_PLUGIN_URL . '/management/assets/js/custom/';
        $custom_admin = ES_PLUGIN_URL . '/admin/assets/js/custom/';

        wp_register_script( 'es-checkbox-script', $custom_admin . 'es-checkboxes.js', array ( 'jquery' ) );

        wp_register_script( 'es-popup-script', $custom_admin . 'es-popup.js', array ( 'jquery' ) );

        wp_enqueue_media();

        wp_register_script( 'es-admin-media-script', $custom_admin . 'media.js', array( 'jquery' ), false, $in_footer = true );
        wp_enqueue_script( 'es-admin-media-script' );

        $deps = array ( 'jquery', 'es-dropdown-script', 'es-popup-script', 'jquery-ui-datepicker', 'jquery-ui-tabs', 'es-checkbox-script' );

        global $es_settings;

        if ( $es_settings->google_api_key ) {
            $deps[] = 'es-map-tab-script';
        }

        // Base front script.
        wp_register_script(
            'es-management-script',
            $custom . 'management.js',
            $deps
        );

        wp_enqueue_script( 'es-management-script' );
    }

    /**
     * Add styles for management page.
     *
     * @return void
     */
    public function enqueue_styles()
    {
        $custom = ES_PLUGIN_URL . '/management/assets/css/custom/';
        $custom_admin = ES_PLUGIN_URL . '/admin/assets/css/custom/';

        wp_register_style( 'es-checkboxes-style', $custom_admin . 'es-checkboxes.css' );
        wp_enqueue_style( 'es-checkboxes-style' );

        // Register base styles for the plugin.
        wp_register_style( 'es-management-style', $custom . 'management.css' );
        wp_enqueue_style( 'es-management-style' );
    }

    /**
     * Add new shortcode for prop management.
     *
     * @param $list
     *
     * @return array
     */
    public function shortcodes( $list )
    {
        return array_merge( $list, array( 'Es_Prop_Management_Shortcode' ) );
    }

    /**
     * Get table columns filter.
     *
     * @param array $columns
     * @param  Es_List_Table $table
     *
     * @return array
     */
    public function list_table_columns( $columns, $table )
    {
        if ( ! empty( $table->settings['table'] ) && $table->settings['table'] == 'management' ) {
            unset( $columns['ID'], $columns['post_title'] );
            $columns = self::push_column( array( 'cb' => '<input type="checkbox" class="js-es-check-all">' ), $columns, 0 );
            $columns = self::push_column( array( 'post_id' => __( 'ID', 'es-plugin' ) ), $columns, 1 );
            $columns = self::push_column( array( 'image' => __( 'Image', 'es-plugin' ) ), $columns, 2 );
            $columns = self::push_column( array( 'title' => __( 'Title', 'es-plugin' ) ), $columns, 3 );
            $columns = self::push_column( array( 'post_date' => __( 'Date added', 'es-plugin' ) ), $columns, 4 );
            $columns = self::push_column( array( 'address' => __( 'Address', 'es-plugin' ) ), $columns, 5 );
            $columns = self::push_column( array( 'category' => __( 'Category', 'es-plugin' ) ), $columns, 6 );
            $columns = self::push_column( array( 'type' => __( 'Type', 'es-plugin' ) ), $columns, 7 );
            $columns = self::push_column( array( 'status' => __( 'Status', 'es-plugin' ) ), $columns, 8 );
        }

        return $columns;
    }

    /**
     * Set values for management properties table.
     *
     * @param string $column
     * @param WP_Post $_post
     * @param Es_List_Table $table
     */
    public function get_table_value( $column, $_post, $table )
    {
        // If we alter a correct table.
        if ( ! empty( $table->settings['table'] ) && $table->settings['table'] == 'management' ) {
            global $post;
            $temp = $post;
            $post = $_post;

            if ( 'cb' == $column )
                echo '<input type="checkbox" name="es_properties[]" value="' . $_post->ID . '">';

            if ( 'post_id' == $column )
                echo '#' . $_post->ID;

            if ( 'image' == $column )
                es_the_post_thumbnail( static::table_image_size() );

            if ( 'type' == $column )
                es_the_types();

            if ( 'status' == $column )
                es_the_status_list();

            if ( 'category' == $column )
                es_the_categories();

            if ( 'address' == $column ) {
                $property = es_get_property( $post->ID );
                echo $property->address;
            }

            if ( 'title' == $column ) {
                echo $post->post_title;

                // Customize view link.
                echo '<div class="es-manage-links__wrap"><a href="' . get_the_permalink( $post->ID ) . '" target="_blank"><i class="fa fa-eye" aria-hidden="true"></i></a>';

                echo '<a href="' . add_query_arg( array( 'es-page' => 'edit-property', 'id' => $post->ID, 'es_filter' => null ) ) . '" 
                        title="' . __( 'Edit', 'es-plugin' ) . '"><i class="fa fa-pencil"></i></a>';

                // Customize remove link.
                echo '<a href="' . $this->get_delete_post_link( $post ) . '" class="es-trash" title="' . __( 'Remove', 'es-plugin' ) . '"><i class="fa fa-trash" aria-hidden="true"></i></a></div>';
            }

            $post = $temp;
        }
    }

    /**
     * Return delete post link.
     *
     * @param $post WP_Post
     * @return mixed
     */
    public function get_delete_post_link( $post ) {
        return apply_filters( 'es_get_delete_post_link', add_query_arg( array(
            'es-action' => 'delete',
            'manage_actions' => wp_create_nonce( 'manage_actions' ),
            'es_properties' => array( $post->ID ),
            '_redirect' => es_get_current_url()
        ) ), $post );
    }

    /**
     * Return image size for thumbnail column.
     *
     * @see add_image_size()
     *
     * @return mixed|string
     */
    protected static function table_image_size()
    {
        return apply_filters( get_post_type() . '_list_table_image_size', 'thumbnail' );
    }

    /**
     * Save property action.
     */
    public function save_property()
    {
        $nonce = 'es_manage_property';

        $messenger = new Es_Messenger( 'manage-property' );

        if ( ! empty( $_REQUEST[ $nonce ] ) && wp_verify_nonce( $_REQUEST[ $nonce ], $nonce ) ) {
            if ( ! empty( $_POST['property']['system'] ) ) {

                $system = $_POST['property']['system'];

                if ( ! empty( $system['post_title'] ) && empty( $system['post_name'] ) ) {
                    $system['post_name'] = $system['post_title'];
                }

                if ( ( ! empty( $system['ID'] ) && current_user_can( 'edit_es_properties' ) ) || ( empty( $system['ID'] ) && current_user_can( 'create_es_properties' ) ) ) {
                    global $es_settings;

                    $post_arr = wp_parse_args( $system, array(
                        'post_type' => Es_Property::get_post_type_name(),
                        'post_status' => $es_settings->listings_publishing == 'automatic' ? 'publish' : 'draft',
                    ) );

                    if ( ! empty( $system['ID'] ) ) {
                        $post_id = wp_update_post( $post_arr, true );
                    } else {
                        $post_id = wp_insert_post( $post_arr, true );
                    }

                    if ( ! empty( $_POST['_redirect'] ) && ! $post_id instanceof WP_Error ) {
                        if ( 'publish' == $post_arr['post_status'] ) {
                            $msg = sprintf(  __( 'Property %s is successfully created.', 'es-plugin' ), $post_id );
                        } else {
                            $msg = sprintf(  __( 'Property %s is successfully created. Please, wait for approve.', 'es-plugin' ), $post_id );
                        }

                        $messenger->set_message( $msg, 'success' );

                        wp_redirect( $_POST['_redirect'] );
                        die;
                    } else {
                        $messenger->set_message( $post_id->get_error_message(), 'error' );
                    }
                }
            }
        }
    }

    /**
     * @return void
     */
    public function manage_actions()
    {
        $nonce = 'manage_actions';
        if ( ! empty( $_REQUEST[ $nonce ] ) && wp_verify_nonce( $_REQUEST[ $nonce ], $nonce )
            && ! empty( $_GET['es-action'] ) && ! empty( $_GET['es_properties'] ) ) {

            $messenger = new Es_Messenger( 'manage-property' );

            switch( $_GET['es-action'] ) {
                case 'copy':
                    array_walk( $_GET['es_properties'], function( $post_id ) use ($messenger) {
                        $clone = new Es_Post_Duplicate( $post_id );
                        $copied = $clone->makeClone();

                        if ( $copied instanceof WP_Error ) {
                            $messenger->set_message( $copied, 'error' );
                        } else {
                            $messenger->set_message(
                                sprintf( __( 'Post %s successfully copied.', 'es-plugin' ), $post_id ),
                                'success' );
                        }
                    } );
                    break;

                // Delete selected posts.
                case 'delete':
                    array_walk( $_GET['es_properties'], function( $post_id ) use ($messenger) {
                        $deleted = wp_delete_post( $post_id, true );

                        if ( ! $deleted ) {
                            $messenger->set_message(
                                sprintf( __( 'Can\'t remove property #%s.' ), $post_id ),
                                'error' );
                        } else {
                            $messenger->set_message(
                                sprintf( __( 'The property #%s successfully removed.', 'es-plugin' ), $post_id ),
                                'success' );
                        }
                    } );
                    break;

                // Publish selected posts.
                case 'publish':
                    if ( current_user_can( 'publish_es_properties' ) ) {
                        array_walk( $_GET['es_properties'], function( $post_id ) use ($messenger) {

                            wp_publish_post( $post_id );
                            $messenger->set_message(
                                sprintf( __( 'The property #%s successfully published.', 'es-plugin' ), $post_id ),
                                'success' );

                        } );
                    } else {
                        $messenger->set_message( __( 'You have no permission to publish properties.', 'es-plugin' ), 'error' );
                    }

                    break;

                // Unpublish selected posts.
                case 'unpublish':
                    array_walk( $_GET['es_properties'], function( $post_id ) use ( $messenger ) {
                        wp_update_post( array( 'ID' => $post_id, 'post_status' => 'draft' ) );
                        $messenger->set_message(
                            sprintf( __( 'The property #%s successfully unpublished.', 'es-plugin' ), $post_id ),
                            'success' );
                    } );
                    break;
            }

            wp_redirect( $_GET['_redirect'] ); die;
        }
    }
}
