<?php

/**
 * Class Es_Prop_Management_Shortcode.
 */
class Es_Prop_Management_Shortcode extends Es_Shortcode
{
    /**
     * @inheritdoc
     */
    public function build( $atts = array() )
    {
        $user = wp_get_current_user();

        // If user is not agent.
        if ( ! $user->has_cap( 'es_manage_properties' ) ) {
            $template = $this->get_template( 'no_access' );
        } else {

            global $post;
            $temp = $post;
            $post = ! empty( $_GET['id'] ) ? get_post( $_GET['id'] ) : null;

            $page = filter_input( INPUT_GET, 'es-page' );
            $messenger = new Es_Messenger( 'manage-property' );

            switch ( $page ) {
                case null:
                case 'properties':
                    $template = $this->get_template( 'properties' );
                    break;

                case 'new-property':
                    if ( current_user_can( 'create_es_properties' ) ) {
                        $template = $this->get_template( 'manage-property' );
                    } else {
                        $template = $this->get_template( 'no_access_2' );
                    }
                    break;
                case 'edit-property':
                    if ( $post && current_user_can( 'edit_es_properties' ) && ( $post->post_author == $user->ID
                            || current_user_can( 'edit_others_es_properties' ) ) ) {

                        $template = $this->get_template( 'manage-property' );
                    } else {
                        $template = $this->get_template( '404' );
                    }
                    break;

                default:
                    $template = $this->get_template( '404' );
            }

            $post = $temp;
        }

        ob_start();
        include $template;

        do_action( 'es_shortcode_after' );

        return ob_get_clean();
    }

    /**
     * @inheritdoc
     */
    public function get_shortcode_name()
    {
        return 'es_prop_management';
    }

    /**
     * Get Shortcode Template path by template type.
     *
     * @param $type
     * @return mixed
     */
    public function get_template( $type ) {
        return apply_filters( 'es_management_get_' . $type . '_template',
            ES_PLUGIN_PATH . ES_DS . 'management' . ES_DS . 'templates' . ES_DS . $type . '.php' );
    }

    /**
     * Return WP Query for property list table.
     *
     * @return WP_Query
     */
    public function get_query()
    {
        global $es_settings;

        if ( get_query_var( 'paged' ) ) {
            $page_num = get_query_var( 'paged' );
        } elseif ( get_query_var( 'page' ) ) {
            $page_num = get_query_var( 'page' );
        } else {
            $page_num = 1;
        }

        $property = es_get_property( null );

        $query_args = array(
            'post_type'           => $property::get_post_type_name(),
            'post_status'         => array( 'publish', 'pending', 'draft' ),
            'paged' => $page_num,
        );

        if ( $es_settings->properties_per_page ) {
            $query_args['posts_per_page'] = $es_settings->properties_per_page;
        }

        // Get filter data.
        $filter = filter_input( INPUT_GET, 'es_filter', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );

        // Flag for reset wp query.
        $flag = false;

        // Filter properties using post ID.
        if ( ! empty( $filter['property_id'] ) ) {
            $query_args['post__in'] = array( $filter['property_id'] );

        } elseif ( ! empty( $filter['address'] ) ) {
            // Create array from address string using delimiters.
            if ( $output = preg_split( "/[,\s]/", $filter['address'] ) ) {

                $ids = array();

                foreach ( $output as $key => $address_part ) {
                    if ( empty( $address_part ) ) continue;
                    $ids = array_merge( $ids, $property::find_by_address( $address_part ) );
                }

                if ( ! empty( $ids ) ) {
                    $query_args['post__in'] = $ids ;
                } else {
                    $flag = true;
                }
            }
        }

        // Filter properties by taxonomies.
        if ( ! empty( $filter['tax'] ) ) {
            foreach ( $filter['tax'] as $tax => $term_id ) {
                if ( empty( $term_id ) ) continue;

                if ( taxonomy_exists( $tax ) ) {
                    $taxFilter[] = array(
                        'taxonomy' => $tax,
                        'terms' => $term_id,
                    );
                }
            }
            if ( isset( $taxFilter ) ) {
                $query_args['tax_query'] = $taxFilter;
                $flag = false;
            }
        }

        // Reset wp query.
        if ( $flag ) {
            $query_args['post__in'] = array( -1 ) ;
        }

        $post_type_object = new WP_Post_Type( Es_Property::get_post_type_name() );

        if ( ! empty( $post_type_object->cap->edit_others_posts ) && ! current_user_can( $post_type_object->cap->edit_others_posts ) ) {
            global $user_ID;
            $query_args['author'] =  $user_ID;
        }

        return new WP_Query( $query_args );
    }

    /**
     * @return mixed
     */
    public function get_tabs() {
        return apply_filters( 'es_prop_management_tabs', array(
            'es-basic-info' => array( 'label' => __( 'Basic information', 'es-plugin' ) ),
            'es-address' =>    array( 'label' => __( 'Address', 'es-plugin' ) ),
            'es-featured' =>    array( 'label' => __( 'Features', 'es-plugin' ) ),
            'es-media' =>      array( 'label' => __( 'Media', 'es-plugin' ) ),
        ) );
    }
}
