<?php

/**
 * Class Es_List_Table.
 */
abstract class Es_List_Table
{
    /**
     * @var array
     */
    public $settings;

    /**
     * Es_List_Table constructor.
     *
     * @param $settings
     */
    public function __construct( $settings = array() )
    {
        $this->settings = wp_parse_args( $settings, array(
            'show_header' => true,
            'show_footer' => false,
        ) );
        add_action( 'es_list_table_value_col', array( $this, 'get_value' ), 10, 2 );
    }

    /**
     * @return array
     */
    abstract public function get_items();

    /**
     * Return columns array as id => label array where id can be the field of row item.
     *
     * @return array
     */
    abstract public function get_columns();

    /**
     * Render table method.
     *
     * @return void
     */
    public function render()
    {
        include $this->get_template_path();
    }

    /**
     * Return column value.
     *
     * @param $column
     * @param $item
     * @return null
     */
    public function get_value( $column, $item )
    {
        $value = null;

        if ( is_object( $item ) && ! empty( $item->{ $column } ) ) {
            $value = $item->{ $column };
        }

        if ( is_array( $item ) && ! empty( $item[ $column ] ) ) {
            $value = $item[ $column ];
        }

        echo $value;
    }

    /**
     * Return table template path.
     *
     * @return mixed
     */
    public function get_template_path()
    {
        return apply_filters( 'es_list_table_template_path', ES_PLUGIN_PATH . ES_DS . 'management' .
            ES_DS . 'templates' . ES_DS . 'table.php' );
    }
}
