<?php

/**
 * Class Es_Post_List_Table.
 */
class Es_Post_List_Table extends Es_List_Table
{
    /**
     * @var WP_Query
     */
    protected $_query;

    /**
     * Es_Post_List_Table constructor.
     *
     * @param WP_Query $wp_query
     * @param array $settings
     */
    public function __construct( $wp_query, $settings = array() )
    {
        $this->_query = $wp_query;
        parent::__construct( $settings );

        add_action( 'es_after_list_table', array( $this, 'pagination' ) );
    }

    /**
     * Return array of items.
     *
     * @return array
     */
    public function get_items()
    {
        return $this->_query->get_posts();
    }

    /**
     * Overridden render method.
     *
     * @return void
     */
    public function render()
    {
        global $wp_query; $temp = $wp_query;
        $wp_query = $this->_query;
        parent::render();
        $wp_query = $temp;
    }

    /**
     * Return columns array as id => label array where id can be the field of row item.
     *
     * @return array
     */
    public function get_columns()
    {
        return apply_filters( 'es_post_list_table_get_columns', array(
            'ID' => __( 'ID', 'es-plugin' ),
            'post_title' => __( 'Title', 'es-plugin' ),
        ), $this );
    }

    /**
     * Display pagination for property table.
     *
     * @return void
     */
    public function pagination() {
        the_posts_pagination( array(
            'prev_next' => false,
            'show_all'     => false,
            'end_size'     => 1,
            'mid_size'     => 1,
            'screen_reader_text' => ' ',
        ) );
    }
}
