<?php

/**
 * Class Es_Order
 *
 * @property integer $status
 * @property string $token
 * @property integer subscription_id
 */
class Es_Order extends Es_Post
{
    /**
     * Return entity custom fields
     *
     * @return array
     */
    public static function get_fields()
    {
        return apply_filters( 'es_order_fields', array(
            'token' => array(),
            'status' => array(),
            'subscription_id' => array(),
            'paypal_profile_id' => array(),
            'subscription_temp' => array(),
        ) );
    }

    /**
     * Return order subscription.
     *
     * @return Es_Subscription
     */
    public function get_subscription()
    {
        return ! empty( $this->subscription_id ) ? es_get_subscription( $this->subscription_id ) : null;
    }

    /**
     * @inheritdoc
     */
    public function get_entity_prefix()
    {
        return apply_filters( 'es_order_entity_prefix', 'es_order_' );
    }

    /**
     * Return order agent.
     *
     * @return Es_Agent|null
     */
    public function get_agent()
    {
        $entity = $this->get_entity();

        return ! empty( $entity->post_author ) ?
            es_get_agent( $entity->post_author ) : null;
    }

    /**
     * Create new order.
     *
     * @param Es_Subscription $subscription
     * @param WP_User $user
     * @param array $subscription_temp
     * @param $token
     * @param array $post_data
     * @param int|string $status
     * @return $this
     * @internal param int $status
     */
    public function create( Es_Subscription $subscription, $user, $subscription_temp = array(), $token, $post_data = array(), $status = 'Pending' ) {
        $subscription_post = $subscription->get_entity();
        $defaults = array(
            'post_author' => $user->ID,
            'post_title' => 'Order ' . $subscription_post->post_title . ' | ' . $user->user_email,
            'post_excerpt' => $subscription_post->post_excerpt,
            'post_status' => 'private',
            'post_type' => 'es_order',
        );

        $postarr = apply_filters( 'es_order_post_array', wp_parse_args( $post_data, $defaults ) );

        $post = wp_insert_post( $postarr );

        if ( $post && ! $post instanceof WP_Error ) {
            $this->_id = $post;

            $this->save_field_value( 'status', $status );
            $this->save_field_value( 'token', $token );
            $this->save_field_value( 'subscription_id', $subscription_post->ID );
            $this->save_field_value( 'subscription_temp', $subscription_temp );

            do_action( 'es_after_order_save', $post, $postarr, $subscription, $user, $token, $status );
        }

        return $this;
    }

    /**
     * Return order by token.
     *
     * @param $field
     * @param $value
     * @return Es_Order|null
     * @internal param $token
     */
    public function get_by_field( $field, $value) {
        $posts = get_posts( array(
            'meta_key' => $this->get_entity_prefix() . $field,
            'meta_value' => $value,
            'post_type' => 'es_order',
            'nopaging' => true,
            'posts_per_page' => 1,
            'post_status' => 'private',
        ) );

        return ! empty( $posts[0] ) ? es_get_order( $posts[0]->ID ) : null;
    }

    /**
     * Change order status.
     *
     * @param $new_status
     */
    public function change_status( $new_status ) {
        $this->save_field_value( 'status', $new_status );
    }

    /**
     * @return string
     */
    public function get_base_field_name()
    {
        return 'order';
    }
}
