<?php

class Es_Subscription_Table
{
    /**
     * Initialize subscription table.
     *
     * @return void
     */
    public static function init()
    {
        $_ = new self();
        $_->actions();

        add_shortcode( 'es_subscription_table', array( $_, 'render_table' ) );
    }

    /**
     * Initialize actions for subscription table.
     *
     * @return void
     */
    public function actions()
    {
        add_action( 'es_subscription_render_table_row_value', array( $this, 'row_value' ), 10, 3 );
        add_action( 'init', array( $this, 'proceed' ) );
    }

    /**
     * Render [es_subscription_table] shortcode.
     *
     * @return void
     */
    public function render_table()
    {
        if ( es_subscription_get_current_paypal_response_page() && es_subscription_get_payment_method_class()->valid_payment() ) {
            include apply_filters( 'es_subscription_table_template_path', ES_PLUGIN_PATH . '/subscriptions/templates/table.php' );
        } else {
            if ( current_user_can( 'publish_pages' ) ) {
                if ( ! es_subscription_get_payment_method_class()->valid_payment() ) {
                    echo '<span class="es-empty">' . __( 'Please, set PayPal credintials on Estatik subscription settings page.', 'es-plugin' ) . '</span>';
                } else {
                    echo '<span class="es-empty">' . __( 'Please, set PayPal success page on Estatik subscription settings page.', 'es-plugin' ) . '</span>';
                }
            } else {
                echo '<span class="es-empty">' . __( 'Nothing to display.', 'es-plugin' ) . '</span>';
            }
        }

        do_action( 'es_shortcode_after' );
    }

    /**
     * Return table header data.
     *
     * @return mixed
     */
    public static function get_header_rows_data()
    {
        return apply_filters( 'es_subscription_header_rows_data', array(
            'cb'                => '',
            'name'              => __( 'Name', 'es-plugin' ),
            'property_qty'      => __( 'All listing', 'es-plugin' ),
            'featured_qty'      => __( 'Featured listings', 'es-plugin' ),
//            'period'            => __( 'Period', 'es-plugin' ),
//            'trial_period'      => __( 'Trial period', 'es-plugin' ),
            'price'             => __( 'Price', 'es-plugin' ),
            'price_renewal'     => __( 'Renewal Price', 'es-plugin' ),
        ) );
    }

    /**
     * Render value for subscription table rows.
     *
     * @param $item
     * @param $id
     * @param $label
     *
     * @return void
     */
    public function row_value( $item, $id, $label )
    {
        $subscription = es_get_subscription( $item->ID );

        global $es_settings;

        $currency = $es_settings->get_label( 'currency', $subscription->currency );

        if ( 'name' == $id ) {
            $value = $item->post_title;
        } else if ( 'cb' == $id ) {
            $value = '<input type="radio" data-price="' . $currency . $subscription->price . '" name="es_subscription[id]" required value="' . $item->ID . '"/>';
        } else if ( 'price_renewal' == $id || 'price' == $id ) {
            $value = ! empty( $subscription->{$id} ) ? $currency . $subscription->{$id} : '-';
        } else {
            $value = $subscription->{$id};
        }

        echo apply_filters( 'es_subscription_table_row_value', $value, $item, $id, $label );
    }

    /**
     * Proceed subscription method.
     *
     * @return void
     */
    public function proceed()
    {
        if ( ! empty( $_POST['es_subscription_checkout'] ) && wp_verify_nonce( $_POST['es_subscription_checkout'], 'es_subscription_checkout' ) ) {
            $class = es_subscription_get_payment_method_class();
            if ( $class ) {
                $class->proceed();
            }
        }
    }
}
