<?php

/**
 * Class Es_Subscription
 *
 * @property int $property_qty
 * @property int $featured_qty
 * @property string $period
 * @property string $currency
 * @property string $trial_period
 * @property float $price
 * @property float $price_renewal
 */
class Es_Subscription extends Es_Post
{
    /**
     * Return custom subscription fields list.
     *
     * @return array
     */
    public static function get_fields()
    {
        global $es_settings;

        $periods = array();
        $periodsList = array_keys( es_get_subscription_periods() );


        if ( ! empty( $periodsList ) ) {
            foreach ( $periodsList as $item ) {
                $periods[ $item ] = __( $item, 'es-plugin' );
            }
        }

        return apply_filters( 'es_subscription_fields', array(

            'property_qty' => array(
                'type' => 'number',
                'options' => array(
                    'min' => 0,
                    'step' => 1,
                    'value' => 10,
                ),
                'label' => __( 'Property Qty', 'es-plugin' ),
            ),

            'featured_qty' => array(
                'type' => 'number',
                'options' => array(
                    'min' => 0,
                    'step' => 1,
                    'value' => 5,
                ),
                'label' => __( 'Featured Qty', 'es-plugin' ),
            ),

            'period' => array(
                'type' => 'list',
                'values' => $periods,
                'label' => __( 'Period', 'es-plugin' ),
            ),

            'currency' => array(
                'type' => 'list',
                'values' => $es_settings::get_setting_values( 'currency' ),
                'label' => __( 'Currency', 'es-plugin' ),
            ),

            'price' => array(
                'type' => 'number',
                'options' => array(
                    'min' => 0.5,
                    'step' => 0.5,
                    'required' => 'required'
                ),
                'label' => __( 'Price', 'es-plugin' ),
            ),

            'price_renewal' => array(
                'type' => 'number',
                'options' => array(
                    'min' => 0,
                ),
                'label' => __( 'Renewal price', 'es-plugin' ),
            ),

            'trial_period' => array(
                'type' => 'list',
                'values' => array_merge( array( __( '-- Select trial period --', 'es-plugin' ) ), $periods ),
                'label' => __( 'Trial', 'es-plugin' ),
            ),
        ) );
    }

    /**
     * @inheritdoc
     */
    public function get_entity_prefix()
    {
        return apply_filters( 'es_subscription_entity_prefix', 'es_subscription_' );
    }

    /**
     * Save post custom fields.
     *
     * @param $post_id
     * @param $post
     */
    public static function save( $post_id, $post )
    {
        $post_data = filter_input( INPUT_POST, 'subscription', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
        $subscription = es_get_subscription( $post_id );

        if ( !empty( $post_data ) && current_user_can( 'edit_post', $post_id ) && $post->post_type == 'es_subscription' ) {
            $fields = self::get_fields();

            foreach ( $fields as $key => $field ) {
                $value = isset( $post_data[ $key ] ) ? $post_data[ $key ] : null;
                $subscription->save_field_value( $key, $value );
            }
        }
    }

    /**
     * @return string
     */
    public function get_base_field_name()
    {
        return 'subscription';
    }
}
